// (C) 2001-2022 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

module eth_f_if_filter5 #(
    parameter SB_WIDTH = 8
) (
    input   logic                   reset,
    input   logic                   clk,
    input   logic   [0:3]           din,
    input   logic   [SB_WIDTH-1:0]  sb_din,     // sideband data aligned with filted inframe bits
    input   logic                   valid_in,
    output  logic   [0:3]           dout,
    output  logic   [SB_WIDTH-1:0]  sb_dout,
    output  logic                   valid_out
);

    logic   [0:3]   hist;

    logic   [0:3]   dout_reg;
    logic           valid_reg;
    logic           ready;

    logic   [SB_WIDTH-1:0]  sb_data_r1;
    logic   [SB_WIDTH-1:0]  sb_data_r2;

    always_ff @(posedge clk) begin
        if (reset) begin
            sb_data_r1  <= '0;
            sb_data_r2  <= '0;
        end else begin
            if (valid_in) begin
                sb_data_r1  <= sb_din;
                sb_data_r2  <= sb_data_r1;
            end else begin
                sb_data_r1  <= sb_data_r1;
                sb_data_r2  <= sb_data_r2;
            end
        end
        sb_dout <= sb_data_r2;
    end

    always_ff @(posedge clk) begin
        if (reset) begin
            valid_reg   <= 1'b0;
            valid_out   <= 1'b0;
            ready       <= 1'b0;
        end else begin
            valid_reg   <= valid_in && ready;
            valid_out   <= valid_reg;
            ready       <= ready | valid_in;
        end

        if (reset) begin
            hist <= 4'b0000;
        end else begin
            if (valid_in) begin
                hist            <= din[0:3];
            end else begin
                hist            <= hist;
            end
        end
	
		if(reset) begin
			dout_reg <= 4'b0000;
		end
        else if (valid_in) begin
            dout_reg        <= f({dout_reg[3], hist, din});
        end else begin
            dout_reg        <= dout_reg;
        end

        dout    <= dout_reg;
    end

    function logic [0:3] f;
        input   logic   [0:8]   din;
        case (din)
            9'b000000000 : f = 4'b0000;
            9'b000000001 : f = 4'b0000;
            9'b000000010 : f = 4'b0000;
            9'b000000011 : f = 4'b0000;
            9'b000000100 : f = 4'b0000;
            9'b000000101 : f = 4'b0000;
            9'b000000110 : f = 4'b0000;
            9'b000000111 : f = 4'b0000;
            9'b000001000 : f = 4'b0000;
            9'b000001001 : f = 4'b0000;
            9'b000001010 : f = 4'b0000;
            9'b000001011 : f = 4'b0000;
            9'b000001100 : f = 4'b0000;
            9'b000001101 : f = 4'b0000;
            9'b000001110 : f = 4'b0000;
            9'b000001111 : f = 4'b0000;
            9'b000010000 : f = 4'b0001;
            9'b000010001 : f = 4'b0000;
            9'b000010010 : f = 4'b0000;
            9'b000010011 : f = 4'b0000;
            9'b000010100 : f = 4'b0000;
            9'b000010101 : f = 4'b0000;
            9'b000010110 : f = 4'b0000;
            9'b000010111 : f = 4'b0000;
            9'b000011000 : f = 4'b0001;
            9'b000011001 : f = 4'b0000;
            9'b000011010 : f = 4'b0000;
            9'b000011011 : f = 4'b0000;
            9'b000011100 : f = 4'b0001;
            9'b000011101 : f = 4'b0000;
            9'b000011110 : f = 4'b0001;
            9'b000011111 : f = 4'b0001;
            9'b000100000 : f = 4'b0010;
            9'b000100001 : f = 4'b0010;
            9'b000100010 : f = 4'b0000;
            9'b000100011 : f = 4'b0000;
            9'b000100100 : f = 4'b0000;
            9'b000100101 : f = 4'b0000;
            9'b000100110 : f = 4'b0000;
            9'b000100111 : f = 4'b0000;
            9'b000101000 : f = 4'b0000;
            9'b000101001 : f = 4'b0000;
            9'b000101010 : f = 4'b0000;
            9'b000101011 : f = 4'b0000;
            9'b000101100 : f = 4'b0000;
            9'b000101101 : f = 4'b0000;
            9'b000101110 : f = 4'b0000;
            9'b000101111 : f = 4'b0000;
            9'b000110000 : f = 4'b0011;
            9'b000110001 : f = 4'b0011;
            9'b000110010 : f = 4'b0000;
            9'b000110011 : f = 4'b0000;
            9'b000110100 : f = 4'b0000;
            9'b000110101 : f = 4'b0000;
            9'b000110110 : f = 4'b0000;
            9'b000110111 : f = 4'b0000;
            9'b000111000 : f = 4'b0011;
            9'b000111001 : f = 4'b0011;
            9'b000111010 : f = 4'b0000;
            9'b000111011 : f = 4'b0000;
            9'b000111100 : f = 4'b0011;
            9'b000111101 : f = 4'b0011;
            9'b000111110 : f = 4'b0011;
            9'b000111111 : f = 4'b0011;
            9'b001000000 : f = 4'b0100;
            9'b001000001 : f = 4'b0100;
            9'b001000010 : f = 4'b0100;
            9'b001000011 : f = 4'b0100;
            9'b001000100 : f = 4'b0000;
            9'b001000101 : f = 4'b0000;
            9'b001000110 : f = 4'b0000;
            9'b001000111 : f = 4'b0000;
            9'b001001000 : f = 4'b0000;
            9'b001001001 : f = 4'b0000;
            9'b001001010 : f = 4'b0000;
            9'b001001011 : f = 4'b0000;
            9'b001001100 : f = 4'b0000;
            9'b001001101 : f = 4'b0000;
            9'b001001110 : f = 4'b0000;
            9'b001001111 : f = 4'b0000;
            9'b001010000 : f = 4'b0001;
            9'b001010001 : f = 4'b0000;
            9'b001010010 : f = 4'b0000;
            9'b001010011 : f = 4'b0000;
            9'b001010100 : f = 4'b0000;
            9'b001010101 : f = 4'b0000;
            9'b001010110 : f = 4'b0000;
            9'b001010111 : f = 4'b0000;
            9'b001011000 : f = 4'b0001;
            9'b001011001 : f = 4'b0000;
            9'b001011010 : f = 4'b0000;
            9'b001011011 : f = 4'b0000;
            9'b001011100 : f = 4'b0001;
            9'b001011101 : f = 4'b0000;
            9'b001011110 : f = 4'b0001;
            9'b001011111 : f = 4'b0001;
            9'b001100000 : f = 4'b0110;
            9'b001100001 : f = 4'b0110;
            9'b001100010 : f = 4'b0110;
            9'b001100011 : f = 4'b0110;
            9'b001100100 : f = 4'b0000;
            9'b001100101 : f = 4'b0000;
            9'b001100110 : f = 4'b0000;
            9'b001100111 : f = 4'b0000;
            9'b001101000 : f = 4'b0000;
            9'b001101001 : f = 4'b0000;
            9'b001101010 : f = 4'b0000;
            9'b001101011 : f = 4'b0000;
            9'b001101100 : f = 4'b0000;
            9'b001101101 : f = 4'b0000;
            9'b001101110 : f = 4'b0000;
            9'b001101111 : f = 4'b0000;
            9'b001110000 : f = 4'b0111;
            9'b001110001 : f = 4'b0111;
            9'b001110010 : f = 4'b0111;
            9'b001110011 : f = 4'b0111;
            9'b001110100 : f = 4'b0000;
            9'b001110101 : f = 4'b0000;
            9'b001110110 : f = 4'b0000;
            9'b001110111 : f = 4'b0000;
            9'b001111000 : f = 4'b0111;
            9'b001111001 : f = 4'b0111;
            9'b001111010 : f = 4'b0111;
            9'b001111011 : f = 4'b0111;
            9'b001111100 : f = 4'b0111;
            9'b001111101 : f = 4'b0111;
            9'b001111110 : f = 4'b0111;
            9'b001111111 : f = 4'b0111;
            9'b010000000 : f = 4'b1000;
            9'b010000001 : f = 4'b1000;
            9'b010000010 : f = 4'b1000;
            9'b010000011 : f = 4'b1000;
            9'b010000100 : f = 4'b1000;
            9'b010000101 : f = 4'b1000;
            9'b010000110 : f = 4'b1000;
            9'b010000111 : f = 4'b1000;
            9'b010001000 : f = 4'b0000;
            9'b010001001 : f = 4'b0000;
            9'b010001010 : f = 4'b0000;
            9'b010001011 : f = 4'b0000;
            9'b010001100 : f = 4'b0000;
            9'b010001101 : f = 4'b0000;
            9'b010001110 : f = 4'b0000;
            9'b010001111 : f = 4'b0000;
            9'b010010000 : f = 4'b0001;
            9'b010010001 : f = 4'b0000;
            9'b010010010 : f = 4'b0000;
            9'b010010011 : f = 4'b0000;
            9'b010010100 : f = 4'b0000;
            9'b010010101 : f = 4'b0000;
            9'b010010110 : f = 4'b0000;
            9'b010010111 : f = 4'b0000;
            9'b010011000 : f = 4'b0001;
            9'b010011001 : f = 4'b0000;
            9'b010011010 : f = 4'b0000;
            9'b010011011 : f = 4'b0000;
            9'b010011100 : f = 4'b0001;
            9'b010011101 : f = 4'b0000;
            9'b010011110 : f = 4'b0001;
            9'b010011111 : f = 4'b0001;
            9'b010100000 : f = 4'b0010;
            9'b010100001 : f = 4'b0010;
            9'b010100010 : f = 4'b0000;
            9'b010100011 : f = 4'b0000;
            9'b010100100 : f = 4'b0000;
            9'b010100101 : f = 4'b0000;
            9'b010100110 : f = 4'b0000;
            9'b010100111 : f = 4'b0000;
            9'b010101000 : f = 4'b0000;
            9'b010101001 : f = 4'b0000;
            9'b010101010 : f = 4'b0000;
            9'b010101011 : f = 4'b0000;
            9'b010101100 : f = 4'b0000;
            9'b010101101 : f = 4'b0000;
            9'b010101110 : f = 4'b0000;
            9'b010101111 : f = 4'b0000;
            9'b010110000 : f = 4'b0011;
            9'b010110001 : f = 4'b0011;
            9'b010110010 : f = 4'b0000;
            9'b010110011 : f = 4'b0000;
            9'b010110100 : f = 4'b0000;
            9'b010110101 : f = 4'b0000;
            9'b010110110 : f = 4'b0000;
            9'b010110111 : f = 4'b0000;
            9'b010111000 : f = 4'b0011;
            9'b010111001 : f = 4'b0011;
            9'b010111010 : f = 4'b0000;
            9'b010111011 : f = 4'b0000;
            9'b010111100 : f = 4'b0011;
            9'b010111101 : f = 4'b0011;
            9'b010111110 : f = 4'b0011;
            9'b010111111 : f = 4'b0011;
            9'b011000000 : f = 4'b1100;
            9'b011000001 : f = 4'b1100;
            9'b011000010 : f = 4'b1100;
            9'b011000011 : f = 4'b1100;
            9'b011000100 : f = 4'b1100;
            9'b011000101 : f = 4'b1100;
            9'b011000110 : f = 4'b1100;
            9'b011000111 : f = 4'b1100;
            9'b011001000 : f = 4'b0000;
            9'b011001001 : f = 4'b0000;
            9'b011001010 : f = 4'b0000;
            9'b011001011 : f = 4'b0000;
            9'b011001100 : f = 4'b0000;
            9'b011001101 : f = 4'b0000;
            9'b011001110 : f = 4'b0000;
            9'b011001111 : f = 4'b0000;
            9'b011010000 : f = 4'b0001;
            9'b011010001 : f = 4'b0000;
            9'b011010010 : f = 4'b0000;
            9'b011010011 : f = 4'b0000;
            9'b011010100 : f = 4'b0000;
            9'b011010101 : f = 4'b0000;
            9'b011010110 : f = 4'b0000;
            9'b011010111 : f = 4'b0000;
            9'b011011000 : f = 4'b0001;
            9'b011011001 : f = 4'b0000;
            9'b011011010 : f = 4'b0000;
            9'b011011011 : f = 4'b0000;
            9'b011011100 : f = 4'b0001;
            9'b011011101 : f = 4'b0000;
            9'b011011110 : f = 4'b0001;
            9'b011011111 : f = 4'b0001;
            9'b011100000 : f = 4'b1110;
            9'b011100001 : f = 4'b1110;
            9'b011100010 : f = 4'b1110;
            9'b011100011 : f = 4'b1110;
            9'b011100100 : f = 4'b1110;
            9'b011100101 : f = 4'b1110;
            9'b011100110 : f = 4'b1110;
            9'b011100111 : f = 4'b1110;
            9'b011101000 : f = 4'b0000;
            9'b011101001 : f = 4'b0000;
            9'b011101010 : f = 4'b0000;
            9'b011101011 : f = 4'b0000;
            9'b011101100 : f = 4'b0000;
            9'b011101101 : f = 4'b0000;
            9'b011101110 : f = 4'b0000;
            9'b011101111 : f = 4'b0000;
            9'b011110000 : f = 4'b1111;
            9'b011110001 : f = 4'b1111;
            9'b011110010 : f = 4'b1111;
            9'b011110011 : f = 4'b1111;
            9'b011110100 : f = 4'b1111;
            9'b011110101 : f = 4'b1111;
            9'b011110110 : f = 4'b1111;
            9'b011110111 : f = 4'b1111;
            9'b011111000 : f = 4'b1111;
            9'b011111001 : f = 4'b1111;
            9'b011111010 : f = 4'b1111;
            9'b011111011 : f = 4'b1111;
            9'b011111100 : f = 4'b1111;
            9'b011111101 : f = 4'b1111;
            9'b011111110 : f = 4'b1111;
            9'b011111111 : f = 4'b1111;
            9'b100000000 : f = 4'b0000;
            9'b100000001 : f = 4'b0000;
            9'b100000010 : f = 4'b0000;
            9'b100000011 : f = 4'b0000;
            9'b100000100 : f = 4'b0000;
            9'b100000101 : f = 4'b0000;
            9'b100000110 : f = 4'b0000;
            9'b100000111 : f = 4'b0000;
            9'b100001000 : f = 4'b0000;
            9'b100001001 : f = 4'b0000;
            9'b100001010 : f = 4'b0000;
            9'b100001011 : f = 4'b0000;
            9'b100001100 : f = 4'b0000;
            9'b100001101 : f = 4'b0000;
            9'b100001110 : f = 4'b0000;
            9'b100001111 : f = 4'b0000;
            9'b100010000 : f = 4'b0001;
            9'b100010001 : f = 4'b0000;
            9'b100010010 : f = 4'b0000;
            9'b100010011 : f = 4'b0000;
            9'b100010100 : f = 4'b0000;
            9'b100010101 : f = 4'b0000;
            9'b100010110 : f = 4'b0000;
            9'b100010111 : f = 4'b0000;
            9'b100011000 : f = 4'b0001;
            9'b100011001 : f = 4'b0000;
            9'b100011010 : f = 4'b0000;
            9'b100011011 : f = 4'b0000;
            9'b100011100 : f = 4'b0001;
            9'b100011101 : f = 4'b0000;
            9'b100011110 : f = 4'b0001;
            9'b100011111 : f = 4'b0001;
            9'b100100000 : f = 4'b0010;
            9'b100100001 : f = 4'b0010;
            9'b100100010 : f = 4'b0000;
            9'b100100011 : f = 4'b0000;
            9'b100100100 : f = 4'b0000;
            9'b100100101 : f = 4'b0000;
            9'b100100110 : f = 4'b0000;
            9'b100100111 : f = 4'b0000;
            9'b100101000 : f = 4'b0000;
            9'b100101001 : f = 4'b0000;
            9'b100101010 : f = 4'b0000;
            9'b100101011 : f = 4'b0000;
            9'b100101100 : f = 4'b0000;
            9'b100101101 : f = 4'b0000;
            9'b100101110 : f = 4'b0000;
            9'b100101111 : f = 4'b0000;
            9'b100110000 : f = 4'b0011;
            9'b100110001 : f = 4'b0011;
            9'b100110010 : f = 4'b0000;
            9'b100110011 : f = 4'b0000;
            9'b100110100 : f = 4'b0000;
            9'b100110101 : f = 4'b0000;
            9'b100110110 : f = 4'b0000;
            9'b100110111 : f = 4'b0000;
            9'b100111000 : f = 4'b0011;
            9'b100111001 : f = 4'b0011;
            9'b100111010 : f = 4'b0000;
            9'b100111011 : f = 4'b0000;
            9'b100111100 : f = 4'b0011;
            9'b100111101 : f = 4'b0011;
            9'b100111110 : f = 4'b0011;
            9'b100111111 : f = 4'b0011;
            9'b101000000 : f = 4'b0100;
            9'b101000001 : f = 4'b0100;
            9'b101000010 : f = 4'b0100;
            9'b101000011 : f = 4'b0100;
            9'b101000100 : f = 4'b0000;
            9'b101000101 : f = 4'b0000;
            9'b101000110 : f = 4'b0000;
            9'b101000111 : f = 4'b0000;
            9'b101001000 : f = 4'b0000;
            9'b101001001 : f = 4'b0000;
            9'b101001010 : f = 4'b0000;
            9'b101001011 : f = 4'b0000;
            9'b101001100 : f = 4'b0000;
            9'b101001101 : f = 4'b0000;
            9'b101001110 : f = 4'b0000;
            9'b101001111 : f = 4'b0000;
            9'b101010000 : f = 4'b0001;
            9'b101010001 : f = 4'b0000;
            9'b101010010 : f = 4'b0000;
            9'b101010011 : f = 4'b0000;
            9'b101010100 : f = 4'b0000;
            9'b101010101 : f = 4'b0000;
            9'b101010110 : f = 4'b0000;
            9'b101010111 : f = 4'b0000;
            9'b101011000 : f = 4'b0001;
            9'b101011001 : f = 4'b0000;
            9'b101011010 : f = 4'b0000;
            9'b101011011 : f = 4'b0000;
            9'b101011100 : f = 4'b0001;
            9'b101011101 : f = 4'b0000;
            9'b101011110 : f = 4'b0001;
            9'b101011111 : f = 4'b0001;
            9'b101100000 : f = 4'b0110;
            9'b101100001 : f = 4'b0110;
            9'b101100010 : f = 4'b0110;
            9'b101100011 : f = 4'b0110;
            9'b101100100 : f = 4'b0000;
            9'b101100101 : f = 4'b0000;
            9'b101100110 : f = 4'b0000;
            9'b101100111 : f = 4'b0000;
            9'b101101000 : f = 4'b0000;
            9'b101101001 : f = 4'b0000;
            9'b101101010 : f = 4'b0000;
            9'b101101011 : f = 4'b0000;
            9'b101101100 : f = 4'b0000;
            9'b101101101 : f = 4'b0000;
            9'b101101110 : f = 4'b0000;
            9'b101101111 : f = 4'b0000;
            9'b101110000 : f = 4'b0111;
            9'b101110001 : f = 4'b0111;
            9'b101110010 : f = 4'b0111;
            9'b101110011 : f = 4'b0111;
            9'b101110100 : f = 4'b0000;
            9'b101110101 : f = 4'b0000;
            9'b101110110 : f = 4'b0000;
            9'b101110111 : f = 4'b0000;
            9'b101111000 : f = 4'b0111;
            9'b101111001 : f = 4'b0111;
            9'b101111010 : f = 4'b0111;
            9'b101111011 : f = 4'b0111;
            9'b101111100 : f = 4'b0111;
            9'b101111101 : f = 4'b0111;
            9'b101111110 : f = 4'b0111;
            9'b101111111 : f = 4'b0111;
            9'b110000000 : f = 4'b1000;
            9'b110000001 : f = 4'b1000;
            9'b110000010 : f = 4'b1000;
            9'b110000011 : f = 4'b1000;
            9'b110000100 : f = 4'b1000;
            9'b110000101 : f = 4'b1000;
            9'b110000110 : f = 4'b1000;
            9'b110000111 : f = 4'b1000;
            9'b110001000 : f = 4'b1000;
            9'b110001001 : f = 4'b1000;
            9'b110001010 : f = 4'b1000;
            9'b110001011 : f = 4'b1000;
            9'b110001100 : f = 4'b1000;
            9'b110001101 : f = 4'b1000;
            9'b110001110 : f = 4'b1000;
            9'b110001111 : f = 4'b1000;
            9'b110010000 : f = 4'b1001;
            9'b110010001 : f = 4'b1000;
            9'b110010010 : f = 4'b1000;
            9'b110010011 : f = 4'b1000;
            9'b110010100 : f = 4'b1000;
            9'b110010101 : f = 4'b1000;
            9'b110010110 : f = 4'b1000;
            9'b110010111 : f = 4'b1000;
            9'b110011000 : f = 4'b1001;
            9'b110011001 : f = 4'b1000;
            9'b110011010 : f = 4'b1000;
            9'b110011011 : f = 4'b1000;
            9'b110011100 : f = 4'b1001;
            9'b110011101 : f = 4'b1000;
            9'b110011110 : f = 4'b1001;
            9'b110011111 : f = 4'b1001;
            9'b110100000 : f = 4'b1010;
            9'b110100001 : f = 4'b1010;
            9'b110100010 : f = 4'b1000;
            9'b110100011 : f = 4'b1000;
            9'b110100100 : f = 4'b1000;
            9'b110100101 : f = 4'b1000;
            9'b110100110 : f = 4'b1000;
            9'b110100111 : f = 4'b1000;
            9'b110101000 : f = 4'b1000;
            9'b110101001 : f = 4'b1000;
            9'b110101010 : f = 4'b1000;
            9'b110101011 : f = 4'b1000;
            9'b110101100 : f = 4'b1000;
            9'b110101101 : f = 4'b1000;
            9'b110101110 : f = 4'b1000;
            9'b110101111 : f = 4'b1000;
            9'b110110000 : f = 4'b1011;
            9'b110110001 : f = 4'b1011;
            9'b110110010 : f = 4'b1000;
            9'b110110011 : f = 4'b1000;
            9'b110110100 : f = 4'b1000;
            9'b110110101 : f = 4'b1000;
            9'b110110110 : f = 4'b1000;
            9'b110110111 : f = 4'b1000;
            9'b110111000 : f = 4'b1011;
            9'b110111001 : f = 4'b1011;
            9'b110111010 : f = 4'b1000;
            9'b110111011 : f = 4'b1000;
            9'b110111100 : f = 4'b1011;
            9'b110111101 : f = 4'b1011;
            9'b110111110 : f = 4'b1011;
            9'b110111111 : f = 4'b1011;
            9'b111000000 : f = 4'b1100;
            9'b111000001 : f = 4'b1100;
            9'b111000010 : f = 4'b1100;
            9'b111000011 : f = 4'b1100;
            9'b111000100 : f = 4'b1100;
            9'b111000101 : f = 4'b1100;
            9'b111000110 : f = 4'b1100;
            9'b111000111 : f = 4'b1100;
            9'b111001000 : f = 4'b1100;
            9'b111001001 : f = 4'b1100;
            9'b111001010 : f = 4'b1100;
            9'b111001011 : f = 4'b1100;
            9'b111001100 : f = 4'b1100;
            9'b111001101 : f = 4'b1100;
            9'b111001110 : f = 4'b1100;
            9'b111001111 : f = 4'b1100;
            9'b111010000 : f = 4'b1101;
            9'b111010001 : f = 4'b1100;
            9'b111010010 : f = 4'b1100;
            9'b111010011 : f = 4'b1100;
            9'b111010100 : f = 4'b1100;
            9'b111010101 : f = 4'b1100;
            9'b111010110 : f = 4'b1100;
            9'b111010111 : f = 4'b1100;
            9'b111011000 : f = 4'b1101;
            9'b111011001 : f = 4'b1100;
            9'b111011010 : f = 4'b1100;
            9'b111011011 : f = 4'b1100;
            9'b111011100 : f = 4'b1101;
            9'b111011101 : f = 4'b1100;
            9'b111011110 : f = 4'b1101;
            9'b111011111 : f = 4'b1101;
            9'b111100000 : f = 4'b1110;
            9'b111100001 : f = 4'b1110;
            9'b111100010 : f = 4'b1110;
            9'b111100011 : f = 4'b1110;
            9'b111100100 : f = 4'b1110;
            9'b111100101 : f = 4'b1110;
            9'b111100110 : f = 4'b1110;
            9'b111100111 : f = 4'b1110;
            9'b111101000 : f = 4'b1110;
            9'b111101001 : f = 4'b1110;
            9'b111101010 : f = 4'b1110;
            9'b111101011 : f = 4'b1110;
            9'b111101100 : f = 4'b1110;
            9'b111101101 : f = 4'b1110;
            9'b111101110 : f = 4'b1110;
            9'b111101111 : f = 4'b1110;
            9'b111110000 : f = 4'b1111;
            9'b111110001 : f = 4'b1111;
            9'b111110010 : f = 4'b1111;
            9'b111110011 : f = 4'b1111;
            9'b111110100 : f = 4'b1111;
            9'b111110101 : f = 4'b1111;
            9'b111110110 : f = 4'b1111;
            9'b111110111 : f = 4'b1111;
            9'b111111000 : f = 4'b1111;
            9'b111111001 : f = 4'b1111;
            9'b111111010 : f = 4'b1111;
            9'b111111011 : f = 4'b1111;
            9'b111111100 : f = 4'b1111;
            9'b111111101 : f = 4'b1111;
            9'b111111110 : f = 4'b1111;
            9'b111111111 : f = 4'b1111;
            default      : f = 4'b0000;
        endcase
    endfunction
endmodule
