// (C) 2001-2022 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


module eth_f_undersized_frame_csr #(
    parameter PREAMBLE_PASSTHROUGH  = 0,
    parameter CRC_PASSTHROUGH       = 0,
    parameter MIN_FRAME_SIZE        = 16'd64
) (
    input   logic           clk,
    input   logic           reset,

    input   logic   [31:0]  avmm_addr,
    input   logic   [31:0]  avmm_data,
    input   logic           avmm_write,

    output  logic   [15:0]  min_mac_octets
);

    logic           crc_pass_en;
    logic           pream_pass_en;

    logic   [15:0]  crc_offset;
    logic   [15:0]  pream_offset;

    assign crc_offset   = crc_pass_en   ? 16'd0 : -16'd4;
    assign pream_offset = pream_pass_en ? 16'd8 :  16'd0;

    always_ff @(posedge clk) begin
        min_mac_octets   <= 16'(MIN_FRAME_SIZE[15:0] + crc_offset + pream_offset);
    end

    always_ff @(posedge clk) begin
        if (reset) begin
            crc_pass_en     <= CRC_PASSTHROUGH;
            pream_pass_en   <= PREAMBLE_PASSTHROUGH;
        end else begin
            crc_pass_en     <= crc_pass_en;
            pream_pass_en   <= pream_pass_en;
            if (avmm_write) begin
                case (avmm_addr) inside
                    32'h507 : crc_pass_en   <= avmm_data[0];
                    32'h50B : pream_pass_en <= avmm_data[0];
                endcase
            end
        end
    end
endmodule
