// (C) 2001-2022 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.




module eth_f_csr   #(
    parameter LANE_NUM          = 1,
    parameter ENABLE_PTP        = 0
  ) (
    input wire                          i_reconfig_clk, 
    input wire                          i_reconfig_reset,
    input wire [13:0]                   i_reconfig_eth_addr,
    input wire                          i_reconfig_eth_read,
    input wire                          i_reconfig_eth_write,
    input wire [31:0]                   i_reconfig_eth_writedata,
    input wire [3:0]                    i_reconfig_eth_byteenable,
    input wire [2:0]                    eth_rate, // [2:0]
    input wire                          anlt_enable, // [0]
    input wire                          modulation_type, // [0]
    input wire [2:0]                    rsfec_type, // [2:0]
    input wire [2:0]                    flow_control_mode, // [2:0]
    input wire [2:0]                    client_intf, // [2:0]
    input wire                          xcvr_type, // [0]
    input wire [3:0]                    num_lanes, // [3:0]
      // DR
    input wire                          i_tx_ehip_preamble_passthru_gui,
    output wire                         o_tx_ehip_preamble_passthru_dr,
    // PTP begin
    input wire                          i_ptp_clk,
    input wire                          i_tx_tod_clk,
    input wire                          i_rx_tod_clk,
    input wire                          i_samp_clk,
    input wire                          i_tx_srst_n,
    input wire                          i_rx_srst_n,
    input wire                          i_tx_tod_srst_n,
    input wire                          i_rx_tod_srst_n,
    input wire                          i_tx_samp_srst_n,
    input wire                          i_rx_samp_srst_n,
    input wire                          i_tx_apulse_wdly_valid       ,
    input wire                          i_tx_apulse_offset_valid     ,
    input wire                          i_tx_apulse_time_valid       ,
    input wire                          i_rx_apulse_wdly_valid       ,
    input wire                          i_rx_apulse_offset_valid     ,
    input wire                          i_rx_apulse_time_valid       ,
    input wire [LANE_NUM-1:0][19:0]     i_tx_apulse_wdly             ,
    input wire [LANE_NUM-1:0][31:0]     i_tx_apulse_offset           ,
    input wire [LANE_NUM-1:0][27:0]     i_tx_apulse_time             ,
    input wire [LANE_NUM-1:0][19:0]     i_rx_apulse_wdly             ,
    input wire [LANE_NUM-1:0][31:0]     i_rx_apulse_offset           ,
    input wire [LANE_NUM-1:0][27:0]     i_rx_apulse_time             ,
    input wire               [31:0]     i_tx_const_adjust            ,
    input wire               [31:0]     i_rx_const_adjust            ,
    input wire                          i_rx_pcs_fully_aligned       ,
    input wire                          i_rx_ptp_vl_snapshot         ,
    input wire                          i_tx_tam_valid               ,
    input wire [47:0]                   i_tx_tam_ui                  ,
    input wire [15:0]                   i_tx_tam_cnt                 ,
    input wire                          i_rx_tam_valid               ,
    input wire [47:0]                   i_rx_tam_ui                  ,
    input wire [15:0]                   i_rx_tam_cnt                 ,
    input wire                          i_tx_ptp_offset_data_valid   ,
    input wire                          i_rx_ptp_offset_data_valid   ,
    input wire                          i_tx_ptp_ready               ,
    input wire                          i_rx_ptp_ready               ,
    output wire               [2:0]     o_tx_ref_lane                ,
    output wire               [2:0]     o_rx_lal                     ,
    output wire              [31:0]     o_tx_calc_adjust             ,
    output wire              [31:0]     o_rx_calc_adjust             ,
    output wire                         o_rx_fec_cw_pos_user_cfg_done,
    output wire                         o_tx_ptp_user_cfg_done       ,
    output wire                         o_rx_ptp_user_cfg_done       ,
    input wire                          i_tx_ptp_user_cfg_done_clrn  ,
    input wire                          i_rx_ptp_user_cfg_done_clrn  ,
    // PTP end
    output reg [31:0]                   o_reconfig_eth_readdata,
    output reg                          o_reconfig_eth_readdata_valid,
    output reg                          o_reconfig_eth_waitrequest,
    input wire                          tx_rst_ack_n, // Acknowledge signal for i_tx_rst_n.
    input wire                          rx_rst_ack_n, // Acknowledge signal for i_rx_rst_n.
    input wire                          rst_ack_n, // Acknowledge signal for i_rst_n.
    input wire                          tx_lanes_stable_status,
    input wire                          ehip_remote_fault_status,
    input wire                          ehip_local_fault_status,
    input wire [15:0]                   rxaib_transfer_ready_ehip,
    input wire [1:0]                    rxaib_transfer_ready_ptp,
    input wire [15:0]                   txaib_transfer_ready_ehip,
    input wire [31:0]                   clk_tx_khz,
    input wire [31:0]                   clk_rx_khz,
    input wire [31:0]                   clk_pll_khz,
    input wire [31:0]                   clk_tx_div_khz,
    input wire [31:0]                   clk_rec_div64_khz,
    input wire [31:0]                   clk_rec_div_khz,
    input wire [63:0]                   dropped_frame_cnt,
    output wire                         dropped_clear,
    output wire                         dropped_frame_snapshot,
    output wire                         eth_reset_eio_sys_rst,
    output wire                         eth_reset_soft_tx_rst,
    output wire                         eth_reset_soft_rx_rst,

    output reg  [7:0]                   eth_reset_tx_clear_alarm,
    output reg  [7:0]                   eth_reset_rx_clear_alarm,
    input  wire [2:0]                   eth_reset_status_tx_lane_current_state_i,
    input  wire [2:0]                   eth_reset_status_rx_lane_current_state_i,
    input  wire [7:0]                   eth_reset_status_tx_alarm_i,
    input  wire [7:0]                   eth_reset_status_rx_alarm_i,
    input  wire [7:0]                   phy_tx_pll_locked_tx_pll_locked_i,
    input  wire [7:0]                   phy_eiofreq_locked_eio_freq_lock_i,
    input  wire                         pcs_status_dskew_status_i,
    input  wire                         pcs_status_dskew_chng_i,
    input  wire                         pcs_status_rx_pcs_ready_i,
    input  wire                         pcs_status_kr_mode_i,
    input  wire                         pcs_status_kr_fec_mode_i,
    output reg                          pcs_control_clr_dskew_chng,
	 
	 output 										reset_swcwbin,
	 input  [31:0] 							cwbin0_stat_block_A_i,
	 input  [31:0] 							cwbin1_stat_block_A_i,
	 input  [31:0] 							cwbin2_stat_block_A_i,
	 input  [31:0] 							cwbin3_stat_block_A_i,
	 input  [31:0] 							cwbin0_stat_block_B_i,
	 input  [31:0] 							cwbin1_stat_block_B_i,
	 input  [31:0] 							cwbin2_stat_block_B_i,
	 input  [31:0] 							cwbin3_stat_block_B_i,
	 output [31:0] 								cwbin_timer_timeout,

//  for AVMM1 bb ports
    input   wire                        pld_avmm1_busy,
    output  wire                        pld_avmm1_clk_rowclk,
    input   wire                        pld_avmm1_cmdfifo_wr_full,
    input   wire                        pld_avmm1_cmdfifo_wr_pfull,
    output  wire                        pld_avmm1_read,
    input   wire  [7:0]                 pld_avmm1_readdata,
    input   wire                        pld_avmm1_readdatavalid,
    output  wire  [9:0]                 pld_avmm1_reg_addr,
    output  wire                        pld_avmm1_request,
    output  wire  [8:0]                 pld_avmm1_reserved_in,
    input   wire  [2:0]                 pld_avmm1_reserved_out,
    output  wire                        pld_avmm1_write,
    output  wire  [7:0]                 pld_avmm1_writedata,
    input   wire                        pld_chnl_cal_done,
    input   wire                        pld_hssi_osc_transfer_en
);

  wire [17:0] avmm_m8_addr;
  wire [7:0]  avmm_m8_wdata;
  wire [7:0]  avmm_m8_readdata;

  logic soft_csr_base_sel, r_soft_csr_base_sel, soft_csr_base_sel_a;   // Base registers implemented in soft logic
  logic soft_csr_ptp_sel, r_soft_csr_ptp_sel, soft_csr_ptp_sel_a;   // PTP registers implemented in soft logic
  logic ehip_sel;           // Hard EHIP regisers
  logic is_maib;            // MAIB adapter registers

  always_comb begin
      soft_csr_base_sel   = 1'b0;
      soft_csr_ptp_sel    = 1'b0;
      ehip_sel            = 1'b0;
      case ({i_reconfig_eth_addr[13:0],2'b00}) inside
          [16'h0100:16'h0420] : soft_csr_base_sel  = 1'b1;
          [16'h0800:16'h0aff] : soft_csr_ptp_sel   = 1'b1;
          default             : ehip_sel           = 1'b1;
      endcase
  end 

  always_comb begin
      case ({i_reconfig_eth_addr[13:0],2'b00}) inside
          [16'h0000:16'h00ff] : is_maib           = 1'b0;
          [16'h0b00:16'h0fff] : is_maib           = 1'b1;
          default             : is_maib           = 1'b0;
      endcase
  end 

 

  //Read Write
  
  wire soft_csr_base_read  = i_reconfig_eth_read && soft_csr_base_sel;
  wire soft_csr_ptp_read   = i_reconfig_eth_read && soft_csr_ptp_sel;
  wire ehip_read           = i_reconfig_eth_read && ehip_sel;
  
  wire soft_csr_base_write = i_reconfig_eth_write && soft_csr_base_sel;
  wire soft_csr_ptp_write  = i_reconfig_eth_write && soft_csr_ptp_sel;
  wire ehip_write          = i_reconfig_eth_write && ehip_sel;
  
  // Wait
  reg r_reconfig_eth_read;
  logic sip_waitrequest;
  wire ehip_waitrequest;

  always @(posedge i_reconfig_clk)
    if (i_reconfig_reset) begin
		r_reconfig_eth_read <= 1'b0;
		r_soft_csr_base_sel <= 1'b0;
		r_soft_csr_ptp_sel  <= 1'b0;
    end
    else begin
		r_reconfig_eth_read <= i_reconfig_eth_read;
                if (i_reconfig_eth_read || i_reconfig_eth_write) begin
		   r_soft_csr_base_sel <= soft_csr_base_sel;
		   r_soft_csr_ptp_sel  <= soft_csr_ptp_sel;
                end
    end

  assign soft_csr_base_sel_a = soft_csr_base_sel | r_soft_csr_base_sel;
  assign soft_csr_ptp_sel_a  = soft_csr_ptp_sel  | r_soft_csr_ptp_sel;

  always_comb begin
      case (1'b1)
          soft_csr_base_sel_a   : o_reconfig_eth_waitrequest = sip_waitrequest;
          soft_csr_ptp_sel_a    : o_reconfig_eth_waitrequest = sip_waitrequest;
          default               : o_reconfig_eth_waitrequest = ehip_waitrequest;
      endcase
  end // always_comb
  
  // Read data
  wire [31:0]  ehip_readdata;
  wire [31:0]  soft_csr_base_readdata;
  wire [31:0]  soft_csr_ptp_readdata;
  logic [31:0] readdata;
  wire         ehip_readdata_valid;
  wire         soft_csr_base_readdata_valid;
  logic        soft_csr_ptp_readdata_valid;
  logic        sip_readdata_valid;

  always_comb begin
      case (1'b1)
          r_soft_csr_base_sel : o_reconfig_eth_readdata = soft_csr_base_readdata;
          r_soft_csr_ptp_sel  : o_reconfig_eth_readdata = soft_csr_ptp_readdata;
          default             : o_reconfig_eth_readdata = ehip_readdata;
      endcase
  end

  always_comb begin
      case (1'b1)
          r_soft_csr_base_sel : o_reconfig_eth_readdata_valid = sip_readdata_valid;
          r_soft_csr_ptp_sel  : o_reconfig_eth_readdata_valid = sip_readdata_valid;
          default             : o_reconfig_eth_readdata_valid = ehip_readdata_valid;
      endcase
  end

  always_ff @(posedge i_reconfig_clk) begin
    // Registered because valid must come at least one cycle after wait goes low
    sip_readdata_valid <= !sip_waitrequest && i_reconfig_eth_read;
  end


  
//----------------------------------
reg	write_d1;
reg     read_d1;
reg	waitreq_wr, waitreq_rd;
reg	valid_wreq_d1;
reg	valid_rreq_d1;

assign  valid_rreq = (i_reconfig_eth_read & ~read_d1);
wire    valid_wreq = (i_reconfig_eth_write & ~write_d1);

always @(posedge i_reconfig_clk) begin
  write_d1      <=  i_reconfig_eth_write; 
  read_d1       <=  i_reconfig_eth_read; 
  valid_wreq_d1 <= valid_wreq;
  valid_rreq_d1 <= valid_rreq;
end

always @(posedge i_reconfig_clk) begin
  if (i_reconfig_reset)		waitreq_wr <= 1'b0;
  else if (valid_wreq)		waitreq_wr <= 1'b1;
  else if (valid_wreq_d1)	waitreq_wr <= 1'b0;
  if (i_reconfig_reset)		waitreq_rd <= 1'b0;
  else if (valid_rreq)		waitreq_rd <= 1'b1;
  else if (valid_rreq_d1)	waitreq_rd <= 1'b0;
end

assign  sip_waitrequest = i_reconfig_reset | (valid_wreq | waitreq_wr) | (valid_rreq | waitreq_rd);

  // 32to8 
  wire [31:0] ehip_sladdress;
  wire        ehip_slread;
  wire        ehip_slwrite;
  wire [7:0]  ehip_slwrite_data;
  wire [7:0]  ehip_slread_data;
  wire        ehip_slwait_request;


  wire [1:0]  device_name;
  wire [2:0]  tile_name;

  eth_f_base_sip_csr soft_csr (
     // .gui_option_device_name                                      (device_name[1:0]), // 0:Stratix 10, 1:Falcon Mesa
     // .gui_option_tile_name                                        (tile_name[2:0]), // 0:H-tile,1:L-tile,2:E-tile,3:F-tile, 4-7:reserved
      .gui_option_eth_rate_i                                       (eth_rate[2:0]),
      .gui_option_anlt_enable_i                                    (anlt_enable),
      .gui_option_modulation_type_i                                (modulation_type),
      .gui_option_rsfec_type_i                                     (rsfec_type[2:0]),
      .gui_option_ptp_enable_i                                     (ENABLE_PTP ? 1'b1: 1'b0),
      .gui_option_flow_control_mode_i                              (flow_control_mode[2:0]),
      .gui_option_client_intf_i                                    (client_intf[2:0]),
      .gui_option_xcvr_type_i                                      (xcvr_type),
      .gui_option_num_lanes_i                                      (num_lanes[3:0]),
      .eth_reset_eio_sys_rst                                       (eth_reset_eio_sys_rst),
      .eth_reset_soft_tx_rst                                       (eth_reset_soft_tx_rst),
      .eth_reset_soft_rx_rst                                       (eth_reset_soft_rx_rst),
      .eth_reset_tx_clear_alarm(eth_reset_tx_clear_alarm[7:0]),
      .eth_reset_rx_clear_alarm(eth_reset_rx_clear_alarm[7:0]),
      .eth_reset_status_rst_ack_n_i                                (rst_ack_n),
      .eth_reset_status_tx_rst_ack_n_i                             (tx_rst_ack_n),
      .eth_reset_status_rx_rst_ack_n_i                             (rx_rst_ack_n),
      .eth_reset_status_tx_lane_current_state_i(eth_reset_status_tx_lane_current_state_i[2:0]),
      .eth_reset_status_rx_lane_current_state_i(eth_reset_status_rx_lane_current_state_i[2:0]),
      .eth_reset_status_tx_alarm_i(eth_reset_status_tx_alarm_i[7:0]),
      .eth_reset_status_rx_alarm_i(eth_reset_status_rx_alarm_i[7:0]),
      .phy_tx_pll_locked_tx_pll_locked_i(phy_tx_pll_locked_tx_pll_locked_i[7:0]),
      .phy_eiofreq_locked_eio_freq_lock_i(phy_eiofreq_locked_eio_freq_lock_i[7:0]),
      .pcs_status_dskew_status_i(pcs_status_dskew_status_i),
      .pcs_status_dskew_chng_i(pcs_status_dskew_chng_i),
      .pcs_status_tx_lanes_stable_i(tx_lanes_stable_status), 
      .pcs_status_rx_pcs_ready_i(pcs_status_rx_pcs_ready_i),
      .pcs_status_kr_mode_i(pcs_status_kr_mode_i),
      .pcs_status_kr_fec_mode_i(pcs_status_kr_fec_mode_i),
      .pcs_control_clr_dskew_chng(pcs_control_clr_dskew_chng),
      .link_fault_status_lfault_i(ehip_local_fault_status),
      .link_fault_status_rfault_i(ehip_remote_fault_status),
      .aib_transfer_ready_status_ehip_rx_transfer_ready_i(rxaib_transfer_ready_ehip),
      .aib_transfer_ready_status_ehip_tx_transfer_ready_i(txaib_transfer_ready_ehip),
      .clk_tx_khz_clk_tx_khz_i                                     (clk_tx_khz),
      .clk_rx_khz_clk_rx_khz_i                                     (clk_rx_khz),
      .clk_pll_khz_clk_pll_khz_i                                   (clk_pll_khz),
      .clk_tx_div_khz_clk_tx_div_khz_i                             (clk_tx_div_khz),
      .clk_rec_div64_khz_clk_rec_div64_khz_i                       (clk_rec_div64_khz),
      .clk_rec_div_khz_clk_rec_div_khz_i                           (clk_rec_div_khz),
      .rxmac_adapt_dropped_31_0_rxmac_adapt_dropped_31_0_i         (dropped_frame_cnt[31:0]),
      .rxmac_adapt_dropped_63_32_rxmac_adapt_dropped_63_32_i       (dropped_frame_cnt[63:32]),
      .rxmac_adapt_dropped_control_rxmac_adapt_dropped_clear       (dropped_clear),
      .rxmac_adapt_dropped_control_rxmac_adapt_dropped_snapshot    (dropped_frame_snapshot),
      .cwbin_control_register_cwbin_control_register					 (reset_swcwbin),
	.cwbin0_A_cwbin0_A_i					  (cwbin0_stat_block_A_i),
	.cwbin1_A_cwbin1_A_i					  (cwbin1_stat_block_A_i),
	.cwbin2_A_cwbin2_A_i					  (cwbin2_stat_block_A_i),
	.cwbin3_A_cwbin3_A_i					  (cwbin3_stat_block_A_i),
	.cwbin0_B_cwbin0_B_i					  (cwbin0_stat_block_B_i),
	.cwbin1_B_cwbin1_B_i					  (cwbin1_stat_block_B_i),
	.cwbin2_B_cwbin2_B_i					  (cwbin2_stat_block_B_i),
	.cwbin3_B_cwbin3_B_i					  (cwbin3_stat_block_B_i),
        .cwbin_timer_cwbin_timer				(cwbin_timer_timeout),
      .clk                                                         (i_reconfig_clk),
      .reset                                                       (i_reconfig_reset),
      .writedata                                                   (i_reconfig_eth_writedata),
      .read                                                        (soft_csr_base_read),
      .write                                                       (soft_csr_base_write),
      .byteenable                                                  (i_reconfig_eth_byteenable),
      .readdata                                                    (soft_csr_base_readdata),
      .readdatavalid                                               (soft_csr_base_readdata_valid),
      .address                                                     ({i_reconfig_eth_addr[8:0],2'b00})
);

  generate if (ENABLE_PTP == 1) begin : g_ptp
    eth_f_ptp_csr #(
      .PL (LANE_NUM)
    ) soft_ptp_csr (
      .clk                            (i_reconfig_clk             ),
      .reset                          (i_reconfig_reset           ),
      .writedata                      (i_reconfig_eth_writedata   ),
      .read                           (soft_csr_ptp_read          ),
      .write                          (soft_csr_ptp_write         ),
      .byteenable                     (i_reconfig_eth_byteenable  ),
      .readdata                       (soft_csr_ptp_readdata      ),
      .readdatavalid                  (soft_csr_ptp_readdata_valid),
      .address                        ({i_reconfig_eth_addr[9:0],2'b00}  ),
      // Input
      .i_tx_ehip_preamble_passthru_gui (i_tx_ehip_preamble_passthru_gui),
      .o_tx_ehip_preamble_passthru_dr  (o_tx_ehip_preamble_passthru_dr),
      // Input
      .i_ptp_clk                      (i_ptp_clk          ),
      .i_tx_tod_clk                   (i_tx_tod_clk       ),
      .i_rx_tod_clk                   (i_rx_tod_clk       ),
      .i_samp_clk                     (i_samp_clk         ),
      .i_tx_srst_n                    (i_tx_srst_n        ),
      .i_rx_srst_n                    (i_rx_srst_n        ),
      .i_tx_tod_srst_n                (i_tx_tod_srst_n    ),
      .i_rx_tod_srst_n                (i_rx_tod_srst_n    ),
      .i_tx_samp_srst_n               (i_tx_samp_srst_n   ),
      .i_rx_samp_srst_n               (i_rx_samp_srst_n   ),
      .i_tx_lanes_stable_status       (tx_lanes_stable_status),
      .i_rx_pcs_fully_aligned         (i_rx_pcs_fully_aligned),
      .i_tx_apulse_wdly_valid         (i_tx_apulse_wdly_valid         ),
      .i_tx_apulse_offset_valid       (i_tx_apulse_offset_valid       ),
      .i_tx_apulse_time_valid         (i_tx_apulse_time_valid         ),
      .i_rx_apulse_wdly_valid         (i_rx_apulse_wdly_valid         ),
      .i_rx_apulse_offset_valid       (i_rx_apulse_offset_valid       ),
      .i_rx_apulse_time_valid         (i_rx_apulse_time_valid         ),
      .i_tx_apulse_wdly               (i_tx_apulse_wdly               ),
      .i_tx_apulse_offset             (i_tx_apulse_offset             ),
      .i_tx_apulse_time               (i_tx_apulse_time               ),
      .i_rx_apulse_wdly               (i_rx_apulse_wdly               ),
      .i_rx_apulse_offset             (i_rx_apulse_offset             ),
      .i_rx_apulse_time               (i_rx_apulse_time               ),
      .i_tx_const_adjust              (i_tx_const_adjust              ),
      .i_rx_const_adjust              (i_rx_const_adjust              ),
      .i_rx_ptp_vl_snapshot           (i_rx_ptp_vl_snapshot           ),
      .i_tx_tam_valid                 (i_tx_tam_valid                 ),
      .i_tx_tam_cnt                   (i_tx_tam_cnt                   ),
      .i_tx_tam_ui                    (i_tx_tam_ui                    ),
      .i_rx_tam_valid                 (i_rx_tam_valid                 ),
      .i_rx_tam_cnt                   (i_rx_tam_cnt                   ),
      .i_rx_tam_ui                    (i_rx_tam_ui                    ),
      .i_tx_ptp_offset_data_valid     (i_tx_ptp_offset_data_valid     ),
      .i_rx_ptp_offset_data_valid     (i_rx_ptp_offset_data_valid     ),
      .i_tx_ptp_ready                 (i_tx_ptp_ready                 ),
      .i_rx_ptp_ready                 (i_rx_ptp_ready                 ),
      .i_tx_ptp_user_cfg_done_clrn    (i_tx_ptp_user_cfg_done_clrn    ),
      .i_rx_ptp_user_cfg_done_clrn    (i_rx_ptp_user_cfg_done_clrn    ),
      // Output
      .o_rx_lal                       (o_rx_lal                       ),
      .o_tx_ref_lane                  (o_tx_ref_lane                  ),
      .o_tx_calc_adjust               (o_tx_calc_adjust               ),
      .o_rx_calc_adjust               (o_rx_calc_adjust               ),
      .o_rx_fec_cw_pos_user_cfg_done  (o_rx_fec_cw_pos_user_cfg_done  ),
      .o_tx_ptp_user_cfg_done         (o_tx_ptp_user_cfg_done         ),
      .o_rx_ptp_user_cfg_done         (o_rx_ptp_user_cfg_done         )
    );
  end
  else begin : no_g_ptp
    // read data is always returned on the next cycle
    always @(posedge i_reconfig_clk)
        if (i_reconfig_reset) soft_csr_ptp_readdata_valid <= 1'b0; else soft_csr_ptp_readdata_valid <= soft_csr_ptp_read;
        
    assign soft_csr_ptp_readdata            = 32'b0;
    assign o_rx_lal                         = 3'b000;
    assign o_tx_ref_lane                    = 3'b000;
    assign o_tx_calc_adjust_valid           = 1'b0;
    assign o_tx_calc_adjust                 = 32'd0;
    assign o_rx_calc_adjust_valid           = 1'b0;
    assign o_rx_calc_adjust                 = 32'd0;
    assign o_rx_vl_offset_user_cfg_done     = 1'b0;
    assign o_rx_fec_cw_pos_user_cfg_done    = 1'b0;
    assign o_tx_ptp_user_cfg_done           = 1'b0;
    assign o_rx_ptp_user_cfg_done           = 1'b0;
  end
  endgenerate



  // TODO: make connection between bridge and avmm1 BB signals
  //
  
  eth_f_ft_avmm_32to8_bridge #(
      .READ_PIPELINE_ENABLE (1),
      .ADDR_WIDTH           (17)
  ) avmm_32b_to_8b (

      // AVMM slave port  
      .i_clk                    (i_reconfig_clk),
      .i_rst                    (i_reconfig_reset),
 
      .i_avmm_s32_addr          ({1'b0,i_reconfig_eth_addr[13:0],2'b00}),
      .i_avmm_s32_wdata         (i_reconfig_eth_writedata[31:0]),
      .i_avmm_s32_write         (ehip_write),
      .i_avmm_s32_read          (ehip_read),
      .i_avmm_s32_byte_enable   (i_reconfig_eth_byteenable[3:0]),
      .o_avmm_s32_readdata      (ehip_readdata[31:0]),
      .o_avmm_s32_waitrequest   (ehip_waitrequest),
      .o_avmm_s32_readdatavalid (ehip_readdata_valid),

      // AVMM Master Port
      .o_avmm_m8_addr           (avmm_m8_addr[17:0]), //[17]: DW access
      .o_avmm_m8_wdata          (avmm_m8_wdata[7:0]),
      .o_avmm_m8_write          (avmm_m8_write),
      .o_avmm_m8_read           (avmm_m8_read),
      .i_avmm_m8_readdata       (avmm_m8_readdata[7:0]),
      .i_avmm_m8_waitrequest    (avmm_m8_waitrequest)
  );

 wire [1:0] addr_b9_b8 = is_maib ? 2'b11: {1'b0, avmm_m8_addr[17]};

 eth_f_ctf_avmm1_soft_logic 
 #(
    .avmm_interfaces (1),  //Number of AVMM interfaces required - one for each bonded_lane, PLL, and Master CGkjkB
    .rcfg_enable     (1)   //Enable/disable reconfig interface in the Native PHY or PLL IP
 )  ctf_avmm1_soft_logic_inst (
  //PORT_LIST_START
  // AVMM slave interface signals (user)
    .avmm_clk                   (i_reconfig_clk),
    .avmm_reset                 (i_reconfig_reset),
    .avmm_writedata             (avmm_m8_wdata[7:0]),
    .avmm_address               ({addr_b9_b8, avmm_m8_addr[7:0]}), //10 bits. addr[9:8]:2'b11 for MAIB, rest for F-tile, addr[8] is Dword Access
    .avmm_reservedin            (avmm_m8_addr[16:8]), //9 bits
    .avmm_write                 (avmm_m8_write),
    .avmm_read                  (avmm_m8_read),
    .avmm_readdata              (avmm_m8_readdata[7:0]),
    .avmm_waitrequest           (avmm_m8_waitrequest),
  // Signals from AVMM1 building block
    .pld_avmm1_busy_real        (pld_avmm1_busy),
    .pld_avmm1_cmdfifo_wr_full_real (pld_avmm1_cmdfifo_wr_full),
    .pld_avmm1_cmdfifo_wr_pfull_real(pld_avmm1_cmdfifo_wr_pfull),
    .pld_avmm1_readdata_real    (pld_avmm1_readdata[7:0]), // 8 bits
    .pld_avmm1_readdatavalid_real(pld_avmm1_readdatavalid),
    .pld_avmm1_reserved_out_real (pld_avmm1_reserved_out), // 3 bits
    .pld_chnl_cal_done_real      (pld_chnl_cal_done), 
    .pld_hssi_osc_transfer_en_real (pld_hssi_osc_transfer_en),
  // Signals to AVMM1 building block
    .pld_avmm1_clk_rowclk_real    (pld_avmm1_clk_rowclk),
    .pld_avmm1_read_real          (pld_avmm1_read),
    .pld_avmm1_reg_addr_real      (pld_avmm1_reg_addr[9:0]), // 10 bits
    .pld_avmm1_request_real       (pld_avmm1_request),
    .pld_avmm1_reserved_in_real   (pld_avmm1_reserved_in[8:0]), // 9 bits
    .pld_avmm1_write_real         (pld_avmm1_write),
    .pld_avmm1_writedata_real     (pld_avmm1_writedata[7:0]) // 8 bits
  //PORT_LIST_END
);
 
endmodule
