module ex_100G (
		input  wire         i_clk_tx,                        //              i_tx_clk.clk,                    TX interface clock, in synchronous mode connect o_clk_pll to this clock
		input  wire         i_clk_rx,                        //              i_rx_clk.clk,                    RX interface clock, in synchronous mode connect o_clk_pll to this clock
		output wire         o_clk_pll,                       //             o_clk_pll.clk,                    System clock, this is /64 clock
		input  wire         i_reconfig_clk,                  //        i_reconfig_clk.clk,                    XCVR reconfiguration clock
		input  wire         i_reconfig_reset,                //      i_reconfig_reset.reset,                  XCVR reconfig reset
		output wire         o_sys_pll_locked,                //      o_sys_pll_locked.o_sys_pll_locked,       SYSTEM PLL Locked
		output wire [3:0]   o_tx_serial,                     //                serial.o_tx_serial,            TX XCVR serial pins
		input  wire [3:0]   i_rx_serial,                     //                      .i_rx_serial,            RX XCVR serial pins
		output wire [3:0]   o_tx_serial_n,                   //                      .o_tx_serial_n,          TX XCVR serial pins
		input  wire [3:0]   i_rx_serial_n,                   //                      .i_rx_serial_n,          RX XCVR serial pins
		input  wire         i_clk_ref,                       //             i_clk_ref.clk,                    Reference clock used by the RX PHY
		input  wire         i_clk_sys,                       //             i_clk_sys.clk,                    Ethernet system clock
		input  wire [13:0]  i_reconfig_eth_addr,             //    reconfig_eth_slave.address,                Address for Ethernet CSRs
		input  wire [3:0]   i_reconfig_eth_byteenable,       //                      .byteenable,             Byte enable for read/write to Ethernet CSRs
		output wire         o_reconfig_eth_readdata_valid,   //                      .readdatavalid,          Read data from Ethernet CSRs is valid
		input  wire         i_reconfig_eth_read,             //                      .read,                   Read command for Ethernet CSRs
		input  wire         i_reconfig_eth_write,            //                      .write,                  Write command for Ethernet CSRs
		output wire [31:0]  o_reconfig_eth_readdata,         //                      .readdata,               Read data from reads to Ethernet CSRs
		input  wire [31:0]  i_reconfig_eth_writedata,        //                      .writedata,              Data for writes to Ethernet CSRs
		output wire         o_reconfig_eth_waitrequest,      //                      .waitrequest,            AVMM stall signal for operation on Ethernet CSRs
		input  wire         i_rst_n,                         //               i_rst_n.reset_n,                Active-low hard reset signal.Resets the TX interface, including the TX PCS and TX MAC. This reset leads to the deassertion of the o_tx_lanes_stable output signal.
		input  wire         i_tx_rst_n,                      //            i_tx_rst_n.reset_n,                Active-low hard reset signal.Resets the TX interface, including the TX PCS and TX MAC. This reset leads to the deassertion of the o_tx_lanes_stable output signal.
		input  wire         i_rx_rst_n,                      //            i_rx_rst_n.reset_n,                Active-low hard reset signal.Resets the RX interface, including the RX PCS and RX MAC. This reset leads to the deassertion of the o_rx_pcs_ready output signal.
		output wire         o_rst_ack_n,                     //    reset_status_ports.o_rst_ack_n,            Acknowledge signal for i_rst_n
		output wire         o_tx_rst_ack_n,                  //                      .o_tx_rst_ack_n,         Acknowledge signal for i_tx_rst_n
		output wire         o_rx_rst_ack_n,                  //                      .o_rx_rst_ack_n,         Acknowledge signal for i_rx_rst_n
		output wire         o_cdr_lock,                      //    clock_status_ports.o_cdr_lock,             CDR lock
		output wire         o_tx_pll_locked,                 //                      .o_tx_pll_locked,        TX PLL Locked
		output wire         o_tx_lanes_stable,               //                      .o_tx_lanes_stable,      Asserted when TX MAC is ready to send data
		output wire         o_rx_pcs_ready,                  //                      .o_rx_pcs_ready,         Asserted when RX PCS is ready to receive data
		output wire         o_clk_tx_div,                    //            clk_tx_div.clk,                    This is /66 clock (390.625 MHz)
		output wire         o_clk_rec_div64,                 //         clk_rec_div64.clk,                    This is /64 recovered clock (402.83 MHz)
		output wire         o_clk_rec_div,                   //           clk_rec_div.clk,                    This is /66 recovered clock (390.625 MHz)
		output wire         o_rx_block_lock,                 //          status_ports.o_rx_block_lock,        Asserted when 66b block alignment is finished on all PCS lanes
		output wire         o_rx_am_lock,                    //                      .o_rx_am_lock,           Asserted when RX PCS has found detected alignment markers and deskewed PCS lanes
		output wire         o_local_fault_status,            //                      .o_local_fault_status,   The RX PCS has detected a problem that prevents it from being able to receive data
		output wire         o_remote_fault_status,           //                      .o_remote_fault_status,  The remote link partner has sent remote fault ordered sets indicating that it is unable to receive data
		input  wire         i_stats_snapshot,                //                      .i_stats_snapshot,       Stats Snapshot
		output wire         o_rx_hi_ber,                     //                      .o_rx_hi_ber,            Asserted when RX PCS detected high BER; in AN/LT mode, this will trigger re-negotiation
		output wire         o_rx_pcs_fully_aligned,          //                      .o_rx_pcs_fully_aligned, PCS is fully aligned
		input  wire [511:0] i_tx_data,                       //          tx_streaming.data,                   Input data to the MAC. Bits 0 is the LSB.
		input  wire         i_tx_valid,                      //                      .valid,                  Indicates data TX data is valid. Must remain high throughout transmission of packet.
		input  wire         i_tx_startofpacket,              //                      .startofpacket,          Start of packet (SOP). Asserted for one cycle at the beginning of frame.
		input  wire         i_tx_endofpacket,                //                      .endofpacket,            End of packet (EOP). Asserted for one cycle on the last cycle of the frame.
		input  wire         i_tx_error,                      //                      .error,                  A valid high on this signal aligned with an eop will cause the tx frame to be treated as an error.
		output wire         o_tx_ready,                      //                      .ready,                  Indicates that the MAC is ready to accept new data.
		input  wire [5:0]   i_tx_empty,                      //                      .empty,                  Indicates the number of empty bytes at the end of the frame.
		input  wire         i_tx_skip_crc,                   //              tx_ports.i_tx_skip_crc,          Driving '0' on this port means TX MAC will add CRC and '1' means MAC will not add CRC i.e. no PAD and no source address insertion
		output wire [511:0] o_rx_data,                       //          rx_streaming.data,                   output data from the MAC, Bit 0 is LSB.
		output wire         o_rx_valid,                      //                      .valid,                  Indicates data RX data, EOP, and SOP are valid.
		output wire         o_rx_startofpacket,              //                      .startofpacket,          Start of packet (SOP). Asserted for one cycle at the beginning of  frame.
		output wire         o_rx_endofpacket,                //                      .endofpacket,            End of packet (EOP). Asserted for one cycle on the last cycle of the frame.
		output wire [5:0]   o_rx_error,                      //                      .error,                  RX error bits asserted on the EOP cycle.
		output wire [5:0]   o_rx_empty,                      //                      .empty,                  Indicates the number of empty bytes at the end of the frame.
		output wire [39:0]  o_rxstatus_data,                 //              rx_ports.o_rxstatus_data,        RX frame status information.
		output wire         o_rxstatus_valid,                //                      .o_rxstatus_valid,       Asserted to indicate that bits on o_rxstatus_data are valid.
		input  wire [7:0]   i_tx_pfc,                        //             pfc_ports.i_tx_pfc,               Used to send a PFC frame
		output wire [7:0]   o_rx_pfc,                        //                      .o_rx_pfc,               Indicates that a PFC frame was received for the priority queue q, where q is the bit number that was asserted
		input  wire         i_tx_pause,                      //             sfc_ports.i_tx_pause,             Asserted to send a pause frame
		output wire         o_rx_pause,                      //                      .o_rx_pause,             Indicates that a pause frame was received
		input  wire [17:0]  i_reconfig_xcvr0_addr,           // reconfig_xcvr_slave_0.address
		input  wire [3:0]   i_reconfig_xcvr0_byteenable,     //                      .byteenable
		output wire         o_reconfig_xcvr0_readdata_valid, //                      .readdatavalid
		input  wire         i_reconfig_xcvr0_read,           //                      .read
		input  wire         i_reconfig_xcvr0_write,          //                      .write
		output wire [31:0]  o_reconfig_xcvr0_readdata,       //                      .readdata
		input  wire [31:0]  i_reconfig_xcvr0_writedata,      //                      .writedata
		output wire         o_reconfig_xcvr0_waitrequest,    //                      .waitrequest
		input  wire [17:0]  i_reconfig_xcvr1_addr,           // reconfig_xcvr_slave_1.address
		input  wire [3:0]   i_reconfig_xcvr1_byteenable,     //                      .byteenable
		output wire         o_reconfig_xcvr1_readdata_valid, //                      .readdatavalid
		input  wire         i_reconfig_xcvr1_read,           //                      .read
		input  wire         i_reconfig_xcvr1_write,          //                      .write
		output wire [31:0]  o_reconfig_xcvr1_readdata,       //                      .readdata
		input  wire [31:0]  i_reconfig_xcvr1_writedata,      //                      .writedata
		output wire         o_reconfig_xcvr1_waitrequest,    //                      .waitrequest
		input  wire [17:0]  i_reconfig_xcvr2_addr,           // reconfig_xcvr_slave_2.address
		input  wire [3:0]   i_reconfig_xcvr2_byteenable,     //                      .byteenable
		output wire         o_reconfig_xcvr2_readdata_valid, //                      .readdatavalid
		input  wire         i_reconfig_xcvr2_read,           //                      .read
		input  wire         i_reconfig_xcvr2_write,          //                      .write
		output wire [31:0]  o_reconfig_xcvr2_readdata,       //                      .readdata
		input  wire [31:0]  i_reconfig_xcvr2_writedata,      //                      .writedata
		output wire         o_reconfig_xcvr2_waitrequest,    //                      .waitrequest
		input  wire [17:0]  i_reconfig_xcvr3_addr,           // reconfig_xcvr_slave_3.address
		input  wire [3:0]   i_reconfig_xcvr3_byteenable,     //                      .byteenable
		output wire         o_reconfig_xcvr3_readdata_valid, //                      .readdatavalid
		input  wire         i_reconfig_xcvr3_read,           //                      .read
		input  wire         i_reconfig_xcvr3_write,          //                      .write
		output wire [31:0]  o_reconfig_xcvr3_readdata,       //                      .readdata
		input  wire [31:0]  i_reconfig_xcvr3_writedata,      //                      .writedata
		output wire         o_reconfig_xcvr3_waitrequest,    //                      .waitrequest
		input  wire         i_clk_pll                        //             i_clk_pll.clk,                    Drives internal clock. Use this clock only when PTP and asynchronous modes are enabled. Connect o_clk_pll of chosen eth_f to this clock
	);
endmodule

