// (C) 2001-2022 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1 ps / 1 ps
`ifndef FTILE_TOP_PATH
`define QUARTUS_TOP_LEVEL_ENTITY_INSTANCE_PATH eth_f_hw_auto_tiles.z1577a_x0_y0_n0
`else
`define QUARTUS_TOP_LEVEL_ENTITY_INSTANCE_PATH `FTILE_TOP_PATH
`endif
`ifndef TILE_INST_PATH
`define tile_instance_path z1577a.z1577a_inst
`else
`define tile_instance_path `TILE_INST_PATH
`endif
module  basic_avl_tb_top ();
    localparam CLIENT_IF_TYPE  = 1;
    localparam AIB_LANES       = 4;
    localparam EMPTY_BITS      = 6;
    localparam DATA_WIDTH      = 8;
    localparam DATA_RATE       = "100G";
    localparam INFRAME         = 4;
    localparam SEG_STATUS      = 12;
    localparam SEG_ERROR       = 4;
    localparam FCS_ERROR       = 4;
    localparam SEG_EOP_EMPTY   = 12;
    localparam PCS_CONTROL     = 32;
    localparam LANE_NUM        = 4;
    localparam INSTANCE_NUM    = 1;
    localparam FEC_LANES       = (DATA_RATE=="400G"? 4 : (DATA_RATE=="200G"? 2 : 1));
    localparam MODULATION      = 0;
    localparam PTP_FP_WIDTH    = 8;
    localparam PKT_LOOP_COUNT  = 1;
    localparam ED_PKTGEN_CONT_MODE  = 0;
    localparam CONT_MODE_TIMEOUT  = 1;
    localparam PORT_NUM        = 32'h0;

//---------------------------------------------
localparam ETH_BASE_ADDR        = 24'h00_0000;
localparam PKT_CLIENT_BASE_ADDR = 24'h10_0000;
localparam XCVR_BASE_ADDR       = 24'h80_0000;
localparam cfg_pkt_client_ctrl_addr  = 24'(PKT_CLIENT_BASE_ADDR + 8'h00);
localparam cfg_test_loop_cnt_addr    = 24'(PKT_CLIENT_BASE_ADDR + 8'h04);
localparam cfg_rom_addr_addr         = 24'(PKT_CLIENT_BASE_ADDR + 8'h08);

//---------------------------------------------
logic         loopback_client_test, sys_rx_path_working, sys_tx_ready_working;
logic [31:0]  rdata, rdata1, rdata2, rdata3;
logic [31:0]  cfg_pkt_tx_cnt, cfg_pkt_rx_cnt;
logic [63:0]  stat_tx_sop_cnt, stat_tx_eop_cnt, stat_tx_err_cnt;
logic [63:0]  stat_rx_sop_cnt, stat_rx_eop_cnt, stat_rx_err_cnt;
logic [63:0]  stat_tx_eop_cnt_r, stat_rx_eop_cnt_r;

//---------------------------------------------
integer k;
logic [7:0]   inst_i;
logic [31:0]  cfg_test_loop_cnt;  
logic [15:0]  init_rom_start_addr;
logic [15:0]  init_rom_end_addr;
logic [31:0]  cfg_pkt_client_ctrl;
//---[0]= 1: start TX; 0: stop TX;
//---[2]= 1:continuous mode; 0:one shot mode;
//---[4]= 1: loopback client enabled; 0: send packets from ROM; 
//---[8]= 1: clear packet TX/RX counters; self-clean;


wire [23:0] ehip_base_addr = (DATA_RATE == "400G") ? 'h5000 :
                             (DATA_RATE == "200G") ? 'h4000 :
                             (DATA_RATE == "100G") ? 'h3000 :
                             (DATA_RATE ==  "50G") ? 'h2000 :
                             (DATA_RATE ==  "40G") ? 'h3000 :
                             (DATA_RATE ==  "25G") ? 'h1000 : 
                                                     'h1000; // 10G

wire [23:0] lphy_stat_base_addr = (DATA_RATE == "400G") ? 'h7e00 :
                                  (DATA_RATE == "200G") ? 'h6e00 :
                                  (DATA_RATE == "100G") ? 'h6600 :
                                  (DATA_RATE ==  "50G") ? 'h6200 :
                                  (DATA_RATE ==  "40G") ? 'h6600 :
                                  (DATA_RATE ==  "25G") ? 'h6000 : 
                                                          'h6000; // 10G

//---------------------------------------------
    // Clocks
    reg                          i_refclk2pll                = 1'b0;
    reg                          i_reconfig_clk              = 1'b0;


    //Resets
    reg [INSTANCE_NUM-1:0]     i_tx_rst_n          = {INSTANCE_NUM{1'b1}};
    reg [INSTANCE_NUM-1:0]     i_rst_n             = {INSTANCE_NUM{1'b1}};
    reg [INSTANCE_NUM-1:0]     i_rx_rst_n          = {INSTANCE_NUM{1'b1}};
    reg                                 i_reconfig_reset    = 1'b0;

    wire [INSTANCE_NUM*4-1:0]    i_rx_serial;
    wire [INSTANCE_NUM*4-1:0]    i_rx_serial_n;
    wire [INSTANCE_NUM*4-1:0]    o_tx_serial;
    wire [INSTANCE_NUM*4-1:0]    o_tx_serial_n;
    wire [INSTANCE_NUM-1:0]    o_rst_ack_n;
    wire [INSTANCE_NUM-1:0]    o_tx_rst_ack_n;
    wire [INSTANCE_NUM-1:0]    o_rx_rst_ack_n;

    // Serial data interface
    assign i_rx_serial   = o_tx_serial;
    assign i_rx_serial_n = o_tx_serial_n;




//---------------------------------------------
//---For Jtag avmm control---
//---------------------------------------------
logic          jtag_read, jtag_write, jtag_rd_vld, jtag_waitrequest;
logic [31:0]   jtag_addr; 
logic [31:0]   jtag_rdata, jtag_wdata;
logic [3:0]    jtag_ben; 
   
assign jtag_waitrequest = basic_avl_tb_top.eth_f_hw.master_waitrequest;
assign jtag_rd_vld      = basic_avl_tb_top.eth_f_hw.master_readdatavalid;
assign jtag_rdata       = basic_avl_tb_top.eth_f_hw.master_readdata;

localparam XCVR_TYPE     = 0;
localparam RSFEC_ENABLED = 2; 

`include "eth_f_register_addr.sv"

logic [31:0] avmm2_addr;
logic [31:0] avmm2_data;
logic [3:0]  xcvr_select;
logic [17:0] cur_lane;
logic [31:0] lane_reg_addr;
assign cur_lane = 18'h3ffff;

initial begin
    force basic_avl_tb_top.eth_f_hw.master_address    = jtag_addr;
    force basic_avl_tb_top.eth_f_hw.master_read       = jtag_read;
    force basic_avl_tb_top.eth_f_hw.master_write      = jtag_write;
    force basic_avl_tb_top.eth_f_hw.master_writedata  = jtag_wdata;
    force basic_avl_tb_top.eth_f_hw.master_byteenable = jtag_ben;
end

//---------------------------------------------
eth_f_hw_auto_tiles eth_f_hw_auto_tiles();

eth_f_hw eth_f_hw (
    .clk50(),
    .cpu_resetn(),
    
    // QSFP
    .qsfp_lowpwr(),// LPMode
    .qsfp_rstn(),  // ResetL
    
    // IO
    .i_rx_serial       (i_rx_serial  ),
    .i_rx_serial_n     (i_rx_serial_n), 
    .o_tx_serial       (o_tx_serial  ), 
    .o_tx_serial_n     (o_tx_serial_n), 
    .user_io(),
    .user_led(),

    //clock
    .i_refclk2pll      (i_refclk2pll),
    .i_reconfig_clk    (i_reconfig_clk),
    .i_reconfig_reset  (i_reconfig_reset),
    //rst
    .i_rx_rst_n        (i_rx_rst_n),
    .i_tx_rst_n        (i_tx_rst_n),
    .i_rst_n           (i_rst_n   ),
    .o_rst_ack_n       (o_rst_ack_n),
    .o_rx_rst_ack_n    (o_rx_rst_ack_n),
    .o_tx_rst_ack_n    (o_tx_rst_ack_n)

);




//---------------------------------------------


initial begin

        i_tx_rst_n       = {INSTANCE_NUM{1'b1}};
        i_rx_rst_n       = {INSTANCE_NUM{1'b1}};
        i_rst_n          = {INSTANCE_NUM{1'b1}};
        i_reconfig_reset = 1'b1;
    repeat (500) @(posedge i_reconfig_clk);
        $display("Applying reset");
        loopback_client_test = 0;
        i_tx_rst_n       = {INSTANCE_NUM{1'b0}};
        i_rx_rst_n       = {INSTANCE_NUM{1'b0}};
        i_rst_n          = {INSTANCE_NUM{1'b0}};
        i_reconfig_reset = 1'b1;

        force   jtag_read  = 1'b0;
        force   jtag_write = 1'b0;
        force   jtag_wdata = 32'b0;
        force   jtag_ben   = 4'hF;
        force   jtag_addr  = 32'b0;
		
        wait (basic_avl_tb_top.eth_f_hw.o_rst_ack_n === {INSTANCE_NUM{1'b0}});
        repeat (250) @(posedge i_reconfig_clk);


        i_tx_rst_n       = {INSTANCE_NUM{1'b1}};
        i_rx_rst_n       = {INSTANCE_NUM{1'b1}};
        i_rst_n          = {INSTANCE_NUM{1'b1}};
        i_reconfig_reset = 1'b0;

        $display("--- IP reset sequence started -----");
        //---TX side
        //wait (basic_avl_tb_top.eth_f_hw.hw_ip_top.dut.eth_f_0.u_reset_wrapper.SRC.tx_lane_current_state[19*3+1] === 1'b1);
        wait (basic_avl_tb_top.eth_f_hw.IP_INST[0].hw_ip_top.dut.eth_f_0.sip_inst.o_tx_lanes_stable === 1'b1);
        $display("---TX reset sequence completed -----");
        //---RX side
        //wait (basic_avl_tb_top.eth_f_hw.hw_ip_top.dut.eth_f_0.u_reset_wrapper.SRC.rx_lane_current_state[19*3+1] === 1'b1);
        //wait (basic_avl_tb_top.eth_f_hw.hw_ip_top.dut.eth_f_0.sip_inst.o_pcs_rx_sf === 1'b0);
        wait (basic_avl_tb_top.eth_f_hw.IP_INST[0].hw_ip_top.dut.eth_f_0.sip_inst.o_rx_pcs_ready === 1'b1);

        $display("---RX reset sequence completed -----");

        repeat (100) @(posedge i_reconfig_clk);
        

    for (inst_i=8'h0; inst_i < INSTANCE_NUM; inst_i++) begin
        //---------------------------------------------
        //---Packet TX/RX test---
        //---------------------------------------------
        //---currently system tx_ready signal and RX path doesn't work for some mode;
        check_sys_tx_ready_status();
        check_sys_rx_path_status(); 
        for (k=0; k<PKT_LOOP_COUNT; k=k+1) begin
            repeat (500) @(posedge i_reconfig_clk);
            cfg_test_loop_cnt = k+1;                    // How many blocks of rom data to send;
            cfg_pkt_tx_cnt    = 16*cfg_test_loop_cnt;   // every data block has 16 packets;
            $display("---IP_INST[%d] Test %4d;   ---Total %6d packets to send-----", inst_i, k, cfg_pkt_tx_cnt);
            set_init_rom_addr();
            avmm_write({inst_i, cfg_rom_addr_addr}, {init_rom_end_addr, init_rom_start_addr});
            avmm_write({inst_i, cfg_test_loop_cnt_addr}, cfg_test_loop_cnt);
		

            //---------------------------------------------
            $display("------IP_INST[%d] Start pkt gen TX-----", inst_i);
			cfg_pkt_client_ctrl = 32'h01;   // start pkt TX in oneshot mode;
            avmm_write({inst_i, cfg_pkt_client_ctrl_addr}, cfg_pkt_client_ctrl); 
            repeat (10) @(posedge basic_avl_tb_top.eth_f_hw.i_clk_tx[inst_i]);


            if (sys_tx_ready_working == 1'b0) begin
                if (CLIENT_IF_TYPE==0) begin
                    $display("---------Force o_tx_mac_ready to 1 for Segmented Interface-----");
                    force   basic_avl_tb_top.eth_f_hw.IP_INST[0].hw_ip_top.o_tx_mac_ready = 1'b1;      // start mac IF;
                end else if (CLIENT_IF_TYPE==1) begin
                    $display("---------Force o_tx_ready to 1 for AVST Interface-----");
                    force   basic_avl_tb_top.eth_f_hw.IP_INST[0].hw_ip_top.o_tx_ready = 1'b1;      // start mac IF;

                end
            end

	// Wait for counters to update
	#200ns;

                pkt_tx_rx_check(0, inst_i);

            cfg_pkt_client_ctrl = 32'h0100;   // stop pkt TX; and clear counters;
                avmm_write({inst_i,cfg_pkt_client_ctrl_addr}, cfg_pkt_client_ctrl); 
        end

            if ((loopback_client_test==1) & (sys_rx_path_working == 1'b1)) begin
                //---loopback test---
                repeat (500) @(posedge basic_avl_tb_top.eth_f_hw.i_clk_tx[inst_i]);
                $display("IP_INST[%d]---Loopback Client test-----", inst_i);
                $display("IP_INST[%d]------Start Loopback Client TX-----", inst_i);
                avmm_write({inst_i,cfg_pkt_client_ctrl_addr}, 32'h0011); 
                repeat (1000) @(posedge basic_avl_tb_top.eth_f_hw.i_clk_tx[inst_i]);
                $display("IP_INST[%d]------Stop  Loopback Client TX-----", inst_i);
                avmm_write({inst_i,cfg_pkt_client_ctrl_addr}, 32'h0010); 
                repeat (10) @(posedge basic_avl_tb_top.eth_f_hw.i_clk_tx[inst_i]);
                pkt_tx_rx_check(1,inst_i);
            end
		

    end

       //---------------------------------------------
       //---avmm read & write test---
       //Skipping the AVMM2 test for Barak currently
       //---------------------------------------------
        #100000;     
    for (inst_i=8'h0; inst_i < INSTANCE_NUM; inst_i++) begin
        avmm_test(inst_i);
        avmm2_test(inst_i);
        repeat (50) @(negedge i_reconfig_clk);
        $display("**** AVMM Read/Write Operation Completed for IP_INST[%d]****", inst_i);
    end
    


      if  (stat_rx_err_cnt!=0 || stat_rx_sop_cnt ==0 ||stat_rx_eop_cnt == 0) begin
		$display("** Packet ERROR detected or NO packets received");
		end else begin
        $display("** Testbench complete");
	end 
        $display("**");
        $display("*****************************************");
      
        #200000;
        $finish();

end

//---------------------------------------------


    always begin
        #2000 i_reconfig_clk = ~i_reconfig_clk;
    end
    always begin
         #3200 i_refclk2pll = ~i_refclk2pll;
    end
    

 
 
//---------------------------------------------
//---set ROM satrt/end address for default initial hex file;
//---For user defined ROM content, address should change accordingly;
//---------------------------------------------
task set_init_rom_addr();
   begin
    if (CLIENT_IF_TYPE==0) begin    //---Segmented IF
        if ((DATA_RATE=="10G") | (DATA_RATE=="25G")) begin
            //defparam basic_avl_tb_top.eth_f_hw.hw_ip_top.PKT_ROM_INIT_FILE = eth_f_hw_pkt_gen_rom_init.25G_SEG.hex;
            init_rom_start_addr = 16'h0000;
            init_rom_end_addr   = 16'h007F;
        end else if ((DATA_RATE=="40G") | (DATA_RATE=="50G")) begin
            init_rom_start_addr = 16'h0000;
            init_rom_end_addr   = 16'h004A;
        end else if (DATA_RATE=="100G") begin
            init_rom_start_addr = 16'h0000;
            init_rom_end_addr   = 16'h002D;
        end else if (DATA_RATE=="200G") begin
            init_rom_start_addr = 16'h0000;
            init_rom_end_addr   = 16'h001F;
        end else if (DATA_RATE=="400G") begin
            init_rom_start_addr = 16'h0000;
            init_rom_end_addr   = 16'h0017;
        end        
    end else if (CLIENT_IF_TYPE==1) begin    //---AvST IF
        if ((DATA_RATE=="10G") | (DATA_RATE=="25G")) begin
            //defparam basic_avl_tb_top.eth_f_hw.hw_ip_top.PKT_ROM_INIT_FILE = eth_f_hw_pkt_gen_rom_init.25G_AVST.hex;
            init_rom_start_addr = 16'h0000;
            init_rom_end_addr   = 16'h007F;
        end else if ((DATA_RATE=="40G") | (DATA_RATE=="50G")) begin
            init_rom_start_addr = 16'h0000;
            init_rom_end_addr   = 16'h003F;
        end else if (DATA_RATE=="100G") begin
            init_rom_start_addr = 16'h0000;
            init_rom_end_addr   = 16'h00F;
        end        
    end  
  end
endtask

//---------------------------------------------
task check_sys_tx_ready_status();
    begin
        if (CLIENT_IF_TYPE==0) begin
            if ((DATA_RATE=="10G") | (DATA_RATE=="40G")) 
                  sys_tx_ready_working = 1'b1;
            else  sys_tx_ready_working = 1'b1;
        end else if (CLIENT_IF_TYPE==1) begin
                  sys_tx_ready_working = 1'b1;
        end
    end
endtask

task check_sys_rx_path_status();
    begin
        if (CLIENT_IF_TYPE==0) begin
            if ((DATA_RATE=="10G") | (DATA_RATE=="40G")) 
                  sys_rx_path_working = 1'b1;
            else  sys_rx_path_working = 1'b1;
        end else if (CLIENT_IF_TYPE==1) begin
                  sys_rx_path_working = 1'b1;
        end
    end
endtask

logic tx_pkt_cnt_match, rx_pkt_cnt_match, pkt_cnt_match;
task pkt_tx_rx_check; 
       input check_mode;    // 0: check if pkt cnt match predefined numbers; 1: check if tx pkt cnt matches rx pkt cnt;
       input [7:0] inst_offset;
    begin
        $display("------Checking Packet TX/RX result-----");
        stat_tx_eop_cnt = 0;
        stat_rx_eop_cnt = 0;
        stat_tx_eop_cnt_r = -1;
        stat_rx_eop_cnt_r = -1;
        repeat (1) @(posedge i_reconfig_clk);
        if (sys_rx_path_working == 1'b0) begin
                   $display("---------System RX path is not working yet; Expected Received packets Number is 0--------");
                   cfg_pkt_rx_cnt = 0;  
        end else   cfg_pkt_rx_cnt = cfg_pkt_tx_cnt;

        pkt_cnt_match = 0;
        while (pkt_cnt_match != 1) begin
            read_tx_rx_counters(inst_offset);
            $display("------------%4d packets Sent;  %4d packets Received--------", stat_tx_eop_cnt, stat_rx_eop_cnt);
            if (stat_tx_eop_cnt != stat_tx_eop_cnt_r) begin
                //$display("------------Sent Packet number =  %4d ...", stat_tx_eop_cnt);
                if (stat_tx_eop_cnt == cfg_pkt_tx_cnt) begin
                    $display("------ALL %4d  packets Sent out---", cfg_pkt_tx_cnt);
                end
            end
            if (stat_rx_eop_cnt != stat_rx_eop_cnt_r) begin
                //$display("------------Received Packet number =  %4d ...", stat_rx_eop_cnt);
                if (stat_rx_eop_cnt == cfg_pkt_rx_cnt) begin
                    $display("------ALL %4d  packets Received---", cfg_pkt_rx_cnt);
                end
            end
            if (stat_tx_err_cnt != 0) begin
                $display("------------Sent ERROR Packet number =  %4d ...", stat_tx_err_cnt);
            end
            if (stat_rx_err_cnt != 0) begin
                $display("------------Received ERROR Packet number =  %4d ...", stat_rx_err_cnt);
            end
            stat_tx_eop_cnt_r = stat_tx_eop_cnt;
            stat_rx_eop_cnt_r = stat_rx_eop_cnt;
            tx_pkt_cnt_match = (stat_tx_eop_cnt == cfg_pkt_tx_cnt);
            rx_pkt_cnt_match = (stat_rx_eop_cnt == cfg_pkt_rx_cnt);

            pkt_cnt_match = (check_mode==0) ? (tx_pkt_cnt_match & rx_pkt_cnt_match) : (stat_tx_eop_cnt == stat_rx_eop_cnt);
            repeat (100) @(posedge i_reconfig_clk);
        end
        if ((pkt_cnt_match == 1) & (stat_rx_err_cnt==0)) begin
                $display("------TX/RX packet check OK---\n");
        end else begin
                $display("------TX/RX packet check FAIL---\n");
        end
        repeat (10) @(posedge i_reconfig_clk);
    end
endtask


task read_tx_rx_counters;
    input [7:0] inst_offset;
    begin


		cfg_pkt_client_ctrl = 32'h0041; 

	   //set snapshot 
            avmm_write({inst_offset, cfg_pkt_client_ctrl_addr}, cfg_pkt_client_ctrl); 
            repeat (10) @(posedge basic_avl_tb_top.eth_f_hw.i_clk_tx[inst_offset]);

        repeat (1) @(posedge i_reconfig_clk);
        avmm_read({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h38}, stat_rx_sop_cnt[31:0], 0, 0);
        avmm_read({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h3C}, stat_rx_sop_cnt[63:32], 0, 0);
	avmm_read({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h40}, stat_rx_eop_cnt[31:0], 0, 0);
        avmm_read({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h44}, stat_rx_eop_cnt[63:32], 0, 0);
	avmm_read({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h48}, stat_rx_err_cnt[31:0], 0, 0);
        avmm_read({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h4C}, stat_rx_err_cnt[63:32], 0, 0);


	avmm_read({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h20}, stat_tx_sop_cnt[31:0], 0, 0);
        avmm_read({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h24}, stat_tx_sop_cnt[63:32], 0, 0);
	avmm_read({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h28}, stat_tx_eop_cnt[31:0], 0, 0);
        avmm_read({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h2C}, stat_tx_eop_cnt[63:32], 0, 0);
	avmm_read({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h30}, stat_tx_err_cnt[31:0], 0, 0);
        avmm_read({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h34}, stat_tx_err_cnt[63:32], 0, 0);


	

		cfg_pkt_client_ctrl = 32'h0001; 



	 

		 //clear snapshot
            avmm_write({inst_offset, cfg_pkt_client_ctrl_addr}, cfg_pkt_client_ctrl); 
            repeat (10) @(posedge basic_avl_tb_top.eth_f_hw.i_clk_tx[inst_offset]);




		cfg_pkt_client_ctrl = 32'h0081; 


	 

		 //set clear
            avmm_write({inst_offset, cfg_pkt_client_ctrl_addr}, cfg_pkt_client_ctrl); 
            repeat (10) @(posedge basic_avl_tb_top.eth_f_hw.i_clk_tx[inst_offset]);



		cfg_pkt_client_ctrl = 32'h0001; 



		 //clear clear
            avmm_write({inst_offset, cfg_pkt_client_ctrl_addr}, cfg_pkt_client_ctrl); 
            repeat (10) @(posedge basic_avl_tb_top.eth_f_hw.i_clk_tx[inst_offset]);



      
    end
endtask


//---------------------------------------------
//---------------------------------------------
task avmm_test;
    input [7:0] inst_offset;
    begin
        //----------------------------------------------------------------------
        //----------------------- AVMM READ/WRITE------------------------------
        //----------------------------------------------------------------------
        $display("****Starting AVMM Read/Write****");



        //Write and Read scratch register
        repeat (10) @(negedge i_reconfig_clk);
        avmm_write({inst_offset, 24'h104}, 32'habcd_ef01);
        repeat (10) @(negedge i_reconfig_clk);
        avmm_read ({inst_offset, 24'h104}, rdata, 32'habcd_ef01, 1);
        repeat (10) @(negedge i_reconfig_clk);

        //Write and Read IP soft reset register
        repeat (10) @(negedge i_reconfig_clk);
        avmm_write({inst_offset, 24'h108}, 32'h0000_0007);
        repeat (10) @(negedge i_reconfig_clk);
        avmm_read ({inst_offset, 24'h108}, rdata, 32'h0000_0007, 1);
        repeat (10) @(negedge i_reconfig_clk);
        //Write and Read non-exist register
        repeat (10) @(negedge i_reconfig_clk);
        avmm_write({inst_offset, 24'h1f0}, 32'hffff_ffff);
        repeat (10) @(negedge i_reconfig_clk);
        avmm_read ({inst_offset, 24'h1f0}, rdata, 32'h0000_0000, 1);
        repeat (10) @(negedge i_reconfig_clk);

        rdata1 = $random;
        avmm_write({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h04}, rdata1);
        avmm_read ({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h04}, rdata, rdata1, 1);
        rdata2 = $random;
        avmm_write({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h08}, rdata2);
        avmm_read ({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h08}, rdata, rdata2, 1);

        avmm_read ({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h80}, rdata, 32'hdeadc0de, 1);
        avmm_write({inst_offset, PKT_CLIENT_BASE_ADDR + 8'h80}, 32'hff);
        avmm_read ({inst_offset, 24'h30_0000 + 8'h80}, rdata, 32'hdeadc0de, 1); 

        //Write and Read non-exist PTP register
        repeat (10) @(negedge i_reconfig_clk);
        avmm_write({inst_offset, 24'haf0}, 32'habcd_0123);
        repeat (10) @(negedge i_reconfig_clk);
        avmm_read({inst_offset, 24'haf0}, rdata, 32'h0000_0000, 1);
        repeat (10) @(negedge i_reconfig_clk);
       //Read and write MAC registers in F-tile
        avmm_read ({inst_offset, ehip_base_addr + 24'h214}, rdata, 32'h2233_4455, 1); // source address low
        repeat (10) @(negedge i_reconfig_clk);
        avmm_read ({inst_offset, ehip_base_addr + 24'h218}, rdata, 32'h0000_0011, 1); // source address high
        repeat (10) @(negedge i_reconfig_clk);
        avmm_read ({inst_offset, ehip_base_addr + 24'h21C}, rdata, 32'd1518, 1); // rx size config
        repeat (10) @(negedge i_reconfig_clk);
        avmm_write({inst_offset, ehip_base_addr + 24'h214}, 32'h0123_4567);
        repeat (10) @(negedge i_reconfig_clk);
        avmm_write({inst_offset, ehip_base_addr + 24'h218}, 32'h000_89ab);
        repeat (10) @(negedge i_reconfig_clk);
        avmm_read ({inst_offset, ehip_base_addr + 24'h214}, rdata, 32'h0123_4567, 1); // source address low
        repeat (10) @(negedge i_reconfig_clk);
        avmm_read ({inst_offset, ehip_base_addr + 24'h218}, rdata, 32'h0000_89ab, 1); // source address high
        repeat (10) @(negedge i_reconfig_clk);
    end
endtask

task avmm2_test;
    input [7:0] inst_offset;
begin
        $display ("%0t  Try to access AVMM2 begin...", $time());
/*
         Both FHT and FGT avmm_cfgcsr 0xF_0000 - 0xF_FFFF
         avmm2_addr = 28'h100_0000+word addr;
         xcvr in the same quad read/write the same register address space
        *******************************************************
         register offset 0x0010 ux_q_dl_ctrl_a_l0[17:0] cfg_rx_lat_bit_for_async
        'hF_0010='d983056 -> devide by 4 -> 'd245764=18'h3c004
         register offset 0x0018 ux_q_dl_ctrl_a_l1[17:0] cfg_rx_lat_bit_for_async
        'hF_0018='d983064 -> devide by 4 -> 'd245766=18'h3c006
         register offset 0x0020 ux_q_dl_ctrl_a_l2[17:0] cfg_rx_lat_bit_for_async
        'hF_0020='d983072 -> devide by 4 -> 'd245768=18'h3c008
         register offset 0x0028 ux_q_dl_ctrl_a_l3[17:0] cfg_rx_lat_bit_for_async
        'hF_0028='d983080 -> devide by 4 -> 'd245770=18'h3c00a
        *******************************************************
         register offset 0x0028 barak_rxdl_async_l0[17:0] cfg_rx_lat_bit_for_async_lane0
        'hF_0020='d983072 -> devide by 4 -> 'd245768=18'h3c008
         register offset 0x0028 barak_rxdl_async_l1 [17:0] cfg_rx_lat_bit_for_async_lane1
        'hF_0024='d983076 -> devide by 4 -> 'd245769=18'h3c009
         register offset 0x0028 barak_rxdl_async_l2 [17:0] cfg_rx_lat_bit_for_async_lane2
        'hF_0028='d983080 -> devide by 4 -> 'd245770=18'h3c00a
         register offset 0x0028 barak_rxdl_async_l3 [17:0] cfg_rx_lat_bit_for_async_lane3
        'hF_002C='d983084 -> devide by 4 -> 'd245771=18'h3c00b
        *******************************************************
        'hF_FFFC='d1048572 -> devide by 4 -> 'd262143=18'h3ffff

*/
    //FGT xcvr

        //------------------------
        $display ("%0t  Try to access AVMM2 begin...", $time());
        xcvr_select= 4'(0);
        avmm2_addr = {inst_offset, XCVR_BASE_ADDR[23:20] + xcvr_select, 20'hF_0010 + 4'h0};
        avmm2_data = {15'h0, 17'd101 + 4'h0}; //'d101='h65

        $display ("%0t  write 0x%X to xcvr %d address 0x%X", $time(), avmm2_data, xcvr_select, avmm2_addr);
        avmm_write(avmm2_addr, avmm2_data);
        $display ("%0t  Try to access AVMM2 end...", $time());
        repeat (10) @(negedge i_reconfig_clk);

        $display ("%0t  read from address 0x%X", $time(), avmm2_addr);
        avmm_read(avmm2_addr, rdata, avmm2_data, 1);
        $display ("%0t  Try to access AVMM2 end...", $time());
        repeat (10) @(negedge i_reconfig_clk);


        //------------------------
        $display ("%0t  Try to access AVMM2 begin...", $time());
        xcvr_select= 4'(1);
        avmm2_addr = {inst_offset, XCVR_BASE_ADDR[23:20] + xcvr_select, 20'hF_0010 + 4'h1};
        avmm2_data = {15'h0, 17'd101 + 4'h1}; //'d101='h65

        $display ("%0t  write 0x%X to xcvr %d address 0x%X", $time(), avmm2_data, xcvr_select, avmm2_addr);
        avmm_write(avmm2_addr, avmm2_data);
        $display ("%0t  Try to access AVMM2 end...", $time());
        repeat (10) @(negedge i_reconfig_clk);

        $display ("%0t  read from address 0x%X", $time(), avmm2_addr);
        avmm_read(avmm2_addr, rdata, avmm2_data, 1);
        $display ("%0t  Try to access AVMM2 end...", $time());
        repeat (10) @(negedge i_reconfig_clk);


        //------------------------
        $display ("%0t  Try to access AVMM2 begin...", $time());
        xcvr_select= 4'(2);
        avmm2_addr = {inst_offset, XCVR_BASE_ADDR[23:20] + xcvr_select, 20'hF_0010 + 4'h2};
        avmm2_data = {15'h0, 17'd101 + 4'h2}; //'d101='h65

        $display ("%0t  write 0x%X to xcvr %d address 0x%X", $time(), avmm2_data, xcvr_select, avmm2_addr);
        avmm_write(avmm2_addr, avmm2_data);
        $display ("%0t  Try to access AVMM2 end...", $time());
        repeat (10) @(negedge i_reconfig_clk);

        $display ("%0t  read from address 0x%X", $time(), avmm2_addr);
        avmm_read(avmm2_addr, rdata, avmm2_data, 1);
        $display ("%0t  Try to access AVMM2 end...", $time());
        repeat (10) @(negedge i_reconfig_clk);


        //------------------------
        $display ("%0t  Try to access AVMM2 begin...", $time());
        xcvr_select= 4'(3);
        avmm2_addr = {inst_offset, XCVR_BASE_ADDR[23:20] + xcvr_select, 20'hF_0010 + 4'h3};
        avmm2_data = {15'h0, 17'd101 + 4'h3}; //'d101='h65

        $display ("%0t  write 0x%X to xcvr %d address 0x%X", $time(), avmm2_data, xcvr_select, avmm2_addr);
        avmm_write(avmm2_addr, avmm2_data);
        $display ("%0t  Try to access AVMM2 end...", $time());
        repeat (10) @(negedge i_reconfig_clk);

        $display ("%0t  read from address 0x%X", $time(), avmm2_addr);
        avmm_read(avmm2_addr, rdata, avmm2_data, 1);
        $display ("%0t  Try to access AVMM2 end...", $time());
        repeat (10) @(negedge i_reconfig_clk);


end
endtask

//---------------------------------------------
//---------------------------------------------
task avmm_write;
       input [31:0] addr;
       input [31:0] data;
       begin
           force jtag_write  = 1'b1;
           force jtag_addr   = addr;
           force jtag_wdata  = data;
           force jtag_ben    = 4'hF;
           @(posedge i_reconfig_clk);
           #1;
           //$display("====> writedata = %h \n", jtag_wdata);
           if ( jtag_waitrequest !== 1'b1) begin
               @(posedge i_reconfig_clk);
           end else begin
               while ( jtag_waitrequest !== 1'b0) @(posedge i_reconfig_clk);
           end
           #1;
           force jtag_write = 1'b0;
       end
endtask

task avmm_read;
        input  [31:0]   addr;
        output [31:0]   readdata;
        input  [31:0]   refdout;
        input           refdout_check;
        begin
           force jtag_read   = 1'b1;
           force jtag_addr   = addr;
           @(posedge i_reconfig_clk);
           #1;
           fork
               begin
                   if (jtag_waitrequest !== 1'b1) begin
                       @(posedge i_reconfig_clk);
                   end
                   else begin
                       while (jtag_waitrequest !== 1'b0) @(posedge i_reconfig_clk);
                   end
                   #1;
                   force jtag_read = 1'b0;
               end
               begin
                   while (jtag_rd_vld !== 1'b1) @(negedge i_reconfig_clk);
                   #1;
                   readdata = jtag_rdata[31:0];
                   if(refdout_check) begin
                       if (refdout[31:0]== jtag_rdata[31:0]) begin
                           $display("====>MATCH!  Read addr = %h, ReaddataValid = %h Readdata = %h Expected_Readdata = %h \n", jtag_addr,jtag_rd_vld, jtag_rdata, refdout[31:0]);
                       end else begin
                           $display("====>Error!! MISMATCH!  Read addr = %h, ReaddataValid = %h, Readdata = %h, Expected_Readdata = %h; \n", jtag_addr, jtag_rd_vld, jtag_rdata, refdout[31:0]);
                           $display("Error ERROR \n");
                           #100000;
                           //$finish();
                       end
                   end
               end
           join
       end
endtask


//---------------------------------------------

always begin
    #10us;
    $display("The time now is %0t \n", $time);
end

initial begin
    // Setting timeout to 1500us
    #1500us;
    $display("====>Error!! Timeout \n");
    $finish();

end
initial begin
//Dump entire hierachy    
//$fsdbDumpvars(0,"+all", basic_avl_tb_top); 
end
//------------------------------------------------------
//------------------------------------------------------
//------------------------------------------------------
/*
localparam DATA_BCNT = DATA_WIDTH*8; 
localparam CTRL_BCNT = (CLIENT_IF_TYPE==0) ? (DATA_WIDTH+1) : 2; 

rom_data_generator rom_data_generator ();
defparam rom_data_generator.CLIENT_IF_TYPE    = CLIENT_IF_TYPE;
defparam rom_data_generator.WORDS             = DATA_WIDTH;
defparam rom_data_generator.CTRL_BCNT         = CTRL_BCNT;
//defparam rom_data_generator.ROM_PKT_CNT       = 10;
//defparam rom_data_generator.ROM_PKT_LENGTH    = 64;
*/
//------------------------------------------------------

endmodule // basic_avl_tb_top

