// (C) 2001-2022 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1 ps / 1 ps
module eth_f_hw_ip_top #(
        parameter CLIENT_IF_TYPE         = 1,        // 0:Segmented; 1:AvST;
        parameter LANE_NUM               = 4,
        parameter DEV_BOARD              = 3,
        parameter WORDS                  = 8,
        parameter WIDTH                  = 64,
        parameter EMPTY_WIDTH            = 6,
        parameter AVMM_ADDR_WIDTH        = 24,
        parameter AVMM_ETH_START_ADDR    = 24'h00_0000,
        parameter AVMM_XCVR_START_ADDR   = 24'h80_0000,
        parameter AVMM_STATUS_START_ADDR = 24'h10_0000,
        parameter NUMPORTS               = 1,
        parameter SIM_EMULATE            = 0
    )(
        input  logic                   i_clk_sys,
        input  logic                   i_clk_ref,
        input  logic                   i_reconfig_clk,
        input  logic                   i_reconfig_reset,


        input  logic                   i_clk_tx,
        input  logic                   i_clk_rx,
        output logic                   o_clk_tx_div,
        output logic                   o_clk_pll,
        output logic                   o_clk_rec_div64,
        output logic                   o_clk_rec_div,
        input  logic                   i_tx_rst_n,
        input  logic                   i_rx_rst_n,
        input  logic                   i_rst_n,     
        output logic                   o_rst_ack_n,
        output logic                   o_tx_rst_ack_n,
        output logic                   o_rx_rst_ack_n,

        // IO
        input  logic [LANE_NUM-1:0]    i_rx_serial,
        input  logic [LANE_NUM-1:0]    i_rx_serial_n,
        output logic [LANE_NUM-1:0]    o_tx_serial,
        output logic [LANE_NUM-1:0]    o_tx_serial_n,
        output logic [9:0]             user_io,
        output logic [7:0]             user_led,
 
        // QSFP
        output logic                   qsfp_lowpwr,// LPMode
        output logic                   qsfp_rstn,  // ResetL

        //signals to probe
        output logic                    o_rx_pcs_ready,
        output logic                    o_tx_lanes_stable,
        output logic                    o_cdr_lock,
        output logic                    o_tx_pll_locked,


        // Jtag avmm bus
        input   logic   [AVMM_ADDR_WIDTH-1:0]  i_jtag_address,
        input   logic                          i_jtag_read,
        input   logic                          i_jtag_write,
        input   logic   [31:0]                 i_jtag_writedata,
        input   logic   [3:0]                  i_jtag_byteenable,
        output  logic   [31:0]                 o_jtag_readdata,
        output  logic                          o_jtag_readdatavalid,
        output  logic                          o_jtag_waitrequest
);

localparam PKT_ROM_INIT_FILE = (1 == 0 & 0 == 0) ? "../hardware_test_design/common/eth_f_hw_pkt_gen_rom_init.100G_SEG.hex"      : 
                               (1 == 1 & 0 == 0) ? "../hardware_test_design/common/eth_f_hw_pkt_gen_rom_init.100G_AVST.hex"     : 
                               (1 == 0 & 0 == 1) ? "../hardware_test_design/common/eth_f_hw_pkt_gen_rom_init.100G_SEG_PTP.hex"  : 
                               (1 == 1 & 0 == 1) ? "../hardware_test_design/common/eth_f_hw_pkt_gen_rom_init.100G_AVST_PTP.hex" : 
                               "../hardware_test_design/common/eth_f_hw_pkt_gen_rom_init.hex";

localparam PKT_ROM_INIT_DATA = "../hardware_test_design/common/init_file_data.100G.hex" ;
 
localparam PKT_ROM_INIT_DATA_B = ("100G" == "400G") ? "../hardware_test_design/common/init_file_data_b.400G.hex": "../hardware_test_design/common/init_file_data_b.hex" ;
localparam PKT_ROM_INIT_CTL = "../hardware_test_design/common/init_file_ctrl.100G.hex";

//---Less entries for higher speed which has larger data width;
localparam LPBK_FIFO_ADDR_WIDTH = ("100G" == "10G")  ? 10 :
                                  ("100G" == "25G")  ? 10 :
                                  ("100G" == "40G")  ? 10 :
                                  ("100G" == "50G")  ? 10 :
                                  ("100G" == "100G") ? 8 :
                                  ("100G" == "200G") ? 8 :
                                  ("100G" == "400G") ? 8 : 8;

`ifdef ALTERA_RESERVED_QIS
localparam AM_INS_PERIOD        = ("100G" == "25G")                               ? 81920 :
                                  ("100G" == "40G")                               ? 32768 :
                                  ("100G" == "50G" && "2" == "0")   ? 32768 :
                                  ("100G" == "50G")                               ? 40960 :
                                  ("100G" == "100G" && "2" == "0")  ? 81920 :
                                  ("100G" == "100G")                              ? 81920 :
                                  ("100G" == "200G" || "100G" == "400G")    ? 40960 : 40960;
`else
localparam AM_INS_PERIOD        = ("100G" == "25G")                               ? 1280 :
                                  ("100G" == "40G")                               ? 128 :
                                  ("100G" == "50G" && "2" == "0")   ? 512 :
                                  ("100G" == "50G")                               ? 640 :
                                  ("100G" == "100G")                              ? 1280 :
                                  ("100G" == "200G" || "100G" == "400G")    ? 640 : 1280;

`endif


localparam AM_INS_CYC         = ("100G" == "25G")                               ? 4 :
                                  ("100G" == "40G")                               ? 2 :
                                  ("100G" == "50G" && "2" == "0")   ? 2 :
                                  ("100G" == "50G")                               ? 2 :
                                  ("100G" == "100G" && "2" == "0")  ? 5 :
                                  ("100G" == "100G")                              ? 5 :
                                  ("100G" == "200G" || "100G" == "400G")    ? 2 : 4;

localparam FEC_TYPE	     = 2;
localparam ERATE	     = "100G";
//---------------------------------------------------------------

//Temp for warning clean up
assign user_io='0;
assign user_led='0;
assign qsfp_lowpwr='0;

    assign qsfp_rstn='1;

//---------------------------------------------------------------
    // Reconfig Interface
    wire                          i_reconfig_eth_write;
    wire                          i_reconfig_eth_read;
    wire     [15:0]               i_reconfig_eth_addr;
    wire     [31:0]               i_reconfig_eth_writedata;
    wire     [3:0]                i_reconfig_eth_byteenable;
    wire     [31:0]               o_reconfig_eth_readdata;
    wire                          o_reconfig_eth_readdata_valid;
    wire                          o_reconfig_eth_waitrequest;

    wire     [LANE_NUM-1:0]       i_reconfig_xcvr_write;
    wire     [LANE_NUM-1:0]       i_reconfig_xcvr_read;
    wire     [LANE_NUM*20-1:0]    i_reconfig_xcvr_addr;
    wire     [LANE_NUM*32-1:0]    i_reconfig_xcvr_writedata;
    wire     [LANE_NUM*4-1:0]     i_reconfig_xcvr_byteenable;
    wire     [LANE_NUM*32-1:0]    o_reconfig_xcvr_readdata;
    wire     [LANE_NUM-1:0]       o_reconfig_xcvr_readdata_valid;
    wire     [LANE_NUM-1:0]       o_reconfig_xcvr_waitrequest;

    wire     [22:0]               i_status_addr;
    wire                          i_status_read;
    wire                          i_status_write;
    wire     [31:0]               i_status_writedata;
    wire     [3:0]                i_status_byteenable;
    wire     [31:0]               o_status_readdata;
    wire                          o_status_readdata_valid;
    wire                          o_status_waitrequest;

//---------------------------------------------------------------

    //  Dummy TX Segmented Interface
    wire  [64*8-1:0]    i_tx_mac_data;
    wire                          i_tx_mac_valid;
    wire  [8-1:0]        i_tx_mac_inframe;
    wire  [8*3-1:0]    i_tx_mac_eop_empty;
    wire                          o_tx_mac_ready;

    wire  [8-1:0]      i_tx_mac_error;
    wire  [8-1:0]        i_tx_mac_skip_crc;
    //Dummy RX Segmented Interface
    wire  [64*8-1:0]    o_rx_mac_data;
    wire                          o_rx_mac_valid;
    wire  [8-1:0]        o_rx_mac_inframe;
    wire  [8*3-1:0]    o_rx_mac_eop_empty;
    wire  [2*8-1:0]      o_rx_mac_error;
    wire [8-1:0]       o_rx_mac_fcs_error;
    wire [8*3-1:0]        o_rx_mac_status;

    // TX AVST INTERFACE
    wire  [64*8-1:0]    i_tx_data;
    wire                          i_tx_valid;
    wire                          i_tx_startofpacket;
    wire                          i_tx_endofpacket;
    wire                          o_tx_ready;

    wire  [6-1:0]       i_tx_empty;
    wire                          i_tx_error;
    wire                          i_tx_skip_crc;

    // RX AVST INTERFACE
    wire    [64*8-1:0]  o_rx_data;
    wire                          o_rx_valid;
    wire                          o_rx_startofpacket;
    wire                          o_rx_endofpacket;
    wire    [6-1:0]     o_rx_empty;
    wire    [6-1:0]               o_rx_error;
    wire    [39:0]                o_rxstatus_data;
    wire                          o_rxstatus_valid;

    wire    [63:0]      i_tx_preamble; 
    wire    [63:0]      o_rx_preamble;
 

//---------------------------------------------------------------
    //---dummy Segmented interface input signals for Avst mode---
    assign o_tx_mac_ready      = 1'b0;
    assign o_rx_mac_data       = {64*8{1'b0}};
    assign o_rx_mac_valid      = 1'b0;
    assign o_rx_mac_inframe    = {8{1'b0}};
    assign o_rx_mac_eop_empty  = {8*3{1'b0}};
    assign o_rx_mac_error      = {8*2{1'b0}};
    assign o_rx_mac_fcs_error  = {8{1'b0}};
    assign o_rx_mac_status     = {8*3{1'b0}};


        assign o_rx_preamble = 64'hFB55_5555_5555_55D5;

//---------------------------------------------------------------
    localparam PKT_CYL = 1;
    localparam PTP_FP_WIDTH = 8;
//---------------------------------------------------------------
    //Flow control Interface
    wire                         i_tx_pause = 1'b0;
    wire    [7:0]                i_tx_pfc   = 8'd0;
    wire                         o_rx_pause;
    wire    [7:0]                o_rx_pfc;

    //Status signals
    wire                         i_stats_snapshot            = 1'b0;
    wire                         o_rx_hi_ber;
    wire                         o_rx_block_lock;
    wire                         o_rx_am_lock;

    wire                         o_local_fault_status;
    wire                         o_remote_fault_status;
    wire 			 o_sys_pll_locked;


//---------------------------------------------------------------
ex_100G dut (
    .i_clk_rx                         (i_clk_rx),
    .i_clk_tx                         (i_clk_tx),
    .o_clk_pll                        (o_clk_pll), 
    .o_clk_tx_div                     (o_clk_tx_div                  ), 
    .o_clk_rec_div64                  (o_clk_rec_div64), 
    .o_clk_rec_div                    (o_clk_rec_div                 ),
    .i_rx_rst_n                       (i_rx_rst_n),
    .i_tx_rst_n                       (i_tx_rst_n),
    .i_rst_n                          (i_rst_n                       ),
    .o_rst_ack_n                      (o_rst_ack_n                   ),
    .o_rx_rst_ack_n                   (o_rx_rst_ack_n                ),
    .o_tx_rst_ack_n                   (o_tx_rst_ack_n                ),
    .i_reconfig_clk                   (i_reconfig_clk),
    .i_reconfig_reset                 (i_reconfig_reset),
    .o_cdr_lock                       (o_cdr_lock                    ), 
    .o_tx_pll_locked                  (o_tx_pll_locked               ),  
    .o_tx_lanes_stable                (o_tx_lanes_stable             ),
    .o_sys_pll_locked                 (o_sys_pll_locked              ),  
    .o_rx_pcs_ready                   (o_rx_pcs_ready                ),
    .i_reconfig_eth_addr              (i_reconfig_eth_addr[15:2]     ),
    .i_reconfig_eth_byteenable        (i_reconfig_eth_byteenable     ),
    .o_reconfig_eth_readdata_valid    (o_reconfig_eth_readdata_valid ),
    .i_reconfig_eth_read              (i_reconfig_eth_read),
    .i_reconfig_eth_write             (i_reconfig_eth_write),
    .o_reconfig_eth_readdata          (o_reconfig_eth_readdata       ),
    .i_reconfig_eth_writedata         (i_reconfig_eth_writedata),
    .o_reconfig_eth_waitrequest       (o_reconfig_eth_waitrequest),
    .i_reconfig_xcvr0_addr            (i_reconfig_xcvr_addr[(0*20)+19:(0*20)+2]),
    .i_reconfig_xcvr0_read            (i_reconfig_xcvr_read[0]),
    .i_reconfig_xcvr0_write           (i_reconfig_xcvr_write[0]),
    .i_reconfig_xcvr0_writedata       (i_reconfig_xcvr_writedata[0*32+:32]),
    .i_reconfig_xcvr0_byteenable      (i_reconfig_xcvr_byteenable[0*4+:4]),
    .o_reconfig_xcvr0_readdata        (o_reconfig_xcvr_readdata[0*32+:32]),
    .o_reconfig_xcvr0_readdata_valid  (o_reconfig_xcvr_readdata_valid[0]),
    .o_reconfig_xcvr0_waitrequest     (o_reconfig_xcvr_waitrequest[0]),
    .i_reconfig_xcvr1_addr            (i_reconfig_xcvr_addr[(1*20)+19:(1*20)+2]),
    .i_reconfig_xcvr1_read            (i_reconfig_xcvr_read[1]),
    .i_reconfig_xcvr1_write           (i_reconfig_xcvr_write[1]),
    .i_reconfig_xcvr1_writedata       (i_reconfig_xcvr_writedata[1*32+:32]),
    .i_reconfig_xcvr1_byteenable      (i_reconfig_xcvr_byteenable[1*4+:4]),
    .o_reconfig_xcvr1_readdata        (o_reconfig_xcvr_readdata[1*32+:32]),
    .o_reconfig_xcvr1_readdata_valid  (o_reconfig_xcvr_readdata_valid[1]),
    .o_reconfig_xcvr1_waitrequest     (o_reconfig_xcvr_waitrequest[1]),
    .i_reconfig_xcvr2_addr            (i_reconfig_xcvr_addr[(2*20)+19:(2*20)+2]),
    .i_reconfig_xcvr2_read            (i_reconfig_xcvr_read[2]),
    .i_reconfig_xcvr2_write           (i_reconfig_xcvr_write[2]),
    .i_reconfig_xcvr2_writedata       (i_reconfig_xcvr_writedata[2*32+:32]),
    .i_reconfig_xcvr2_byteenable      (i_reconfig_xcvr_byteenable[2*4+:4]),
    .o_reconfig_xcvr2_readdata        (o_reconfig_xcvr_readdata[2*32+:32]),
    .o_reconfig_xcvr2_readdata_valid  (o_reconfig_xcvr_readdata_valid[2]),
    .o_reconfig_xcvr2_waitrequest     (o_reconfig_xcvr_waitrequest[2]),
    .i_reconfig_xcvr3_addr            (i_reconfig_xcvr_addr[(3*20)+19:(3*20)+2]),
    .i_reconfig_xcvr3_read            (i_reconfig_xcvr_read[3]),
    .i_reconfig_xcvr3_write           (i_reconfig_xcvr_write[3]),
    .i_reconfig_xcvr3_writedata       (i_reconfig_xcvr_writedata[3*32+:32]),
    .i_reconfig_xcvr3_byteenable      (i_reconfig_xcvr_byteenable[3*4+:4]),
    .o_reconfig_xcvr3_readdata        (o_reconfig_xcvr_readdata[3*32+:32]),
    .o_reconfig_xcvr3_readdata_valid  (o_reconfig_xcvr_readdata_valid[3]),
    .o_reconfig_xcvr3_waitrequest     (o_reconfig_xcvr_waitrequest[3]),
    .o_rx_block_lock                  (o_rx_block_lock       ),        
    .o_rx_am_lock                     (o_rx_am_lock          ),        
    .o_local_fault_status             (o_local_fault_status  ),   
    .o_remote_fault_status            (o_remote_fault_status ),      
    .o_rx_hi_ber                      (o_rx_hi_ber           ),        
    .o_rx_pcs_fully_aligned           (o_rx_pcs_fully_aligned),

    .i_clk_sys                        (i_clk_sys),  //Need to connect with sys_pll IP
    .i_clk_ref                        (i_clk_ref),
    .i_rx_serial                      (i_rx_serial  ),
    .i_rx_serial_n                    (i_rx_serial_n), 
    .o_tx_serial                      (o_tx_serial  ), 
    .o_tx_serial_n                    (o_tx_serial_n), 
    .i_tx_pause                       (i_tx_pause),
    .i_tx_pfc                         (i_tx_pfc),
    .o_rx_pfc                         (o_rx_pfc),
    .o_rx_pause                       (o_rx_pause),
    .i_tx_data                        (i_tx_data),
    .i_tx_valid                       (i_tx_valid),
    .i_tx_startofpacket               (i_tx_startofpacket), 
    .i_tx_endofpacket                 (i_tx_endofpacket),
    .i_tx_error                       (i_tx_error                    ),
    .o_tx_ready                       (o_tx_ready),
    .i_tx_empty                       (i_tx_empty),
    .i_tx_skip_crc                    (i_tx_skip_crc), 
    .o_rx_data                        (o_rx_data),
    .o_rx_valid                       (o_rx_valid),
    .o_rx_startofpacket               (o_rx_startofpacket),
    .o_rx_endofpacket                 (o_rx_endofpacket),
    .o_rx_empty                       (o_rx_empty),
    .o_rx_error                       (o_rx_error),
    .o_rxstatus_valid                 (o_rxstatus_valid),
    .o_rxstatus_data                  (o_rxstatus_data),
   
    .i_stats_snapshot                 (i_stats_snapshot)
);

//---------------------------------------------------------------
eth_f_hw_ip_avmm_decoder ip_avmm_decoder (
        .i_reconfig_clk           (i_reconfig_clk),
        .i_reconfig_reset         (i_reconfig_reset),

        //---avmm master from Jtag---
        .i_jtag_address           (i_jtag_address),
        .i_jtag_read              (i_jtag_read),
        .i_jtag_write             (i_jtag_write),
        .i_jtag_writedata         (i_jtag_writedata),
        .i_jtag_byteenable        (i_jtag_byteenable),
        .o_jtag_readdata          (o_jtag_readdata),
        .o_jtag_readdatavalid     (o_jtag_readdatavalid),
        .o_jtag_waitrequest       (o_jtag_waitrequest),

        //---eth reconfig if---
        .o_reconfig_eth_addr              (i_reconfig_eth_addr          ),
        .o_reconfig_eth_read              (i_reconfig_eth_read          ),
        .o_reconfig_eth_write             (i_reconfig_eth_write         ),
        .o_reconfig_eth_writedata         (i_reconfig_eth_writedata     ),
        .o_reconfig_eth_byteenable        (i_reconfig_eth_byteenable    ),
        .i_reconfig_eth_readdata          (o_reconfig_eth_readdata      ),
        .i_reconfig_eth_readdata_valid    (o_reconfig_eth_readdata_valid),
        .i_reconfig_eth_waitrequest       (o_reconfig_eth_waitrequest   ),

        //---eth reconfig if---

        .o_reconfig_xcvr_addr              (i_reconfig_xcvr_addr          ),
        .o_reconfig_xcvr_read              (i_reconfig_xcvr_read          ),
        .o_reconfig_xcvr_write             (i_reconfig_xcvr_write         ),
        .o_reconfig_xcvr_writedata         (i_reconfig_xcvr_writedata     ),
        .o_reconfig_xcvr_byteenable        (i_reconfig_xcvr_byteenable    ),
        .i_reconfig_xcvr_readdata          (o_reconfig_xcvr_readdata      ),
        .i_reconfig_xcvr_readdata_valid    (o_reconfig_xcvr_readdata_valid),
        .i_reconfig_xcvr_waitrequest       (o_reconfig_xcvr_waitrequest   ),

        //---eth reconfig if---
        .o_status_addr              (i_status_addr          ),
        .o_status_read              (i_status_read          ),
        .o_status_write             (i_status_write         ),
        .o_status_writedata         (i_status_writedata     ),
        .o_status_byteenable        (i_status_byteenable    ),
        .i_status_readdata          (o_status_readdata      ),
        .i_status_readdata_valid    (o_status_readdata_valid),
        .i_status_waitrequest       (o_status_waitrequest   ),

        //---Ctrl IF---
        .i_eth_start_addr           (AVMM_ETH_START_ADDR),
        .i_xcvr_start_addr          (AVMM_XCVR_START_ADDR),
        .i_status_start_addr        (AVMM_STATUS_START_ADDR)
);	
defparam    ip_avmm_decoder.LANE_NUM           = 4;
defparam    ip_avmm_decoder.AVMM_ADDR_WIDTH    = 24;
defparam    ip_avmm_decoder.ETH_ADDR_WIDTH     = 16;
defparam    ip_avmm_decoder.XCVR_ADDR_WIDTH    = 20;
defparam    ip_avmm_decoder.STATUS_ADDR_WIDTH  = 23;

//---------------------------------------------------------------

logic tx_pll_locked_reconfig_sync;
logic rst_n_reconfig_sync;
logic pkt_client_rst_n;

eth_f_altera_std_synchronizer_nocut tx_pll_locked_reconfig_sync_inst (
    .clk        (i_reconfig_clk),
    .reset_n    (1'b1),
    .din        (o_tx_pll_locked),
    .dout       (tx_pll_locked_reconfig_sync)
);

eth_f_altera_std_synchronizer_nocut i_rst_reconfig_sync_inst (
    .clk        (i_reconfig_clk),
    .reset_n    (1'b1),
    .din        (i_rst_n),
    .dout       (rst_n_reconfig_sync)
);

always @(posedge i_reconfig_clk) begin
      pkt_client_rst_n <= tx_pll_locked_reconfig_sync & rst_n_reconfig_sync;
end

eth_f_altera_std_synchronizer_nocut pkt_client_rst_n_sync_inst (
    .clk        (i_clk_tx),
    .reset_n    (1'b1),
    .din        (pkt_client_rst_n),
    .dout       (pkt_client_rst_n_sync)
);

eth_f_packet_client_top packet_client_top (
        .i_arst                       (!pkt_client_rst_n_sync),
        .i_clk_rx                     (i_clk_rx),
        .i_clk_tx                     (i_clk_tx),
        .i_clk_pll		      (o_clk_pll),
        .i_clk_status                 (i_reconfig_clk),
        .i_clk_status_rst             (i_reconfig_reset),

        .i_tx_mac_ready               (o_tx_mac_ready),
        .o_tx_mac_valid               (i_tx_mac_valid),
        .o_tx_mac_inframe             (i_tx_mac_inframe),
        .o_tx_mac_eop_empty           (i_tx_mac_eop_empty),
        .o_tx_mac_data                (i_tx_mac_data),
        .o_tx_mac_error               (i_tx_mac_error),
        .o_tx_mac_skip_crc            (i_tx_mac_skip_crc),

        .i_rx_mac_valid               (o_rx_mac_valid),
        .i_rx_mac_inframe             (o_rx_mac_inframe),
        .i_rx_mac_eop_empty           (o_rx_mac_eop_empty),
        .i_rx_mac_data                (o_rx_mac_data),
        .i_rx_mac_error               (o_rx_mac_error),
        .i_rx_mac_fcs_error           (o_rx_mac_fcs_error),
        .i_rx_mac_status              (o_rx_mac_status),

        .i_tx_ready                   (o_tx_ready),
        .o_tx_valid                   (i_tx_valid),
        .o_tx_sop                     (i_tx_startofpacket), 
        .o_tx_eop                     (i_tx_endofpacket),
        .o_tx_empty                   (i_tx_empty),
        .o_tx_data                    (i_tx_data),
        .o_tx_error                   (i_tx_error),
        .o_tx_skip_crc                (i_tx_skip_crc), 

        .i_rx_valid                   (o_rx_valid),
        .i_rx_sop                     (o_rx_startofpacket),
        .i_rx_eop                     (o_rx_endofpacket),
        .i_rx_empty                   (o_rx_empty),
        .i_rx_data                    (o_rx_data),
        .i_rx_error                   (o_rx_error),
        .i_rxstatus_valid             (o_rxstatus_valid),
        .i_rxstatus_data              (o_rxstatus_data),

        .i_rx_preamble                (o_rx_preamble), 
        .o_tx_preamble                (i_tx_preamble),

        .i_status_addr                (i_status_addr),
        .i_status_read                (i_status_read),
        .i_status_write               (i_status_write),
        .i_status_writedata           (i_status_writedata),
        .o_status_readdata            (o_status_readdata),
        .o_status_readdata_valid      (o_status_readdata_valid),
        .o_status_waitrequest         (o_status_waitrequest)
   );

reg sys_pll_lock_inv;
//wire o_sys_pll_locked_r; 
//reg clkdiv2_pll_lock_reg1;


//assign sys_pll_lock_inv = !o_sys_pll_locked;
always @ (posedge o_clk_pll) begin:REGISTER_SYS_PLL_LOCK
  sys_pll_lock_inv <= !o_sys_pll_locked;
end

 

defparam    packet_client_top.ENABLE_PTP           = 0;
defparam    packet_client_top.PKT_CYL       	   = PKT_CYL;
defparam    packet_client_top.PTP_FP_WIDTH         = PTP_FP_WIDTH;
defparam    packet_client_top.CLIENT_IF_TYPE       = CLIENT_IF_TYPE;
defparam    packet_client_top.READY_LATENCY        = 0;
defparam    packet_client_top.WORDS                = 8;
defparam    packet_client_top.EMPTY_WIDTH          = 6;
defparam    packet_client_top.LPBK_FIFO_ADDR_WIDTH = LPBK_FIFO_ADDR_WIDTH;
defparam    packet_client_top.PKT_ROM_INIT_FILE    = PKT_ROM_INIT_FILE;


//---------------------------------------------------------------
//---------------------------------------------------------------

//---------------------------------------------------------------
endmodule


