// (C) 2001-2022 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1 ps / 1 ps

module eth_f_hw # (
    parameter INSTANCE_NUM = 1,
    parameter EHIP_RATE = "100G"
) (
// ---- Ports for IP_TC = 0 begin
        input  wire                  i_refclk2pll,
`ifndef ALTERA_RESERVED_QIS // The code that is not used for Quartus Synthesis 
        input  wire                  i_reconfig_reset,
        input  wire                  cpu_resetn,

        input  wire [INSTANCE_NUM-1:0]  i_rst_n,
        input  wire [INSTANCE_NUM-1:0]  i_tx_rst_n,
        input  wire [INSTANCE_NUM-1:0]  i_rx_rst_n,
        output wire [INSTANCE_NUM-1:0]  o_rst_ack_n,
        output wire [INSTANCE_NUM-1:0]  o_tx_rst_ack_n,
        output wire [INSTANCE_NUM-1:0]  o_rx_rst_ack_n,
`endif

     	input  wire                  i_reconfig_clk,
     	input  wire                  clk50,
        //---IO---
        input  wire [INSTANCE_NUM*4-1:0]  i_rx_serial,
        input  wire [INSTANCE_NUM*4-1:0]  i_rx_serial_n,
        output wire [INSTANCE_NUM*4-1:0]  o_tx_serial,
        output wire [INSTANCE_NUM*4-1:0]  o_tx_serial_n,
        output wire [9:0]            user_io,
        output wire [7:0]            user_led,

	//workaround for bcm


        //---QSFP---
        output wire                  qsfp_lowpwr,   // LPMode
        output wire                  qsfp_rstn,      // ResetL
		  
      ///////// SI5340A0 /////////
      inout              SI5340A0_I2C_SCL,
      inout              SI5340A0_I2C_SDA,
      output             SI5340A0_OE_n,
      output             SI5340A0_RST_n,

      ///////// SI5340A1 /////////
      inout              SI5340A1_I2C_SCL,
      inout              SI5340A1_I2C_SDA,
      output             SI5340A1_OE_n,
      output             SI5340A1_RST_n		  
);

//=======================================================
//  Enable SI5340A. use default setting
//=======================================================

assign SI5340A0_RST_n = 1'b1;
assign SI5340A0_OE_n = 1'b0;

assign SI5340A1_RST_n = 1'b1;
assign SI5340A1_OE_n = 1'b0;

//---------------------------------------------------------------
// Clocks
//---------------------------------------------------------------
    wire [INSTANCE_NUM-1:0] o_clk_pll;
    wire [INSTANCE_NUM-1:0] o_clk_tx_div;
    wire [INSTANCE_NUM-1:0] o_clk_rec_div64;
    wire [INSTANCE_NUM-1:0] o_clk_rec_div;
    
    // Clock assignment for TEST_SIP=0
    wire [INSTANCE_NUM-1:0] i_clk_tx;
    wire [INSTANCE_NUM-1:0] i_clk_rx;


wire [13:0]     i_kr_reconfig_addr;
wire            i_kr_reconfig_read;
wire            i_kr_reconfig_write;
wire [3:0]      i_kr_reconfig_byte_en;
wire [31:0]     i_kr_reconfig_writedata;
wire [31:0]     o_kr_reconfig_readdata;
wire            o_kr_reconfig_readdata_valid;
wire            o_kr_reconfig_waitrequest;

    genvar inst_i;
    generate
        for (inst_i = 0; inst_i < INSTANCE_NUM; inst_i = inst_i + 1) begin: clks
            assign i_clk_rx     [inst_i] = o_clk_pll[0]   ;
            assign i_clk_tx     [inst_i] = o_clk_pll[0]   ;

        end
    endgenerate

//---------------------------------------------------------------
// jtag_avalon and avmm_decoder
//---------------------------------------------------------------
wire     [31:0]               master_address;
wire                          master_read;
wire                          master_write;
wire     [31:0]               master_writedata;
wire     [3:0]                master_byteenable;
wire     [31:0]               master_readdata;
wire                          master_readdatavalid;
wire                          master_waitrequest;
// ETH_F AVMM signals
wire   [INSTANCE_NUM*24-1:0]  reconfig_hw_ip_addr;
wire   [INSTANCE_NUM-1:0]     reconfig_hw_ip_read;
wire   [INSTANCE_NUM-1:0]     reconfig_hw_ip_write;
wire   [INSTANCE_NUM*32-1:0]  reconfig_hw_ip_writedata;
wire   [INSTANCE_NUM*4-1:0]   reconfig_hw_ip_byteenable;
wire   [INSTANCE_NUM*32-1:0]  reconfig_hw_ip_readdata;
wire   [INSTANCE_NUM-1:0]     reconfig_hw_ip_readdata_valid;
wire   [INSTANCE_NUM-1:0]     reconfig_hw_ip_waitrequest;
// PTP_ADPT_F AVMM signals
wire   [11:0]                 reconfig_p2p_addr;
wire   [3:0]                  reconfig_p2p_byteenable;
wire                          reconfig_p2p_write;
wire   [31:0]                 reconfig_p2p_writedata;
wire                          reconfig_p2p_read;
wire   [31:0]                 reconfig_p2p_readdata;
wire                          reconfig_p2p_readdata_valid;
wire                          reconfig_p2p_waitrequest;
wire   [11:0]                 reconfig_asym_addr;
wire   [3:0]                  reconfig_asym_byteenable;
wire                          reconfig_asym_write;
wire   [31:0]                 reconfig_asym_writedata;
wire                          reconfig_asym_read;
wire   [31:0]                 reconfig_asym_readdata;
wire                          reconfig_asym_readdata_valid;
wire                          reconfig_asym_waitrequest;
// PTP_MASTER_TOD AVMM signals
wire                          ptp_master_tod_csr_read;
wire                          ptp_master_tod_csr_write;
wire                          ptp_master_tod_csr_waitrequest;
wire    [31:0]                ptp_master_tod_csr_writedata;
wire    [31:0]                ptp_master_tod_csr_readdata;
wire    [5:0]                 ptp_master_tod_csr_addr;
wire                          ptp_master_tod_csr_readdata_valid;
wire [INSTANCE_NUM-1:0] rst_n;
wire [INSTANCE_NUM-1:0] tx_rst_n;
wire [INSTANCE_NUM-1:0] rx_rst_n;
wire                    reconfig_reset;
wire [INSTANCE_NUM-1:0] tx_lanes_stable;
wire [INSTANCE_NUM-1:0] rx_pcs_ready;
wire [INSTANCE_NUM-1:0] cdr_lock;
wire [INSTANCE_NUM-1:0] tx_pll_locked;

wire reconfig_reset_sync;

eth_f_xcvr_resync_std #(
    .SYNC_CHAIN_LENGTH(3),
    .WIDTH(1),
    .INIT_VALUE(1)
) sync_reconfig_reset (
    .clk    (i_reconfig_clk),
    .reset  (1'b0),
    .d  (reconfig_reset),
    .q  (reconfig_reset_sync)
);

`ifdef ALTERA_RESERVED_QIS // The code that is used for Quartus Synthesis 
    wire source_rst_n;
    wire source_rst_n_sync;
    wire source_tx_rst_n;
    wire source_rx_rst_n;
    wire source_reconfig_reset;
    wire ninit_done;
    wire ninit_done_sync;
    reg  arst;

    eth_f_reset_synchronizer source_rst_sync (
        .clk            (i_reconfig_clk),
        .aclr           (source_rst_n),
        .aclr_sync      (source_rst_n_sync)
    );
    eth_f_reset_synchronizer ninit_done_rst_sync (
        .clk            (i_reconfig_clk),
        .aclr           (ninit_done),
        .aclr_sync      (ninit_done_sync)
    );

    always @(posedge i_reconfig_clk) begin
        arst     <= ~(~source_rst_n_sync | ninit_done_sync);
    end


       wire                  i_reconfig_reset;
    wire                  cpu_resetn;
   
    wire [INSTANCE_NUM-1:0]  i_rst_n;
    wire [INSTANCE_NUM-1:0]  i_tx_rst_n;
    wire [INSTANCE_NUM-1:0]  i_rx_rst_n;
    wire [INSTANCE_NUM-1:0] o_rx_rst_ack_n;
    wire [INSTANCE_NUM-1:0] o_tx_rst_ack_n;
    wire [INSTANCE_NUM-1:0] o_rst_ack_n;

    genvar i;
    generate
        for (i = 0;i < INSTANCE_NUM; i= i + 1) begin: resets
            assign rst_n[i]     = arst;
            assign tx_rst_n[i]  = source_tx_rst_n;
            assign rx_rst_n[i]  = source_rx_rst_n;
        end
    endgenerate
 
    assign reconfig_reset = (source_reconfig_reset | ninit_done_sync); 

    wire [7:0] system_status;
    assign system_status[0] = arst;
    assign system_status[1] = &rx_pcs_ready;
    assign system_status[2] = &tx_lanes_stable;
    assign system_status[3] = &tx_pll_locked;
    assign system_status[4] = &cdr_lock;
    assign system_status[5] = |o_rx_rst_ack_n;
    assign system_status[6] = |o_tx_rst_ack_n;
    assign system_status[7] = |o_rst_ack_n;
    probe8 prb (
        .source  ({source_rst_n, source_tx_rst_n, source_rx_rst_n, source_reconfig_reset}),
        .probe   (system_status)
    );
    reset_ip reset (
        .ninit_done (ninit_done)
    );


eth_f_jtag_avalon jtag_master (
        .clk_clk                  (i_reconfig_clk),
        .clk_reset_reset          (reconfig_reset_sync),
        .master_address           (master_address),
        .master_read              (master_read),
        .master_write             (master_write),
        .master_writedata         (master_writedata),
        .master_byteenable        (master_byteenable),
        .master_readdata          (master_readdata),
        .master_readdatavalid     (master_readdatavalid),
        .master_waitrequest       (master_waitrequest),
        .master_reset_reset       ()
);

`else // The code that is used for other software 
    assign rst_n             = i_rst_n;
    assign tx_rst_n          = i_tx_rst_n;
    assign rx_rst_n          = i_rx_rst_n;
    assign reconfig_reset    = i_reconfig_reset;

`endif

eth_f_hw_top_avmm_decoder hw_top_avmm_decoder (
    .i_reconfig_clk           (i_reconfig_clk),
    .i_reconfig_reset         (reconfig_reset_sync),

    //---avmm master from Jtag---
    .i_jtag_address           (master_address),
    .i_jtag_read              (master_read),
    .i_jtag_write             (master_write),
    .i_jtag_writedata         (master_writedata),
    .i_jtag_byteenable        (master_byteenable),
    .o_jtag_readdata          (master_readdata),
    .o_jtag_readdatavalid     (master_readdatavalid),
    .o_jtag_waitrequest       (master_waitrequest),

    //---p2p reconfig if---
    .o_reconfig_p2p_addr              (reconfig_p2p_addr          ),
    .o_reconfig_p2p_read              (reconfig_p2p_read          ),
    .o_reconfig_p2p_write             (reconfig_p2p_write         ),
    .o_reconfig_p2p_writedata         (reconfig_p2p_writedata     ),
    .o_reconfig_p2p_byteenable        (reconfig_p2p_byteenable    ),
    .i_reconfig_p2p_readdata          (reconfig_p2p_readdata      ),
    .i_reconfig_p2p_readdata_valid    (reconfig_p2p_readdata_valid),
    .i_reconfig_p2p_waitrequest       (reconfig_p2p_waitrequest   ),

    //---asym reconfig if---
    .o_reconfig_asym_addr              (reconfig_asym_addr          ),
    .o_reconfig_asym_read              (reconfig_asym_read          ),
    .o_reconfig_asym_write             (reconfig_asym_write         ),
    .o_reconfig_asym_writedata         (reconfig_asym_writedata     ),
    .o_reconfig_asym_byteenable        (reconfig_asym_byteenable    ),
    .i_reconfig_asym_readdata          (reconfig_asym_readdata      ),
    .i_reconfig_asym_readdata_valid    (reconfig_asym_readdata_valid),
    .i_reconfig_asym_waitrequest       (reconfig_asym_waitrequest   ),

    //---hw_ip reconfig if---

    .o_reconfig_hw_ip_addr              (reconfig_hw_ip_addr          ),
    .o_reconfig_hw_ip_read              (reconfig_hw_ip_read          ),
    .o_reconfig_hw_ip_write             (reconfig_hw_ip_write         ),
    .o_reconfig_hw_ip_writedata         (reconfig_hw_ip_writedata     ),
    .o_reconfig_hw_ip_byteenable        (reconfig_hw_ip_byteenable    ),
    .i_reconfig_hw_ip_readdata          (reconfig_hw_ip_readdata      ),
    .i_reconfig_hw_ip_readdata_valid    (reconfig_hw_ip_readdata_valid),
    .i_reconfig_hw_ip_waitrequest       (reconfig_hw_ip_waitrequest   ),
    
    //---hw_ip reconfig if---

    .o_reconfig_kr_ip_addr              (i_kr_reconfig_addr           ),
    .o_reconfig_kr_ip_read              (i_kr_reconfig_read           ),
    .o_reconfig_kr_ip_write             (i_kr_reconfig_write          ),
    .o_reconfig_kr_ip_writedata         (i_kr_reconfig_writedata      ),
    .o_reconfig_kr_ip_byteenable        (i_kr_reconfig_byte_en        ),
    .i_reconfig_kr_ip_readdata          (o_kr_reconfig_readdata       ),
    .i_reconfig_kr_ip_readdata_valid    (o_kr_reconfig_readdata_valid ),
    .i_reconfig_kr_ip_waitrequest       (o_kr_reconfig_waitrequest    ),

    //---tod reconfig if---
    .o_tod_addr                         (ptp_master_tod_csr_addr          ),
    .o_tod_read                         (ptp_master_tod_csr_read          ),
    .o_tod_write                        (ptp_master_tod_csr_write         ),
    .o_tod_writedata                    (ptp_master_tod_csr_writedata     ),
    .o_tod_byteenable                   (),
    .i_tod_readdata                     (ptp_master_tod_csr_readdata      ),
    .i_tod_readdata_valid               (ptp_master_tod_csr_readdata_valid),
    .i_tod_waitrequest                  (ptp_master_tod_csr_waitrequest   ),

    //---Ctrl IF---
    .i_hw_ip_start_addr         (32'h0000_0000),
    .i_tod_start_addr           (32'h1000_0000),
    .i_asym_start_addr          (32'h1001_5000),
    .i_kr_ip_start_addr         (32'h1010_0000),
    .i_p2p_start_addr           (32'h1002_5000)
);
defparam hw_top_avmm_decoder.IP_NUM             = 1;
defparam hw_top_avmm_decoder.AVMM_ADDR_WIDTH    = 32;
defparam hw_top_avmm_decoder.P2P_ADDR_WIDTH     = 12;
defparam hw_top_avmm_decoder.ASYM_ADDR_WIDTH    = 12;
defparam hw_top_avmm_decoder.KR_IP_ADDR_WIDTH   = 14;
defparam hw_top_avmm_decoder.HW_IP_ADDR_WIDTH   = 24;
defparam hw_top_avmm_decoder.TOD_ADDR_WIDTH     = 6;

// PTP is disabled, driving PTP AVMM signals to 0
assign reconfig_p2p_readdata                = 32'h0;
assign reconfig_p2p_readdata_valid          = 1'b0;
assign reconfig_p2p_waitrequest             = 1'b0;
assign reconfig_asym_readdata               = 32'h0;
assign reconfig_asym_readdata_valid         = 1'b0;
assign reconfig_asym_waitrequest            = 1'b0;
assign ptp_master_tod_csr_readdata          = 32'h0;
assign ptp_master_tod_csr_readdata_valid    = 1'b0;
assign ptp_master_tod_csr_waitrequest       = 1'b0;

assign o_kr_reconfig_readdata = 32'd0;
assign o_kr_reconfig_readdata_valid = 1'b0;
assign o_kr_reconfig_waitrequest = 1'b0;



//---------------------------------------------------------------
// ETH_F_HW_IP_TOP
//---------------------------------------------------------------

//---------------------------------------------------------------
wire i_clk_sys;
wire i_clk_ref;

//---------------------------------------------------------------
  systemclk_f_0 sys_pll(
     .out_systempll_clk_0(i_clk_sys),               // out_systempll_clk_0.clk
     .in_refclk_fgt_0(i_refclk2pll),                // refclk_fgt.in_refclk_fgt_0
     .out_refclk_fgt_0(i_clk_ref)                 // out_refclk_fgt_0.clk	
);


//---------------------------------------------------------------
genvar portid;

generate
    for (portid =0; portid < INSTANCE_NUM; portid = portid+1) begin: IP_INST

    eth_f_hw_ip_top hw_ip_top (
            .i_clk_sys                        (i_clk_sys                ),
            .i_clk_ref                        (i_clk_ref                ),
            .i_reconfig_clk                   (i_reconfig_clk           ),
            .i_reconfig_reset                 (reconfig_reset_sync      ),

            .i_clk_rx                         (i_clk_rx         [portid]),
            .i_clk_tx                         (i_clk_tx         [portid]),
            .o_clk_pll                        (o_clk_pll        [portid]),
            .o_clk_tx_div                     (o_clk_tx_div     [portid]),
            .o_clk_rec_div64                  (o_clk_rec_div64  [portid]),
            .o_clk_rec_div                    (o_clk_rec_div    [portid]),

            .i_rx_rst_n                       (rx_rst_n         [portid]),
            .i_tx_rst_n                       (tx_rst_n         [portid]),
            .i_rst_n                          (rst_n            [portid]),
            .o_rst_ack_n                      (o_rst_ack_n      [portid]),
            .o_rx_rst_ack_n                   (o_rx_rst_ack_n   [portid]),
            .o_tx_rst_ack_n                   (o_tx_rst_ack_n   [portid]),

            .i_rx_serial                      (i_rx_serial      [portid*4+:4]),
            .i_rx_serial_n                    (i_rx_serial_n    [portid*4+:4]), 
            .o_tx_serial                      (o_tx_serial      [portid*4+:4]), 
            .o_tx_serial_n                    (o_tx_serial_n    [portid*4+:4]),
            .user_io                          (user_io                  ),
            .user_led                         (user_led                 ),
            .qsfp_lowpwr                      (qsfp_lowpwr              ),
            .qsfp_rstn                        (qsfp_rstn                ),
            .o_rx_pcs_ready                   (rx_pcs_ready    [portid]),
            .o_tx_lanes_stable                (tx_lanes_stable [portid]),
            .o_cdr_lock                       (cdr_lock        [portid]),
            .o_tx_pll_locked                  (tx_pll_locked   [portid]),
		


    
            //---avmm master from hw top decoder--
            .i_jtag_address                   (reconfig_hw_ip_addr           [portid*24+:24]),
            .i_jtag_read                      (reconfig_hw_ip_read           [portid]),
            .i_jtag_write                     (reconfig_hw_ip_write          [portid]),
            .i_jtag_writedata                 (reconfig_hw_ip_writedata      [portid*32+:32]),
            .i_jtag_byteenable                (reconfig_hw_ip_byteenable     [portid*4+:4]),
            .o_jtag_readdata                  (reconfig_hw_ip_readdata       [portid*32+:32]),
            .o_jtag_readdatavalid             (reconfig_hw_ip_readdata_valid [portid]),
            .o_jtag_waitrequest               (reconfig_hw_ip_waitrequest    [portid])
);
    end
endgenerate


// ---- IP_TC = 0 end


//---------------------------------------------------------------




endmodule

