// SPDX-License-Identifier: GPL-2.0
/*-
 * Copyright(c) 2020-21 Intel Corporation. All rights reserved.
 */
#ifndef _IFC_MCDMA_CHR_
#define _IFC_MCDMA_CHR_

#include <linux/cdev.h>
#include <linux/mm_types.h>
#include "ifc_mcdma_pci.h"
#include <ifc_mcdma.h>
#include <mcdma_ip_params.h>

#define IFC_MCDMA_CLASS_NAME	"ifc_mcdma_class"
#define IFC_MCDMA_DRIVER_NAME	"ifc_mcdma_driver"
#define IFC_NUM_DESC_INDEXES	65536

struct ifc_mcdma_desc {
	/* word 1,2 */
	uint64_t src;
	/* word 3,4 */
	uint64_t dest;
	/* word 5 */
	uint32_t len:20;
	uint32_t rsvd:12;
	/* word 6 */
	uint32_t didx:16;
	uint32_t msix_en:1;
	uint32_t wb_en:1;
	uint32_t rsvd2:14;
	/* word 7 */
	uint32_t rx_pyld_cnt:20;
	uint32_t rsvd3:10;
#ifdef IFC_QDMA_INTF_AVST
	uint32_t sof:1;
	uint32_t eof:1;
#else
	uint32_t rsvd4:2;
#endif
	/* word 8 */
	uint32_t rsvd5:28;
	uint32_t pad_len:2;
	uint32_t desc_invalid:1;
	uint32_t link:1;
};

/**
 * DMA Memory Structure
 */
struct ifc_dma_mem_info {
	struct list_head list;
	struct dma_pool *dmem_pool;
	dma_addr_t mem_dma_addr;
	void *mem_virt_addr;
	unsigned long user_virt_addr;
	size_t size;
	struct mutex *mem_lock;
};

struct ifc_mcdma_desc_sw {
	void *ctx_data;
};

struct ifc_queue_state {
	int head;
	int tail;
	uint16_t didx;
	int head_didx;
	int tail_didx;
	uint16_t eof_didx;
	int prep_counter;
	int file_size;
	int payload;
#ifndef IFC_QDMA_INTF_AVST
	uint64_t base;
	uint64_t addr;
#endif
};

/**
 * Queue Context
 */
struct ifc_mcdma_queue {
	void *qcsr;
	uint32_t dir;

	void *ring_virt_addr;		/* Ring Virtual Address */
	uint64_t ring_phy_addr;		/* Ring Physical Address */
	dma_addr_t ring_dma_addr;

	uint32_t qlen;			/* Descriptor ring length */
	int fifo_len;			/* FPGA FIFO Length */

	int consumed_head;		/* WB live copy */
	int cur_con_head;
	int prev_con_head;
	int head;			/* Ring Head */
	int tail;			/* Ring Tail */
	int processed_tail;
	int tid_updates;

	int head_didx;
	int tail_didx;

	uint16_t didx;			/* descriptor index */
	uint16_t ite_count;		/* descriptor index */
	uint16_t num_desc_bits;		/* bits required for didx */
	uint32_t wbe:1;			/* write back */
	uint32_t ire:1;			/* interrupt enable */
	uint32_t sof_rcvd:1;
	uint32_t num_desc_pages:11;
	uint32_t rsvd:18;

	struct ifc_mcdma_desc_sw *ctx;
	struct ifc_mcdma_channel *chnl;

	int irq_index;			/* index to interrupt context */
	struct eventfd_ctx *ectx;	/* eventfds passed from userland */
	uint16_t eof_didx;
	int file_size;
	int payload;
	int prep_counter;

#ifndef IFC_QDMA_INTF_AVST
	uint64_t base;
	uint64_t addr;
#endif
	struct ifc_queue_state qstate;
#ifdef IFC_QDMA_MSI_ENABLE
	int irq;
#endif
};

/**
 * Channel Context
 */
struct ifc_mcdma_channel {
	uint32_t channel_id;
#ifdef IFC_QDMA_MSI_ENABLE
	int direction;
#endif
	struct ifc_mcdma_queue rx;
	struct ifc_mcdma_queue tx;
	struct list_head mem_list;
	struct mutex mem_lock;
	struct mutex poll_lock;

	struct ifc_chr_dev_ctx *dev_ctx;
};

/**
 * Device Context
 */
struct ifc_chr_dev_ctx {
	void *qcsr;
	unsigned int major;
	unsigned int minor;
	struct cdev ifc_cdev;
	struct device *ifc_device;
	struct dma_pool *dmem_pool;
	size_t dmem_pool_size;
	char chr_dev_name[30];

	struct mutex chnl_lock;
	struct mutex tid_lock;

	struct ifc_mcdma_channel *channel_context[IFC_MCDMA_CHANNEL_MAX];
	bool channel_bitmap[IFC_MCDMA_CHANNEL_MAX];

	struct ifc_mcdma_dev_ctx *pdev_ctx;
};

int ifc_global_chr_init(void);
void ifc_global_chr_exit(void);
int ifc_device_chr_init(struct ifc_chr_dev_ctx **dctx,
			struct ifc_mcdma_dev_ctx *dev_ctx);
void ifc_device_chr_exit(struct ifc_chr_dev_ctx *dctx);
int ifc_mcdma_node_create(struct ifc_chr_dev_ctx *dctx);
void ifc_mcdma_node_destroy(struct ifc_chr_dev_ctx *dctx);
int ifc_mcdma_ip_reset(struct ifc_chr_dev_ctx *dctx);
#endif
