// ============================================================================
// Copyright (c) 2022 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use
//   or functionality of this code.
//
// ============================================================================
//
//  Terasic Technologies Inc
//  No.80, Fenggong Rd., Hukou Township, Hsinchu County 303035. Taiwan
//
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Tue May 31 14:42:44 2022
// ============================================================================

`define ENABLE_DDR4A
`define ENABLE_DDR4B
//`define ENABLE_QSFP28
//`define ENABLE_FMCP
//`define ENABLE_HPS
//`define ENABLE_HDMI
//`define ENABLE_PCIE

module ALCK(


      ///////// CLOCK /////////
      input              CLK_100_B3A_p,
      input              CLK_100_p,
      input              CLK_148M5_p,
      input              CLK_50_B2D_p,
      input              CLK_50_B2E_p,
      input              CLK_50_B3A_p,
      input              CLK_50_B3C_p,

      ///////// Buttons /////////
      input              CPU_RESET_n,
      input    [ 1: 0]   BUTTON, //BUTTON is Low-Active

      ///////// Swtiches /////////
      input    [ 1: 0]   SW,

      ///////// LED /////////
      output   [ 1: 0]   LED, //LED is Low-Active

`ifdef ENABLE_DDR4A
      ///////// DDR4A /////////
      input              DDR4A_REFCLK_p,
      output   [16: 0]   DDR4A_A,
      output   [ 1: 0]   DDR4A_BA,
      output   [ 1: 0]   DDR4A_BG,
      output             DDR4A_CK,
      output             DDR4A_CK_n,
      output             DDR4A_CKE,
      inout    [ 8: 0]   DDR4A_DQS,
      inout    [ 8: 0]   DDR4A_DQS_n,
      inout    [71: 0]   DDR4A_DQ,
      inout    [ 8: 0]   DDR4A_DBI_n,
      output             DDR4A_CS_n,
      output             DDR4A_RESET_n,
      output             DDR4A_ODT,
      output             DDR4A_PAR,
      input              DDR4A_ALERT_n,
      output             DDR4A_ACT_n,
      input              DDR4A_EVENT_n,
      inout              DDR4A_SCL,
      inout              DDR4A_SDA,
      input              DDR4A_RZQ,
`endif /*ENABLE_DDR4A*/

`ifdef ENABLE_DDR4B
      ///////// DDR4B /////////
      input              DDR4B_REFCLK_p,
      output   [16: 0]   DDR4B_A,
      output   [ 1: 0]   DDR4B_BA,
      output   [ 0: 0]   DDR4B_BG,
      output             DDR4B_CK,
      output             DDR4B_CK_n,
      output             DDR4B_CKE,
      inout    [ 8: 0]   DDR4B_DQS,
      inout    [ 8: 0]   DDR4B_DQS_n,
      inout    [71: 0]   DDR4B_DQ,
      inout    [ 8: 0]   DDR4B_DBI_n,
      output             DDR4B_CS_n,
      output             DDR4B_RESET_n,
      output             DDR4B_ODT,
      output             DDR4B_PAR,
      input              DDR4B_ALERT_n,
      output             DDR4B_ACT_n,
      input              DDR4B_RZQ,
`endif /*ENABLE_DDR4B*/

`ifdef ENABLE_HDMI
      ///////// HDMI /////////
      output   [ 3: 0]   HDMI_TX_D_p,
      output   [ 3: 0]   HDMI_TX_D_n,
      input              HDMI_REFCLK_100_p,
      input              HDMI_REFCLK_148M5_p,
      output             DVI_TX_5V,
      input              DVI_TX_CEC_IN_n,
      output             DVI_TX_CEC_OUT_n,
      input              DVI_TX_HPD_n,
      inout              DVI_TX_SCL,
      inout              DVI_TX_SDA,
      inout              HDMI_TX_SCL,
      inout              HDMI_TX_SDA,
`endif /*ENABLE_HDMI*/

      ///////// SI5340A0 /////////
      inout              SI5340A0_I2C_SCL,
      inout              SI5340A0_I2C_SDA,
      output             SI5340A0_OE_n,
      output             SI5340A0_RST_n,

      ///////// SI5340A1 /////////
      inout              SI5340A1_I2C_SCL,
      inout              SI5340A1_I2C_SDA,
      output             SI5340A1_OE_n,
      output             SI5340A1_RST_n,

      ///////// GPIO /////////
      input    [ 1: 0]   GPIO_CLK,
      inout    [ 3: 0]   GPIO_P,

`ifdef ENABLE_FMCP
      ///////// FMCP /////////
      inout              FMCP_CLK2_BIDIR_p,
      inout              FMCP_CLK2_BIDIR_n,
      inout              FMCP_CLK3_BIDIR_p,
      inout              FMCP_CLK3_BIDIR_n,
      output   [15: 0]   FMCP_DP_C2M_p,
      output   [15: 0]   FMCP_DP_C2M_n,
      input    [15: 0]   FMCP_DP_M2C_p,
      input    [15: 0]   FMCP_DP_M2C_n,
      inout    [23: 0]   FMCP_HA_p,
      inout    [23: 0]   FMCP_HA_n,
      inout    [21: 0]   FMCP_HB_p,
      inout    [21: 0]   FMCP_HB_n,
      inout    [33: 0]   FMCP_LA_p,
      inout    [33: 0]   FMCP_LA_n,
      input    [ 1: 0]   FMCP_CLK_M2C_p,
      input    [ 1: 0]   FMCP_GBTCLK_M2C_p,
      output             FMCP_REFCLK_C2M_p,
      input              FMCP_REFCLK_M2C_p,
      input    [ 3: 0]   FMCP_REFCLK_p,
      inout    [ 1: 0]   FMCP_GA,
      input              FMCP_RES0,
      inout              FMCP_RES1,
      inout              FMCP_SCL,
      inout              FMCP_SDA,
      output             FMCP_SYNC_C2M_p,
      input              FMCP_SYNC_M2C_p,
`endif /*ENABLE_FMCP*/

`ifdef ENABLE_PCIE
      ///////// PCIE /////////
      inout              PCIE_SMBCLK,
      inout              PCIE_SMBDAT,
      output   [ 7: 0]   PCIE_TX_p,
      output   [ 7: 0]   PCIE_TX_n,
      input    [ 7: 0]   PCIE_RX_p,
      input    [ 7: 0]   PCIE_RX_n,
      input              PCIE_REFCLK_p,
      input              PCIE_PERST_n,
`endif /*ENABLE_PCIE*/

`ifdef ENABLE_QSFP28
      ///////// QSFP28 /////////
      output   [ 3: 0]   QSFP28_TX_p,
      output   [ 3: 0]   QSFP28_TX_n,
      input    [ 3: 0]   QSFP28_RX_p,
      input    [ 3: 0]   QSFP28_RX_n,
      input    [ 1: 0]   QSFP28_REFCLK_p,
      input              QSFP28_INTERRUPT_n,
      output             QSFP28_LP_MODE,
      input              QSFP28_MOD_PRS_n,
      output             QSFP28_MOD_SEL_n,
      output             QSFP28_RST_n,
      inout              QSFP28_SCL,
      inout              QSFP28_SDA,
`endif /*ENABLE_QSFP28*/

      ///////// EXP /////////
      input              EXP_EN,

`ifdef ENABLE_HPS
      ///////// HPS /////////
      output             HPS_ENET_MDC,
      inout              HPS_ENET_MDIO,
      input              HPS_ENET_RX_CLK,
      input              HPS_ENET_RX_CTL,
      input    [ 3: 0]   HPS_ENET_RX_DATA,
      output             HPS_ENET_TX_CLK,
      output             HPS_ENET_TX_CTL,
      output   [ 3: 0]   HPS_ENET_TX_DATA,
      input              HPS_JTAG_TCK,
      input              HPS_JTAG_TDI,
      output             HPS_JTAG_TDO,
      input              HPS_JTAG_TMS,
      inout              HPS_KEY,
      inout              HPS_LED,
      input              HPS_OSC_CLK,
      output             HPS_SD_CLK,
      inout              HPS_SD_CMD,
      inout    [ 3: 0]   HPS_SD_DATA,
      input              HPS_UART_RX,
      output             HPS_UART_TX,
`endif /*ENABLE_HPS*/

      ///////// INFO /////////
      output             INFO_SPI_SCLK,
      input              INFO_SPI_MISO,
      output             INFO_SPI_MOSI,
      output             INFO_SPI_CS_n

);


//=======================================================
//  REG/WIRE declarations
//=======================================================

/////////////////////////// DDR4 x4 Verify ////////////////////////////////
wire user_clk_a; // clock for test controllers
wire user_clk_b; // clock for test controllers

/// test status ..
//DDR4 Verify (A)
parameter DDR4A_AVALON_ADDR_W = 27;
parameter DDR4A_AVALON_DATA_W = 512;
parameter DDR4A_BYTE_ENABLE = DDR4A_AVALON_DATA_W >> 3;
wire ddr4a_test_pass/*synthesis keep*/;
wire ddr4a_test_fail/*synthesis keep*/;
wire ddr4a_test_complete/*synthesis keep*/;
wire ddr4a_local_cal_success/*synthesis keep*/;
wire ddr4a_local_cal_fail/*synthesis keep*/;
wire 				calbusA_read;
wire 				calbusA_write;
wire [19:0]		calbusA_address;
wire [31:0]		calbusA_wdata;
wire [31:0]		calbusA_rdata;
wire [4095:0]	calbusA_seq_param_tbl;
/// test status ..
//DDR4 Verify (B)
parameter DDR4B_AVALON_ADDR_W = 27;
parameter DDR4B_AVALON_DATA_W = 576;
parameter DDR4B_BYTE_ENABLE = DDR4B_AVALON_DATA_W >> 3;
wire ddr4b_test_pass/*synthesis keep*/;
wire ddr4b_test_fail/*synthesis keep*/;
wire ddr4b_test_complete/*synthesis keep*/;
wire ddr4b_local_cal_success/*synthesis keep*/;
wire ddr4b_local_cal_fail/*synthesis keep*/;
wire 				calbusB_read;
wire 				calbusB_write;
wire [19:0]		calbusB_address;
wire [31:0]		calbusB_wdata;
wire [31:0]		calbusB_rdata;
wire [4095:0]	calbusB_seq_param_tbl;

//=======================================================
//  Structural coding
//=======================================================

wire global_reset;

edge_detector edge_detector_Inst(

	.iCLK(CLK_50_B3C_p),
	.iRST_n(S_global_resetn),
	.iIn(!BUTTON[1]),
	.oFallING_EDGE(),
	.oRISING_EDGE(),
	.oDEBOUNCE_OUT(global_reset)
);

wire S_global_resetn;
assign S_global_resetn = !ninit_done;

wire ninit_done;
    reset_release reset_release_inst (
        .ninit_done (ninit_done)  //  output,  width = 1, ninit_done.ninit_done
    );
	 
wire calbusA_clk;
wire calbusB_clk;

    DDR4_Cal DDR4A_Cal_Inst (
        .calbus_read_0          (calbusA_read),          //  output,     width = 1,   emif_calbus_0.calbus_read
        .calbus_write_0         (calbusA_write),         //  output,     width = 1,                .calbus_write
        .calbus_address_0       (calbusA_address),       //  output,    width = 20,                .calbus_address
        .calbus_wdata_0         (calbusA_wdata),         //  output,    width = 32,                .calbus_wdata
        .calbus_rdata_0         (calbusA_rdata),         //   input,    width = 32,                .calbus_rdata
        .calbus_seq_param_tbl_0 (calbusA_seq_param_tbl), //   input,  width = 4096,                .calbus_seq_param_tbl
        .calbus_clk             (calbusA_clk)              //  output,     width = 1, emif_calbus_clk.clk
    );

    DDR4_Cal DDR4B_Cal_Inst (
        .calbus_read_0          (calbusB_read),          //  output,     width = 1,   emif_calbus_0.calbus_read
        .calbus_write_0         (calbusB_write),         //  output,     width = 1,                .calbus_write
        .calbus_address_0       (calbusB_address),       //  output,    width = 20,                .calbus_address
        .calbus_wdata_0         (calbusB_wdata),         //  output,    width = 32,                .calbus_wdata
        .calbus_rdata_0         (calbusB_rdata),         //   input,    width = 32,                .calbus_rdata
        .calbus_seq_param_tbl_0 (calbusB_seq_param_tbl), //   input,  width = 4096,                .calbus_seq_param_tbl
        .calbus_clk             (calbusB_clk)              //  output,     width = 1, emif_calbus_clk.clk
    );

	 
    DDR4A DDR4A_Inst (
        .local_reset_req           (global_reset),           //   input,     width = 1,           local_reset_req.local_reset_req
        .local_reset_done          (ddr4a_reset_done),          //  output,     width = 1,        local_reset_status.local_reset_done
        .pll_ref_clk               (DDR4A_REFCLK_p),               //   input,     width = 1,               pll_ref_clk.clk
        .oct_rzqin                 (DDR4A_RZQ),                 //   input,     width = 1,                       oct.oct_rzqin
        .mem_ck                    (DDR4A_CK),                    //  output,     width = 1,                       mem.mem_ck
        .mem_ck_n                  (DDR4A_CK_n),                  //  output,     width = 1,                          .mem_ck_n
        .mem_a                     (DDR4A_A),                     //  output,    width = 17,                          .mem_a
        .mem_act_n                 (DDR4A_ACT_n),                 //  output,     width = 1,                          .mem_act_n
        .mem_ba                    (DDR4A_BA),                    //  output,     width = 2,                          .mem_ba
        .mem_bg                    (DDR4A_BG),                    //  output,     width = 2,                          .mem_bg
        .mem_cke                   (DDR4A_CKE),                   //  output,     width = 1,                          .mem_cke
        .mem_cs_n                  (DDR4A_CS_n),                  //  output,     width = 1,                          .mem_cs_n
        .mem_odt                   (DDR4A_ODT),                   //  output,     width = 1,                          .mem_odt
        .mem_reset_n               (DDR4A_RESET_n),               //  output,     width = 1,                          .mem_reset_n
        .mem_par                   (DDR4A_PAR),                   //  output,     width = 1,                          .mem_par
        .mem_alert_n               (DDR4A_ALERT_n),               //   input,     width = 1,                          .mem_alert_n
        .mem_dqs                   (DDR4A_DQS),                   //   inout,     width = 9,                          .mem_dqs
        .mem_dqs_n                 (DDR4A_DQS_n),                 //   inout,     width = 9,                          .mem_dqs_n
        .mem_dq                    (DDR4A_DQ),                    //   inout,    width = 72,                          .mem_dq
        .mem_dbi_n                 (DDR4A_DBI_n),                 //   inout,     width = 9,                          .mem_dbi_n
        .local_cal_success         (ddr4a_local_cal_success),         //  output,     width = 1,                    status.local_cal_success
        .local_cal_fail            (ddr4a_local_cal_fail),            //  output,     width = 1,                          .local_cal_fail
        .calbus_read               (calbusA_read),               //   input,     width = 1,               emif_calbus.calbus_read
        .calbus_write              (calbusA_write),              //   input,     width = 1,                          .calbus_write
        .calbus_address            (calbusA_address),            //   input,    width = 20,                          .calbus_address
        .calbus_wdata              (calbusA_wdata),              //   input,    width = 32,                          .calbus_wdata
        .calbus_rdata              (calbusA_rdata),              //  output,    width = 32,                          .calbus_rdata
        .calbus_seq_param_tbl      (calbusA_seq_param_tbl),      //  output,  width = 4096,                          .calbus_seq_param_tbl
        .calbus_clk                (calbusA_clk),                //   input,     width = 1,           emif_calbus_clk.clk
        .emif_usr_reset_n          (ddr4a_emif_usr_reset_n),          //  output,     width = 1,          emif_usr_reset_n.reset_n
        .emif_usr_clk              (user_clk_a),              //  output,     width = 1,              emif_usr_clk.clk
        .ctrl_ecc_user_interrupt_0 (), //  output,     width = 1, ctrl_ecc_user_interrupt_0.ctrl_ecc_user_interrupt
        .amm_ready_0               (ddr4a_avl_ready),               //  output,     width = 1,                ctrl_amm_0.waitrequest_n
        .amm_read_0                (ddr4a_avl_read_req),                //   input,     width = 1,                          .read
        .amm_write_0               (ddr4a_avl_write_req),               //   input,     width = 1,                          .write
        .amm_address_0             (ddr4a_avl_addr),             //   input,    width = 27,                          .address
        .amm_readdata_0            (ddr4a_avl_rdata),            //  output,   width = 512,                          .readdata
        .amm_writedata_0           (ddr4a_avl_wdata),           //   input,   width = 512,                          .writedata
        .amm_burstcount_0          (ddr4a_avl_size),          //   input,     width = 7,                          .burstcount
        .amm_byteenable_0          ({DDR4A_BYTE_ENABLE{1'b1}}),          //   input,    width = 64,                          .byteenable
        .amm_readdatavalid_0       (ddr4a_avl_rdata_valid)        //  output,     width = 1,                          .readdatavalid
    );

/////////////////// DDR4(A) Test ///////////////////
wire         							ddr4a_avl_ready;                  //          avl.waitrequest_n
wire [DDR4A_AVALON_ADDR_W-1:0]	ddr4a_avl_addr;                   //             .address
wire         							ddr4a_avl_rdata_valid;            //             .readdatavalid
wire [DDR4A_AVALON_DATA_W-1:0]	ddr4a_avl_rdata;                  //             .readdata
wire [DDR4A_AVALON_DATA_W-1:0]	ddr4a_avl_wdata;                  //             .writedata
wire         							ddr4a_avl_read_req;               //             .read
wire         							ddr4a_avl_write_req;              //             .write
wire [6:0]   							ddr4a_avl_size;                   //             .burstcount
wire 			 							ddr4a_emif_usr_reset_n;
wire 			 							ddr4a_reset_done;

Avalon_bus_RW_Test DDR4A_Verify(
		    .iCLK(user_clk_a),
		    .iRST_n(ddr4a_emif_usr_reset_n ),
		    .insert_error(BUTTON[0]),

		    .local_init_done(ddr4a_local_cal_success),
		    .avl_waitrequest_n(ddr4a_avl_ready),
		    .avl_address(ddr4a_avl_addr),
		    .avl_readdatavalid(ddr4a_avl_rdata_valid),
		    .avl_readdata(ddr4a_avl_rdata),
		    .avl_writedata(ddr4a_avl_wdata),
		    .avl_read(ddr4a_avl_read_req),
		    .avl_write(ddr4a_avl_write_req),
		    .avl_size(ddr4a_avl_size),

		    .drv_status_pass(ddr4a_test_pass),
		    .drv_status_fail(ddr4a_test_fail),
		    .drv_status_test_complete(ddr4a_test_complete)

);
defparam DDR4A_Verify.ADDR_W = DDR4A_AVALON_ADDR_W;
defparam DDR4A_Verify.DATA_W = DDR4A_AVALON_DATA_W;

	
	
    DDR4B DDR4B_Inst (
        .local_reset_req      (global_reset),      //   input,     width = 1,    local_reset_req.local_reset_req
        .local_reset_done     (ddr4b_reset_done),     //  output,     width = 1, local_reset_status.local_reset_done
        .pll_ref_clk          (DDR4B_REFCLK_p),          //   input,     width = 1,        pll_ref_clk.clk
        .oct_rzqin            (DDR4B_RZQ),            //   input,     width = 1,                oct.oct_rzqin
        .mem_ck               (DDR4B_CK),               //  output,     width = 1,                mem.mem_ck
        .mem_ck_n             (DDR4B_CK_n),             //  output,     width = 1,                   .mem_ck_n
        .mem_a                (DDR4B_A),                //  output,    width = 17,                   .mem_a
        .mem_act_n            (DDR4B_ACT_n),            //  output,     width = 1,                   .mem_act_n
        .mem_ba               (DDR4B_BA),               //  output,     width = 2,                   .mem_ba
        .mem_bg               (DDR4B_BG),               //  output,     width = 2,                   .mem_bg
        .mem_cke              (DDR4B_CKE),              //  output,     width = 1,                   .mem_cke
        .mem_cs_n             (DDR4B_CS_n),             //  output,     width = 1,                   .mem_cs_n
        .mem_odt              (DDR4B_ODT),              //  output,     width = 1,                   .mem_odt
        .mem_reset_n          (DDR4B_RESET_n),          //  output,     width = 1,                   .mem_reset_n
        .mem_par              (DDR4B_PAR),              //  output,     width = 1,                   .mem_par
        .mem_alert_n          (DDR4B_ALERT_n),          //   input,     width = 1,                   .mem_alert_n
        .mem_dqs              (DDR4B_DQS),              //   inout,     width = 9,                   .mem_dqs
        .mem_dqs_n            (DDR4B_DQS_n),            //   inout,     width = 9,                   .mem_dqs_n
        .mem_dq               (DDR4B_DQ),               //   inout,    width = 72,                   .mem_dq
        .mem_dbi_n            (DDR4B_DBI_n),            //   inout,     width = 9,                   .mem_dbi_n
        .local_cal_success    (ddr4b_local_cal_success),    //  output,     width = 1,             status.local_cal_success
        .local_cal_fail       (ddr4b_local_cal_fail),       //  output,     width = 1,                   .local_cal_fail
        .emif_usr_reset_n     (ddr4b_emif_usr_reset_n),     //  output,     width = 1,   emif_usr_reset_n.reset_n
        .emif_usr_clk         (user_clk_b),         //  output,     width = 1,       emif_usr_clk.clk
        .amm_ready_0          (ddr4b_avl_ready),          //  output,     width = 1,         ctrl_amm_0.waitrequest_n
        .amm_read_0           (ddr4b_avl_read_req),           //   input,     width = 1,                   .read
        .amm_write_0          (ddr4b_avl_write_req),          //   input,     width = 1,                   .write
        .amm_address_0        (ddr4b_avl_addr),        //   input,    width = 27,                   .address
        .amm_readdata_0       (ddr4b_avl_rdata),       //  output,   width = 576,                   .readdata
        .amm_writedata_0      (ddr4b_avl_wdata),      //   input,   width = 576,                   .writedata
        .amm_burstcount_0     (ddr4b_avl_size),     //   input,     width = 7,                   .burstcount
        .amm_byteenable_0     ({DDR4B_BYTE_ENABLE{1'b1}}),     //   input,    width = 72,                   .byteenable
        .amm_readdatavalid_0  (ddr4b_avl_rdata_valid),  //  output,     width = 1,                   .readdatavalid
        .calbus_read          (calbusB_read),          //   input,     width = 1,        emif_calbus.calbus_read
        .calbus_write         (calbusB_write),         //   input,     width = 1,                   .calbus_write
        .calbus_address       (calbusB_address),       //   input,    width = 20,                   .calbus_address
        .calbus_wdata         (calbusB_wdata),         //   input,    width = 32,                   .calbus_wdata
        .calbus_rdata         (calbusB_rdata),         //  output,    width = 32,                   .calbus_rdata
        .calbus_seq_param_tbl (calbusB_seq_param_tbl), //  output,  width = 4096,                   .calbus_seq_param_tbl
        .calbus_clk           (calbusB_clk)            //   input,     width = 1,    emif_calbus_clk.clk
    );

/////////////////// DDR4(B) Test ///////////////////
wire         							ddr4b_avl_ready;                  //          avl.waitrequest_n
wire [DDR4B_AVALON_ADDR_W-1:0]	ddr4b_avl_addr;                   //             .address
wire         							ddr4b_avl_rdata_valid;            //             .readdatavalid
wire [DDR4B_AVALON_DATA_W-1:0]	ddr4b_avl_rdata;                  //             .readdata
wire [DDR4B_AVALON_DATA_W-1:0]	ddr4b_avl_wdata;                  //             .writedata
wire         							ddr4b_avl_read_req;               //             .read
wire         							ddr4b_avl_write_req;              //             .write
wire [6:0]   							ddr4b_avl_size;                   //             .burstcount
wire 			 							ddr4b_emif_usr_reset_n;
wire 			 							ddr4b_reset_done;


Avalon_bus_RW_Test DDR4B_Verify(
		    .iCLK(user_clk_b),
		    .iRST_n(ddr4b_emif_usr_reset_n ),
		    .insert_error(BUTTON[0]),

		    .local_init_done(ddr4b_local_cal_success),
		    .avl_waitrequest_n(ddr4b_avl_ready),
		    .avl_address(ddr4b_avl_addr),
		    .avl_readdatavalid(ddr4b_avl_rdata_valid),
		    .avl_readdata(ddr4b_avl_rdata),
		    .avl_writedata(ddr4b_avl_wdata),
		    .avl_read(ddr4b_avl_read_req),
		    .avl_write(ddr4b_avl_write_req),
		    .avl_size(ddr4b_avl_size),

		    .drv_status_pass(ddr4b_test_pass),
		    .drv_status_fail(ddr4b_test_fail),
		    .drv_status_test_complete(ddr4b_test_complete)

);
defparam DDR4B_Verify.ADDR_W = DDR4B_AVALON_ADDR_W;
defparam DDR4B_Verify.DATA_W = DDR4B_AVALON_DATA_W;
	 
wire heart_led;
heart_beat  heart_beat_0 ( .CLK(CLK_50_B2D_p ),     .CLK_FREQ  (50_000_000) , . CK_1HZ (heart_led) );

assign LED[0] =  (ddr4a_local_cal_success) ?~(ddr4a_test_pass & ddr4a_test_complete):heart_led;
assign LED[1] =  (ddr4b_local_cal_success) ?~(ddr4b_test_pass & ddr4b_test_complete):heart_led;


	

	
endmodule
