// ============================================================================
// Copyright (c) 2022 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use
//   or functionality of this code.
//
// ============================================================================
//
//  Terasic Technologies Inc
//  No.80, Fenggong Rd., Hukou Township, Hsinchu County 303035. Taiwan
//
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Fri Jul 15 14:16:19 2022
// ============================================================================

//`define ENABLE_DDR4A
//`define ENABLE_DDR4B
//`define ENABLE_QSFP28
//`define ENABLE_FMCP
//`define ENABLE_HPS
//`define ENABLE_HDMI
//`define ENABLE_PCIE

module golden_top(

      ///////// CLOCK /////////
      input              CLK_100_B3A_p,
      input              CLK_100_p,
      input              CLK_148M5_p,
      input              CLK_50_B2D_p,
      input              CLK_50_B2E_p,
      input              CLK_50_B3A_p,
      input              CLK_50_B3C_p,

      ///////// Buttons /////////
      input              CPU_RESET_n,
      input    [ 1: 0]   BUTTON, //BUTTON is Low-Active

      ///////// Swtiches /////////
      input    [ 1: 0]   SW,

      ///////// LED /////////
      output   [ 1: 0]   LED, //LED is Low-Active

`ifdef ENABLE_DDR4A
      ///////// DDR4A /////////
      input              DDR4A_REFCLK_p,
      output   [16: 0]   DDR4A_A,
      output   [ 1: 0]   DDR4A_BA,
      output   [ 1: 0]   DDR4A_BG,
      output             DDR4A_CK,
      output             DDR4A_CK_n,
      output             DDR4A_CKE,
      inout    [ 8: 0]   DDR4A_DQS,
      inout    [ 8: 0]   DDR4A_DQS_n,
      inout    [71: 0]   DDR4A_DQ,
      inout    [ 8: 0]   DDR4A_DBI_n,
      output             DDR4A_CS_n,
      output             DDR4A_RESET_n,
      output             DDR4A_ODT,
      output             DDR4A_PAR,
      input              DDR4A_ALERT_n,
      output             DDR4A_ACT_n,
      input              DDR4A_EVENT_n,
      inout              DDR4A_SCL,
      inout              DDR4A_SDA,
      input              DDR4A_RZQ,
`endif /*ENABLE_DDR4A*/

`ifdef ENABLE_DDR4B
      ///////// DDR4B /////////
      input              DDR4B_REFCLK_p,
      output   [16: 0]   DDR4B_A,
      output   [ 1: 0]   DDR4B_BA,
      output   [ 0: 0]   DDR4B_BG,
      output             DDR4B_CK,
      output             DDR4B_CK_n,
      output             DDR4B_CKE,
      inout    [ 8: 0]   DDR4B_DQS,
      inout    [ 8: 0]   DDR4B_DQS_n,
      inout    [71: 0]   DDR4B_DQ,
      inout    [ 8: 0]   DDR4B_DBI_n,
      output             DDR4B_CS_n,
      output             DDR4B_RESET_n,
      output             DDR4B_ODT,
      output             DDR4B_PAR,
      input              DDR4B_ALERT_n,
      output             DDR4B_ACT_n,
      input              DDR4B_RZQ,
`endif /*ENABLE_DDR4B*/

`ifdef ENABLE_HDMI
      ///////// HDMI /////////
      output   [ 3: 0]   HDMI_TX_D_p,
      output   [ 3: 0]   HDMI_TX_D_n,
      input              HDMI_REFCLK_100_p,
      input              HDMI_REFCLK_148M5_p,
      output             DVI_TX_5V,
      input              DVI_TX_CEC_IN_n,
      output             DVI_TX_CEC_OUT_n,
      input              DVI_TX_HPD_n,
      inout              DVI_TX_SCL,
      inout              DVI_TX_SDA,
      inout              HDMI_TX_SCL,
      inout              HDMI_TX_SDA,
`endif /*ENABLE_HDMI*/

      ///////// SI5340A0 /////////
      inout              SI5340A0_I2C_SCL,
      inout              SI5340A0_I2C_SDA,
      output             SI5340A0_OE_n,
      output             SI5340A0_RST_n,

      ///////// SI5340A1 /////////
      inout              SI5340A1_I2C_SCL,
      inout              SI5340A1_I2C_SDA,
      output             SI5340A1_OE_n,
      output             SI5340A1_RST_n,

      ///////// GPIO /////////
      input    [ 1: 0]   GPIO_CLK,
      inout    [ 3: 0]   GPIO_P,

`ifdef ENABLE_FMCP
      ///////// FMCP /////////
      inout              FMCP_CLK2_BIDIR_p,
      inout              FMCP_CLK2_BIDIR_n,
      inout              FMCP_CLK3_BIDIR_p,
      inout              FMCP_CLK3_BIDIR_n,
      output   [15: 0]   FMCP_DP_C2M_p,
      output   [15: 0]   FMCP_DP_C2M_n,
      input    [15: 0]   FMCP_DP_M2C_p,
      input    [15: 0]   FMCP_DP_M2C_n,
      inout    [23: 0]   FMCP_HA_p,
      inout    [23: 0]   FMCP_HA_n,
      inout    [21: 0]   FMCP_HB_p,
      inout    [21: 0]   FMCP_HB_n,
      inout    [33: 0]   FMCP_LA_p,
      inout    [33: 0]   FMCP_LA_n,
      input    [ 1: 0]   FMCP_CLK_M2C_p,
      input    [ 1: 0]   FMCP_GBTCLK_M2C_p,
      output             FMCP_REFCLK_C2M_p,
      input              FMCP_REFCLK_M2C_p,
      input    [ 3: 0]   FMCP_REFCLK_p,
      inout    [ 1: 0]   FMCP_GA,
      input              FMCP_RES0,
      inout              FMCP_RES1,
      inout              FMCP_SCL,
      inout              FMCP_SDA,
      output             FMCP_SYNC_C2M_p,
      input              FMCP_SYNC_M2C_p,
`endif /*ENABLE_FMCP*/

`ifdef ENABLE_PCIE
      ///////// PCIE /////////
      inout              PCIE_SMBCLK,
      inout              PCIE_SMBDAT,
      output   [ 7: 0]   PCIE_TX_p,
      output   [ 7: 0]   PCIE_TX_n,
      input    [ 7: 0]   PCIE_RX_p,
      input    [ 7: 0]   PCIE_RX_n,
      input              PCIE_REFCLK_p,
      input              PCIE_PERST_n,
`endif /*ENABLE_PCIE*/

`ifdef ENABLE_QSFP28
      ///////// QSFP28 /////////
      output   [ 3: 0]   QSFP28_TX_p,
      output   [ 3: 0]   QSFP28_TX_n,
      input    [ 3: 0]   QSFP28_RX_p,
      input    [ 3: 0]   QSFP28_RX_n,
      input    [ 1: 0]   QSFP28_REFCLK_p,
      input              QSFP28_INTERRUPT_n,
      output             QSFP28_LP_MODE,
      input              QSFP28_MOD_PRS_n,
      output             QSFP28_MOD_SEL_n,
      output             QSFP28_RST_n,
      inout              QSFP28_SCL,
      inout              QSFP28_SDA,
`endif /*ENABLE_QSFP28*/

      ///////// EXP /////////
      input              EXP_EN,

`ifdef ENABLE_HPS
      ///////// HPS /////////
      output             HPS_ENET_MDC,
      inout              HPS_ENET_MDIO,
      input              HPS_ENET_RX_CLK,
      input              HPS_ENET_RX_CTL,
      input    [ 3: 0]   HPS_ENET_RX_DATA,
      output             HPS_ENET_TX_CLK,
      output             HPS_ENET_TX_CTL,
      output   [ 3: 0]   HPS_ENET_TX_DATA,
      input              HPS_JTAG_TCK,
      input              HPS_JTAG_TDI,
      output             HPS_JTAG_TDO,
      input              HPS_JTAG_TMS,
      inout              HPS_KEY,
      inout              HPS_LED,
      input              HPS_OSC_CLK,
      output             HPS_SD_CLK,
      inout              HPS_SD_CMD,
      inout    [ 3: 0]   HPS_SD_DATA,
      input              HPS_UART_RX,
      output             HPS_UART_TX,
`endif /*ENABLE_HPS*/

      ///////// INFO /////////
      output             INFO_SPI_SCLK,
      input              INFO_SPI_MISO,
      output             INFO_SPI_MOSI,
      output             INFO_SPI_CS_n

);


//=======================================================
//  REG/WIRE declarations
//=======================================================
//System Monitor IP
//IP interface
wire        Info_Valid;
wire [15:0] Board_Version	;//A~F.ex BOARD_VERSION = 16'h000b ,express to revB
wire [15:0] MaxCode_Version;//0~F.ex MAX10_VERSION = 16'h0004 ,express to rev4
wire [15:0] PowerIn_Voltage;//unit : Celsius degree    
wire [15:0] PowerIn_Current;//unit : Celsius degree    
wire [15:0] CORE_Voltage	;//unit : mv   
wire [15:0] CORE_Current	;//unit : 10mA  
wire [15:0] Fan_Speed	   ;//unit : rpm  
wire [15:0] Fan_Speed2	   ;//unit : rpm  
wire [15:0] Temp_FPGA	   ;//unit : Celsius degree 
wire [15:0] Temp_Board	   ;//unit : Celsius degree 
wire [15:0] PIN_STATUS	   ;//
wire [15:0] Temp_SDM	      ;//unit : Celsius degree 
wire [15:0] Temp_FTILE12C	;//unit : Celsius degree 
wire [15:0] Temp_FTILE13A	;//unit : Celsius degree 
wire [15:0] Temp_Board2	   ;//unit : Celsius degree 
wire [15:0] Shutdown_flag  ;


//----reg
reg [15:0] rBoard_Version	;
reg [15:0] rMaxCode_Version;
reg [15:0] rPowerIn_Voltage;
reg [15:0] rPowerIn_Current;
reg [15:0] rCORE_Voltage	;
reg [15:0] rCORE_Current	;
reg [15:0] rFan_Speed	   ;
reg [15:0] rFan_Speed2	   ;
reg [15:0] rTemp_FPGA	   ; 
reg [15:0] rTemp_Board	   ; 
reg [15:0] rPIN_STATUS	   ; 
reg [15:0] rTemp_SDM	      ;
reg [15:0] rTemp_FTILE12C	;
reg [15:0] rTemp_FTILE13A	;
reg [15:0] rCORE_Voltage2	;
reg [15:0] rCORE_Current2	;
reg [15:0] rTemp_Board2	   ; 
reg [15:0] rShutdown_flag  ;


//=======================================================
//  Structural coding
//=======================================================

          

                
BOARD_INFO BOARD_INFO_i(    
             .RESET_N          (BUTTON[0]    ),  // input
             .CLK_50           (CLK_50_B2D_p   ),  // input

             // 4-pin SPI master interface
             .SCLK             (INFO_SPI_SCLK), // output
             .MISO             (INFO_SPI_MISO), // input
             .MOSI             (INFO_SPI_MOSI), // output
             .CS_n             (INFO_SPI_CS_n), // output

             // status report (output)
             .Info_Valid       ( Info_Valid     ), //1: following data is valid
             .Board_Version	 ( Board_Version	), //A~F.ex BOARD_VERSION = 16'h000b ,express to revB
             .MaxCode_Version  ( MaxCode_Version), //0~F.ex MAX10_VERSION = 16'h0004 ,express to rev4
             .PowerIn_Voltage  ( PowerIn_Voltage), //unit : Celsius degree    
             .PowerIn_Current  ( PowerIn_Current), //unit : Celsius degree    
             .CORE_Voltage	    ( CORE_Voltage	), //unit : mv   
             .CORE_Current	    ( CORE_Current	), //unit : 10mA  
				 .Fan_Speed	       ( Fan_Speed	   ), //unit : rpm                                     
				 .Fan_Speed2	    ( Fan_Speed2	   ), //unit : rpm                                     
             .Temp_FPGA	       ( Temp_FPGA	   ), //unit : Celsius degree 
				 .Temp_Board	    ( Temp_Board	   ), //unit : Celsius degree
				 .PIN_STATUS	    ( PIN_STATUS	   ),  
				 .Temp_SDM	       ( Temp_SDM	      ), //unit : Celsius degree
				 .Temp_FTILE12C	 ( Temp_FTILE12C	), //unit : Celsius degree 
				 .Temp_FTILE13A	 ( Temp_FTILE13A	), //unit : Celsius degree 
				 .Temp_Board2	    ( Temp_Board2	   ), //unit : Celsius degree 
             .Shutdown_flag	 ( Shutdown_flag  )			 
				 
);





always @(posedge CLK_50_B2D_p ) begin
    if (Info_Valid) begin
        rBoard_Version	<= Board_Version	;
        rMaxCode_Version<= MaxCode_Version;
        rPowerIn_Voltage<= PowerIn_Voltage;
        rPowerIn_Current<= PowerIn_Current;
        rCORE_Voltage	<= CORE_Voltage	;
        rCORE_Current	<= CORE_Current	;
        rFan_Speed	   <= Fan_Speed	   ;
        rFan_Speed2	   <= Fan_Speed2	   ;
		  rTemp_FPGA	   <= Temp_FPGA	   ;
		  rTemp_Board	   <= Temp_Board	   ;
		  rPIN_STATUS	   <= PIN_STATUS	   ;
		  rTemp_SDM	      <= Temp_SDM	      ;
		  rTemp_FTILE12C	<= Temp_FTILE12C	   ;
		  rTemp_FTILE13A	<= Temp_FTILE13A	   ;
		  rTemp_Board2	   <= Temp_Board2	   ;
		  rShutdown_flag  <= Shutdown_flag  ;
    end
end




endmodule
