** (C) 2001-2020 Intel Corporation. All rights reserved.
** Your use of Intel Corporation's design tools, logic functions and other 
** software and tools, and its AMPP partner logic functions, and any output 
** files from any of the foregoing (including device programming or simulation 
** files), and any associated documentation or information are expressly subject 
** to the terms and conditions of the Intel Program License Subscription 
** Agreement, Intel FPGA IP License Agreement, or other applicable 
** license agreement, including, without limitation, that your use is for the 
** sole purpose of programming logic devices manufactured by Intel and sold by 
** Intel or its authorized distributors.  Please refer to the applicable 
** agreement for further details.


******************************************************************************
** This file provides a template for customers to insert a simulation model of
** the 12-line address/command PCB model from the FPGA to the multi-rank
** address/command model. This model should ideally focus on Lane 0 signals of
** the address/command channel as they have the highest activity factor and 
** also implement the memory clock, but models for Lanes 1/2/3 of the address/
** command channel can be used as well.
**
** In the single-component case, this model should contain the extraction from
** the balls of the FPGA to the balls of the memory component, and the multi-
** rank model (ac_mr_conn_wrapper) as well as the flyby model
** (ac_dimm_flyby_wrapper) should simply be pass-throughs.
******************************************************************************

.subckt ac_pcb_wrapper         fpga_0   fpga_1   fpga_2   fpga_3   fpga_4   fpga_5
+                              fpga_6   fpga_7   fpga_8   fpga_9   fpga_10  fpga_11
+                              acch_0   acch_1   acch_2   acch_3   acch_4   acch_5
+                              acch_6   acch_7   acch_8   acch_9   acch_10  acch_11
+                              pcb_gnd
+ USE_PCB_EXTRACTION    = 0
+ PCB_EXTRACTION_FILE   = str("model")

*******************************
* Parameters
*******************************
.param DEFAULT_TL_TD = 2000p
.param DEFAULT_TL_Z0 = 50

*******************************
* Models
*******************************
.model sac_pcb_model S TSTONEFILE=str(PCB_EXTRACTION_FILE)


.IF (USE_PCB_EXTRACTION == 0)

* Generic 2ns Transmission Line Interface
Tpcb_0  fpga_0   pcb_gnd acch_0   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_1  fpga_1   pcb_gnd acch_1   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_2  fpga_2   pcb_gnd acch_2   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_3  fpga_3   pcb_gnd acch_3   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_4  fpga_4   pcb_gnd acch_4   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_5  fpga_5   pcb_gnd acch_5   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_6  fpga_6   pcb_gnd acch_6   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_7  fpga_7   pcb_gnd acch_7   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_8  fpga_8   pcb_gnd acch_8   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_9  fpga_9   pcb_gnd acch_9   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_10 fpga_10  pcb_gnd acch_10  pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_11 fpga_11  pcb_gnd acch_11  pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0

.ELSE
* Use customer extraction
Sac_pcb_model
+ fpga_0   fpga_1  fpga_2  fpga_3  fpga_4  fpga_5  fpga_6  fpga_7  fpga_8  fpga_9  fpga_10 fpga_11
+ acch_0   acch_1  acch_2  acch_3  acch_4  acch_5  acch_6  acch_7  acch_8  acch_9  acch_10 acch_11
+ pcb_gnd
+ mname=sac_pcb_model

.ENDIF


.ends ac_pcb_wrapper
