** (C) 2001-2020 Intel Corporation. All rights reserved.
** Your use of Intel Corporation's design tools, logic functions and other 
** software and tools, and its AMPP partner logic functions, and any output 
** files from any of the foregoing (including device programming or simulation 
** files), and any associated documentation or information are expressly subject 
** to the terms and conditions of the Intel Program License Subscription 
** Agreement, Intel FPGA IP License Agreement, or other applicable 
** license agreement, including, without limitation, that your use is for the 
** sole purpose of programming logic devices manufactured by Intel and sold by 
** Intel or its authorized distributors.  Please refer to the applicable 
** agreement for further details.


******************************************************************************
** This file provides a template for customers to perform a simulation of the
** address/command channel of their external memory interface.  The simulation
** assumes the existence of a 12-line simulation model or extraction taken
** from Lane 0 of the address/command channel. This deck will automatically 
** create victim-aggressor PRBS patterns as well a memory clock pattern.
** 
** This simulation is especially important for interfaces that use multiple
** components in a custom fly-by topology.  The spacing between individual
** components, impedance of the fly-by bus as well as the values of the VTT
** termination resistors can be swept using this simulation deck.
** 
** A data eye mask is supplied to check for adequate margin at the memory
** address/command receiver. At minimum, the mask must fit within the
** address/command eye of the victim signal for every component in each rank,
** and across all ranks.
******************************************************************************

* TITLE **************************************************
.title ac_top

* SIMULATION PARAMETERS **********************************

* SIMULATION OPTIONS *************************************
.inc "finesim_options.inc"

* INCLUDE FILES ******************************************
.inc "membsi_ip_parameters.dat"
.inc "pin_parasitics.dat"
.inc "ac_pcb_wrapper.sp"
.inc "ac_dimm_flyby_wrapper.sp"
.inc "ac_mr_conn_wrapper.sp"
.inc "prbs_10_x3.sp"
.inc "prbs_15.sp"
.inc "dqs_wave.sp"
.inc "rx_buffer.sp"
.inc "tx_buffer.sp"
.inc "lane_tx12.sp"
.inc "lane_rx12.sp"


* MAIN ***************************************************
.param dtrise='65p' dtfall='65p' dist=0 delay1=0
.param gnd=0
.param ac_period='MEM_AC_FREQ_PS/1000000000000.0'


***************
* Power Sources
***************
Vvcc  vcc   gnd MEM_VCC
Vvtt  vtt   gnd 'MEM_VCC/2'


*******************************
* IBIS Driver Bank
*******************************
xlane_tx12 vcc gnd fpga_0 fpga_1 fpga_2 fpga_3 fpga_4 fpga_5 fpga_6 fpga_7 fpga_8 fpga_9 fpga_10 fpga_11 lane_tx12 
+ IBIS_FILE=str(FPGA_IBIS_MODEL_FILE) period=ac_period
+ IBIS_MODEL_0_NAME=str(MEMAC_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_IBIS)
+ BUFFER_FUNCTION_P0=AC_F_P0_BUFTYPE  BUFFER_DATATYPE_P0=AC_F_P0_DATATYPE  BUFFER_DATAPHASE_P0=AC_F_P0_DATAPHASE  BUFFER_IBISTYPE_P0=AC_F_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_F_P1_BUFTYPE  BUFFER_DATATYPE_P1=AC_F_P1_DATATYPE  BUFFER_DATAPHASE_P1=AC_F_P1_DATAPHASE  BUFFER_IBISTYPE_P1=AC_F_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_F_P2_BUFTYPE  BUFFER_DATATYPE_P2=AC_F_P2_DATATYPE  BUFFER_DATAPHASE_P2=AC_F_P2_DATAPHASE  BUFFER_IBISTYPE_P2=AC_F_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_F_P3_BUFTYPE  BUFFER_DATATYPE_P3=AC_F_P3_DATATYPE  BUFFER_DATAPHASE_P3=AC_F_P3_DATAPHASE  BUFFER_IBISTYPE_P3=AC_F_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_F_P4_BUFTYPE  BUFFER_DATATYPE_P4=AC_F_P4_DATATYPE  BUFFER_DATAPHASE_P4=AC_F_P4_DATAPHASE  BUFFER_IBISTYPE_P4=AC_F_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_F_P5_BUFTYPE  BUFFER_DATATYPE_P5=AC_F_P5_DATATYPE  BUFFER_DATAPHASE_P5=AC_F_P5_DATAPHASE  BUFFER_IBISTYPE_P5=AC_F_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_F_P6_BUFTYPE  BUFFER_DATATYPE_P6=AC_F_P6_DATATYPE  BUFFER_DATAPHASE_P6=AC_F_P6_DATAPHASE  BUFFER_IBISTYPE_P6=AC_F_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_F_P7_BUFTYPE  BUFFER_DATATYPE_P7=AC_F_P7_DATATYPE  BUFFER_DATAPHASE_P7=AC_F_P7_DATAPHASE  BUFFER_IBISTYPE_P7=AC_F_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_F_P8_BUFTYPE  BUFFER_DATATYPE_P8=AC_F_P8_DATATYPE  BUFFER_DATAPHASE_P8=AC_F_P8_DATAPHASE  BUFFER_IBISTYPE_P8=AC_F_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_F_P9_BUFTYPE  BUFFER_DATATYPE_P9=AC_F_P9_DATATYPE  BUFFER_DATAPHASE_P9=AC_F_P9_DATAPHASE  BUFFER_IBISTYPE_P9=AC_F_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_F_P10_BUFTYPE  BUFFER_DATATYPE_P10=AC_F_P10_DATATYPE  BUFFER_DATAPHASE_P10=AC_F_P10_DATAPHASE  BUFFER_IBISTYPE_P10=AC_F_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_F_P11_BUFTYPE  BUFFER_DATATYPE_P11=AC_F_P11_DATATYPE  BUFFER_DATAPHASE_P11=AC_F_P11_DATAPHASE  BUFFER_IBISTYPE_P11=AC_F_P11_IBISTYPE
+ r_pin_p0 =AC_F_P0_RVAL  c_pin_p0 =AC_F_P0_CVAL  l_pin_p0 =AC_F_P0_LVAL
+ r_pin_p1 =AC_F_P1_RVAL  c_pin_p1 =AC_F_P1_CVAL  l_pin_p1 =AC_F_P1_LVAL 
+ r_pin_p2 =AC_F_P2_RVAL  c_pin_p2 =AC_F_P2_CVAL  l_pin_p2 =AC_F_P2_LVAL 
+ r_pin_p3 =AC_F_P3_RVAL  c_pin_p3 =AC_F_P3_CVAL  l_pin_p3 =AC_F_P3_LVAL 
+ r_pin_p4 =AC_F_P4_RVAL  c_pin_p4 =AC_F_P4_CVAL  l_pin_p4 =AC_F_P4_LVAL 
+ r_pin_p5 =AC_F_P5_RVAL  c_pin_p5 =AC_F_P5_CVAL  l_pin_p5 =AC_F_P5_LVAL 
+ r_pin_p6 =AC_F_P6_RVAL  c_pin_p6 =AC_F_P6_CVAL  l_pin_p6 =AC_F_P6_LVAL 
+ r_pin_p7 =AC_F_P7_RVAL  c_pin_p7 =AC_F_P7_CVAL  l_pin_p7 =AC_F_P7_LVAL 
+ r_pin_p8 =AC_F_P8_RVAL  c_pin_p8 =AC_F_P8_CVAL  l_pin_p8 =AC_F_P8_LVAL 
+ r_pin_p9 =AC_F_P9_RVAL  c_pin_p9 =AC_F_P9_CVAL  l_pin_p9 =AC_F_P9_LVAL 
+ r_pin_p10=AC_F_P10_RVAL c_pin_p10=AC_F_P10_CVAL l_pin_p10=AC_F_P10_LVAL
+ r_pin_p11=AC_F_P11_RVAL c_pin_p11=AC_F_P11_CVAL l_pin_p11=AC_F_P11_LVAL


*******************************
* Main PCB Channel Structure
*******************************
xac_pcb_channel fpga_0   fpga_1   fpga_2   fpga_3   fpga_4   fpga_5    fpga_6   fpga_7   fpga_8   fpga_9   fpga_10  fpga_11
+               acch_0   acch_1   acch_2   acch_3   acch_4   acch_5    acch_6   acch_7   acch_8   acch_9   acch_10  acch_11
+               gnd ac_pcb_wrapper
+ USE_PCB_EXTRACTION=USE_AC_PCB_EXTRACTION
+ PCB_EXTRACTION_FILE=str(AC_PCB_EXTRACTION_FILE)

*******************************
* Multi-Rank PCB Structure
*******************************
xac_mr_conn    acch_0   acch_1   acch_2   acch_3   acch_4   acch_5   acch_6   acch_7   acch_8   acch_9   acch_10  acch_11
+              rank0_0  rank0_1  rank0_2  rank0_3  rank0_4  rank0_5  rank0_6  rank0_7  rank0_8  rank0_9  rank0_10 rank0_11
+              rank1_0  rank1_1  rank1_2  rank1_3  rank1_4  rank1_5  rank1_6  rank1_7  rank1_8  rank1_9  rank1_10 rank1_11
+              rank2_0  rank2_1  rank2_2  rank2_3  rank2_4  rank2_5  rank2_6  rank2_7  rank2_8  rank2_9  rank2_10 rank2_11
+              rank3_0  rank3_1  rank3_2  rank3_3  rank3_4  rank3_5  rank3_6  rank3_7  rank3_8  rank3_9  rank3_10 rank3_11
+              gnd ac_mr_conn_wrapper
+ USE_AC_MULTIRANK_CONNECTOR_EXTRACTION=USE_AC_MULTIRANK_CONNECTOR_EXTRACTION
+ AC_MULTIRANK_CONNECTOR_EXTRACTION_FILE=str(AC_MULTIRANK_CONNECTOR_EXTRACTION_FILE)
+ MEMAC_RANKS=MEMAC_RANKS
 

*******************************
* Receiver Structures
*******************************

* Flyby structure for Rank 0
xac_flyby_r0 rank0_0 rank0_1 rank0_2 rank0_3 rank0_4 rank0_5 rank0_6 rank0_7 rank0_8 rank0_9 rank0_10 rank0_11
+            rank0_comp0_0 rank0_comp0_1 rank0_comp0_2 rank0_comp0_3 rank0_comp0_4 rank0_comp0_5 rank0_comp0_6 rank0_comp0_7 rank0_comp0_8 rank0_comp0_9 rank0_comp0_10 rank0_comp0_11
+            rank0_comp1_0 rank0_comp1_1 rank0_comp1_2 rank0_comp1_3 rank0_comp1_4 rank0_comp1_5 rank0_comp1_6 rank0_comp1_7 rank0_comp1_8 rank0_comp1_9 rank0_comp1_10 rank0_comp1_11
+            rank0_comp2_0 rank0_comp2_1 rank0_comp2_2 rank0_comp2_3 rank0_comp2_4 rank0_comp2_5 rank0_comp2_6 rank0_comp2_7 rank0_comp2_8 rank0_comp2_9 rank0_comp2_10 rank0_comp2_11
+            rank0_comp3_0 rank0_comp3_1 rank0_comp3_2 rank0_comp3_3 rank0_comp3_4 rank0_comp3_5 rank0_comp3_6 rank0_comp3_7 rank0_comp3_8 rank0_comp3_9 rank0_comp3_10 rank0_comp3_11
+            rank0_comp4_0 rank0_comp4_1 rank0_comp4_2 rank0_comp4_3 rank0_comp4_4 rank0_comp4_5 rank0_comp4_6 rank0_comp4_7 rank0_comp4_8 rank0_comp4_9 rank0_comp4_10 rank0_comp4_11
+            rank0_comp5_0 rank0_comp5_1 rank0_comp5_2 rank0_comp5_3 rank0_comp5_4 rank0_comp5_5 rank0_comp5_6 rank0_comp5_7 rank0_comp5_8 rank0_comp5_9 rank0_comp5_10 rank0_comp5_11
+            rank0_comp6_0 rank0_comp6_1 rank0_comp6_2 rank0_comp6_3 rank0_comp6_4 rank0_comp6_5 rank0_comp6_6 rank0_comp6_7 rank0_comp6_8 rank0_comp6_9 rank0_comp6_10 rank0_comp6_11
+            rank0_comp7_0 rank0_comp7_1 rank0_comp7_2 rank0_comp7_3 rank0_comp7_4 rank0_comp7_5 rank0_comp7_6 rank0_comp7_7 rank0_comp7_8 rank0_comp7_9 rank0_comp7_10 rank0_comp7_11
+            rank0_comp8_0 rank0_comp8_1 rank0_comp8_2 rank0_comp8_3 rank0_comp8_4 rank0_comp8_5 rank0_comp8_6 rank0_comp8_7 rank0_comp8_8 rank0_comp8_9 rank0_comp8_10 rank0_comp8_11
+            vtt gnd ac_dimm_flyby_wrapper
+ USE_AC_FLYBY_EXTRACTION  = USE_AC_FLYBY_EXTRACTION
+ AC_FLYBY_EXTRACTION_FILE = str(AC_FLYBY_EXTRACTION_FILE)
+ MEMAC_COMPS_PER_RANK  = MEMAC_COMPS_PER_RANK

xlane_rx12_r0c0 vcc gnd rank0_comp0_0 rank0_comp0_1 rank0_comp0_2 rank0_comp0_3 rank0_comp0_4 rank0_comp0_5 rank0_comp0_6 rank0_comp0_7 rank0_comp0_8 rank0_comp0_9 rank0_comp0_10 rank0_comp0_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL


.IF (MEMAC_COMPS_PER_RANK > 1) 
xlane_rx12_r0c1 vcc gnd rank0_comp1_0 rank0_comp1_1 rank0_comp1_2 rank0_comp1_3 rank0_comp1_4 rank0_comp1_5 rank0_comp1_6 rank0_comp1_7 rank0_comp1_8 rank0_comp1_9 rank0_comp1_10 rank0_comp1_11 lane_rx12 
.ENDIF

.IF (MEMAC_COMPS_PER_RANK > 2) 
xlane_rx12_r0c2 vcc gnd rank0_comp2_0 rank0_comp2_1 rank0_comp2_2 rank0_comp2_3 rank0_comp2_4 rank0_comp2_5 rank0_comp2_6 rank0_comp2_7 rank0_comp2_8 rank0_comp2_9 rank0_comp2_10 rank0_comp2_11 lane_rx12 
.ENDIF

.IF (MEMAC_COMPS_PER_RANK > 3) 
xlane_rx12_r0c3 vcc gnd rank0_comp3_0 rank0_comp3_1 rank0_comp3_2 rank0_comp3_3 rank0_comp3_4 rank0_comp3_5 rank0_comp3_6 rank0_comp3_7 rank0_comp3_8 rank0_comp3_9 rank0_comp3_10 rank0_comp3_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL
.ENDIF

.IF (MEMAC_COMPS_PER_RANK > 4) 
xlane_rx12_r0c4 vcc gnd rank0_comp4_0 rank0_comp4_1 rank0_comp4_2 rank0_comp4_3 rank0_comp4_4 rank0_comp4_5 rank0_comp4_6 rank0_comp4_7 rank0_comp4_8 rank0_comp4_9 rank0_comp4_10 rank0_comp4_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL
.ENDIF

.IF (MEMAC_COMPS_PER_RANK > 5) 
xlane_rx12_r0c5 vcc gnd rank0_comp5_0 rank0_comp5_1 rank0_comp5_2 rank0_comp5_3 rank0_comp5_4 rank0_comp5_5 rank0_comp5_6 rank0_comp5_7 rank0_comp5_8 rank0_comp5_9 rank0_comp5_10 rank0_comp5_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL
.ENDIF

.IF (MEMAC_COMPS_PER_RANK > 6) 
xlane_rx12_r0c6 vcc gnd rank0_comp6_0 rank0_comp6_1 rank0_comp6_2 rank0_comp6_3 rank0_comp6_4 rank0_comp6_5 rank0_comp6_6 rank0_comp6_7 rank0_comp6_8 rank0_comp6_9 rank0_comp6_10 rank0_comp6_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL
.ENDIF

.IF (MEMAC_COMPS_PER_RANK > 7) 
xlane_rx12_r0c7 vcc gnd rank0_comp7_0 rank0_comp7_1 rank0_comp7_2 rank0_comp7_3 rank0_comp7_4 rank0_comp7_5 rank0_comp7_6 rank0_comp7_7 rank0_comp7_8 rank0_comp7_9 rank0_comp7_10 rank0_comp7_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL
.ENDIF

.IF (MEMAC_COMPS_PER_RANK > 8) 
xlane_rx12_r0c8 vcc gnd rank0_comp8_0 rank0_comp8_1 rank0_comp8_2 rank0_comp8_3 rank0_comp8_4 rank0_comp8_5 rank0_comp8_6 rank0_comp8_7 rank0_comp8_8 rank0_comp8_9 rank0_comp8_10 rank0_comp8_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL
.ENDIF


* Flyby structure for Rank 1
.IF (MEMAC_RANKS > 1)
xac_flyby_r1 rank1_0 rank1_1 rank1_2 rank1_3 rank1_4 rank1_5 rank1_6 rank1_7 rank1_8 rank1_9 rank1_10 rank1_11
+            rank1_comp0_0 rank1_comp0_1 rank1_comp0_2 rank1_comp0_3 rank1_comp0_4 rank1_comp0_5 rank1_comp0_6 rank1_comp0_7 rank1_comp0_8 rank1_comp0_9 rank1_comp0_10 rank1_comp0_11
+            rank1_comp1_0 rank1_comp1_1 rank1_comp1_2 rank1_comp1_3 rank1_comp1_4 rank1_comp1_5 rank1_comp1_6 rank1_comp1_7 rank1_comp1_8 rank1_comp1_9 rank1_comp1_10 rank1_comp1_11
+            rank1_comp2_0 rank1_comp2_1 rank1_comp2_2 rank1_comp2_3 rank1_comp2_4 rank1_comp2_5 rank1_comp2_6 rank1_comp2_7 rank1_comp2_8 rank1_comp2_9 rank1_comp2_10 rank1_comp2_11
+            rank1_comp3_0 rank1_comp3_1 rank1_comp3_2 rank1_comp3_3 rank1_comp3_4 rank1_comp3_5 rank1_comp3_6 rank1_comp3_7 rank1_comp3_8 rank1_comp3_9 rank1_comp3_10 rank1_comp3_11
+            rank1_comp4_0 rank1_comp4_1 rank1_comp4_2 rank1_comp4_3 rank1_comp4_4 rank1_comp4_5 rank1_comp4_6 rank1_comp4_7 rank1_comp4_8 rank1_comp4_9 rank1_comp4_10 rank1_comp4_11
+            rank1_comp5_0 rank1_comp5_1 rank1_comp5_2 rank1_comp5_3 rank1_comp5_4 rank1_comp5_5 rank1_comp5_6 rank1_comp5_7 rank1_comp5_8 rank1_comp5_9 rank1_comp5_10 rank1_comp5_11
+            rank1_comp6_0 rank1_comp6_1 rank1_comp6_2 rank1_comp6_3 rank1_comp6_4 rank1_comp6_5 rank1_comp6_6 rank1_comp6_7 rank1_comp6_8 rank1_comp6_9 rank1_comp6_10 rank1_comp6_11
+            rank1_comp7_0 rank1_comp7_1 rank1_comp7_2 rank1_comp7_3 rank1_comp7_4 rank1_comp7_5 rank1_comp7_6 rank1_comp7_7 rank1_comp7_8 rank1_comp7_9 rank1_comp7_10 rank1_comp7_11
+            rank1_comp8_0 rank1_comp8_1 rank1_comp8_2 rank1_comp8_3 rank1_comp8_4 rank1_comp8_5 rank1_comp8_6 rank1_comp8_7 rank1_comp8_8 rank1_comp8_9 rank1_comp8_10 rank1_comp8_11
+            vtt gnd ac_dimm_flyby_wrapper
+ USE_AC_FLYBY_EXTRACTION  = USE_AC_FLYBY_EXTRACTION
+ AC_FLYBY_EXTRACTION_FILE = str(AC_FLYBY_EXTRACTION_FILE)
+ MEMAC_COMPS_PER_RANK  = MEMAC_COMPS_PER_RANK

xlane_rx12_r1c0 vcc gnd rank1_comp0_0 rank1_comp0_1 rank1_comp0_2 rank1_comp0_3 rank1_comp0_4 rank1_comp0_5 rank1_comp0_6 rank1_comp0_7 rank1_comp0_8 rank1_comp0_9 rank1_comp0_10 rank1_comp0_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.IF (MEMAC_COMPS_PER_RANK > 1) 
xlane_rx12_r1c1 vcc gnd rank1_comp1_0 rank1_comp1_1 rank1_comp1_2 rank1_comp1_3 rank1_comp1_4 rank1_comp1_5 rank1_comp1_6 rank1_comp1_7 rank1_comp1_8 rank1_comp1_9 rank1_comp1_10 rank1_comp1_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 2) 
xlane_rx12_r1c2 vcc gnd rank1_comp2_0 rank1_comp2_1 rank1_comp2_2 rank1_comp2_3 rank1_comp2_4 rank1_comp2_5 rank1_comp2_6 rank1_comp2_7 rank1_comp2_8 rank1_comp2_9 rank1_comp2_10 rank1_comp2_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 3) 
xlane_rx12_r1c3 vcc gnd rank1_comp3_0 rank1_comp3_1 rank1_comp3_2 rank1_comp3_3 rank1_comp3_4 rank1_comp3_5 rank1_comp3_6 rank1_comp3_7 rank1_comp3_8 rank1_comp3_9 rank1_comp3_10 rank1_comp3_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 4) 
xlane_rx12_r1c4 vcc gnd rank1_comp4_0 rank1_comp4_1 rank1_comp4_2 rank1_comp4_3 rank1_comp4_4 rank1_comp4_5 rank1_comp4_6 rank1_comp4_7 rank1_comp4_8 rank1_comp4_9 rank1_comp4_10 rank1_comp4_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 5) 
xlane_rx12_r1c5 vcc gnd rank1_comp5_0 rank1_comp5_1 rank1_comp5_2 rank1_comp5_3 rank1_comp5_4 rank1_comp5_5 rank1_comp5_6 rank1_comp5_7 rank1_comp5_8 rank1_comp5_9 rank1_comp5_10 rank1_comp5_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 6) 
xlane_rx12_r1c6 vcc gnd rank1_comp6_0 rank1_comp6_1 rank1_comp6_2 rank1_comp6_3 rank1_comp6_4 rank1_comp6_5 rank1_comp6_6 rank1_comp6_7 rank1_comp6_8 rank1_comp6_9 rank1_comp6_10 rank1_comp6_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 7) 
xlane_rx12_r1c7 vcc gnd rank1_comp7_0 rank1_comp7_1 rank1_comp7_2 rank1_comp7_3 rank1_comp7_4 rank1_comp7_5 rank1_comp7_6 rank1_comp7_7 rank1_comp7_8 rank1_comp7_9 rank1_comp7_10 rank1_comp7_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 8) 
xlane_rx12_r1c8 vcc gnd rank1_comp8_0 rank1_comp8_1 rank1_comp8_2 rank1_comp8_3 rank1_comp8_4 rank1_comp8_5 rank1_comp8_6 rank1_comp8_7 rank1_comp8_8 rank1_comp8_9 rank1_comp8_10 rank1_comp8_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.ENDIF

* Flyby structure for Rank 2
.IF (MEMAC_RANKS > 2)
xac_flyby_r2 rank2_0 rank2_1 rank2_2 rank2_3 rank2_4 rank2_5 rank2_6 rank2_7 rank2_8 rank2_9 rank2_10 rank2_11
+            rank2_comp0_0 rank2_comp0_1 rank2_comp0_2 rank2_comp0_3 rank2_comp0_4 rank2_comp0_5 rank2_comp0_6 rank2_comp0_7 rank2_comp0_8 rank2_comp0_9 rank2_comp0_10 rank2_comp0_11
+            rank2_comp1_0 rank2_comp1_1 rank2_comp1_2 rank2_comp1_3 rank2_comp1_4 rank2_comp1_5 rank2_comp1_6 rank2_comp1_7 rank2_comp1_8 rank2_comp1_9 rank2_comp1_10 rank2_comp1_11
+            rank2_comp2_0 rank2_comp2_1 rank2_comp2_2 rank2_comp2_3 rank2_comp2_4 rank2_comp2_5 rank2_comp2_6 rank2_comp2_7 rank2_comp2_8 rank2_comp2_9 rank2_comp2_10 rank2_comp2_11
+            rank2_comp3_0 rank2_comp3_1 rank2_comp3_2 rank2_comp3_3 rank2_comp3_4 rank2_comp3_5 rank2_comp3_6 rank2_comp3_7 rank2_comp3_8 rank2_comp3_9 rank2_comp3_10 rank2_comp3_11
+            rank2_comp4_0 rank2_comp4_1 rank2_comp4_2 rank2_comp4_3 rank2_comp4_4 rank2_comp4_5 rank2_comp4_6 rank2_comp4_7 rank2_comp4_8 rank2_comp4_9 rank2_comp4_10 rank2_comp4_11
+            rank2_comp5_0 rank2_comp5_1 rank2_comp5_2 rank2_comp5_3 rank2_comp5_4 rank2_comp5_5 rank2_comp5_6 rank2_comp5_7 rank2_comp5_8 rank2_comp5_9 rank2_comp5_10 rank2_comp5_11
+            rank2_comp6_0 rank2_comp6_1 rank2_comp6_2 rank2_comp6_3 rank2_comp6_4 rank2_comp6_5 rank2_comp6_6 rank2_comp6_7 rank2_comp6_8 rank2_comp6_9 rank2_comp6_10 rank2_comp6_11
+            rank2_comp7_0 rank2_comp7_1 rank2_comp7_2 rank2_comp7_3 rank2_comp7_4 rank2_comp7_5 rank2_comp7_6 rank2_comp7_7 rank2_comp7_8 rank2_comp7_9 rank2_comp7_10 rank2_comp7_11
+            rank2_comp8_0 rank2_comp8_1 rank2_comp8_2 rank2_comp8_3 rank2_comp8_4 rank2_comp8_5 rank2_comp8_6 rank2_comp8_7 rank2_comp8_8 rank2_comp8_9 rank2_comp8_10 rank2_comp8_11
+            vtt gnd ac_dimm_flyby_wrapper
+ USE_AC_FLYBY_EXTRACTION  = USE_AC_FLYBY_EXTRACTION
+ AC_FLYBY_EXTRACTION_FILE = str(AC_FLYBY_EXTRACTION_FILE)
+ MEMAC_COMPS_PER_RANK  = MEMAC_COMPS_PER_RANK

xlane_rx12_r2c0 vcc gnd rank2_comp0_0 rank2_comp0_1 rank2_comp0_2 rank2_comp0_3 rank2_comp0_4 rank2_comp0_5 rank2_comp0_6 rank2_comp0_7 rank2_comp0_8 rank2_comp0_9 rank2_comp0_10 rank2_comp0_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.IF (MEMAC_COMPS_PER_RANK > 1) 
xlane_rx12_r2c1 vcc gnd rank2_comp1_0 rank2_comp1_1 rank2_comp1_2 rank2_comp1_3 rank2_comp1_4 rank2_comp1_5 rank2_comp1_6 rank2_comp1_7 rank2_comp1_8 rank2_comp1_9 rank2_comp1_10 rank2_comp1_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 2) 
xlane_rx12_r2c2 vcc gnd rank2_comp2_0 rank2_comp2_1 rank2_comp2_2 rank2_comp2_3 rank2_comp2_4 rank2_comp2_5 rank2_comp2_6 rank2_comp2_7 rank2_comp2_8 rank2_comp2_9 rank2_comp2_10 rank2_comp2_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 3) 
xlane_rx12_r2c3 vcc gnd rank2_comp3_0 rank2_comp3_1 rank2_comp3_2 rank2_comp3_3 rank2_comp3_4 rank2_comp3_5 rank2_comp3_6 rank2_comp3_7 rank2_comp3_8 rank2_comp3_9 rank2_comp3_10 rank2_comp3_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 4) 
xlane_rx12_r2c4 vcc gnd rank2_comp4_0 rank2_comp4_1 rank2_comp4_2 rank2_comp4_3 rank2_comp4_4 rank2_comp4_5 rank2_comp4_6 rank2_comp4_7 rank2_comp4_8 rank2_comp4_9 rank2_comp4_10 rank2_comp4_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 5) 
xlane_rx12_r2c5 vcc gnd rank2_comp5_0 rank2_comp5_1 rank2_comp5_2 rank2_comp5_3 rank2_comp5_4 rank2_comp5_5 rank2_comp5_6 rank2_comp5_7 rank2_comp5_8 rank2_comp5_9 rank2_comp5_10 rank2_comp5_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 6) 
xlane_rx12_r2c6 vcc gnd rank2_comp6_0 rank2_comp6_1 rank2_comp6_2 rank2_comp6_3 rank2_comp6_4 rank2_comp6_5 rank2_comp6_6 rank2_comp6_7 rank2_comp6_8 rank2_comp6_9 rank2_comp6_10 rank2_comp6_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 7) 
xlane_rx12_r2c7 vcc gnd rank2_comp7_0 rank2_comp7_1 rank2_comp7_2 rank2_comp7_3 rank2_comp7_4 rank2_comp7_5 rank2_comp7_6 rank2_comp7_7 rank2_comp7_8 rank2_comp7_9 rank2_comp7_10 rank2_comp7_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 8) 
xlane_rx12_r2c8 vcc gnd rank2_comp8_0 rank2_comp8_1 rank2_comp8_2 rank2_comp8_3 rank2_comp8_4 rank2_comp8_5 rank2_comp8_6 rank2_comp8_7 rank2_comp8_8 rank2_comp8_9 rank2_comp8_10 rank2_comp8_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.ENDIF

* Flyby structure for Rank 3
.IF (MEMAC_RANKS > 3)
xac_flyby_r3 rank3_0 rank3_1 rank3_2 rank3_3 rank3_4 rank3_5 rank3_6 rank3_7 rank3_8 rank3_9 rank3_10 rank3_11
+            rank3_comp0_0 rank3_comp0_1 rank3_comp0_2 rank3_comp0_3 rank3_comp0_4 rank3_comp0_5 rank3_comp0_6 rank3_comp0_7 rank3_comp0_8 rank3_comp0_9 rank3_comp0_10 rank3_comp0_11
+            rank3_comp1_0 rank3_comp1_1 rank3_comp1_2 rank3_comp1_3 rank3_comp1_4 rank3_comp1_5 rank3_comp1_6 rank3_comp1_7 rank3_comp1_8 rank3_comp1_9 rank3_comp1_10 rank3_comp1_11
+            rank3_comp2_0 rank3_comp2_1 rank3_comp2_2 rank3_comp2_3 rank3_comp2_4 rank3_comp2_5 rank3_comp2_6 rank3_comp2_7 rank3_comp2_8 rank3_comp2_9 rank3_comp2_10 rank3_comp2_11
+            rank3_comp3_0 rank3_comp3_1 rank3_comp3_2 rank3_comp3_3 rank3_comp3_4 rank3_comp3_5 rank3_comp3_6 rank3_comp3_7 rank3_comp3_8 rank3_comp3_9 rank3_comp3_10 rank3_comp3_11
+            rank3_comp4_0 rank3_comp4_1 rank3_comp4_2 rank3_comp4_3 rank3_comp4_4 rank3_comp4_5 rank3_comp4_6 rank3_comp4_7 rank3_comp4_8 rank3_comp4_9 rank3_comp4_10 rank3_comp4_11
+            rank3_comp5_0 rank3_comp5_1 rank3_comp5_2 rank3_comp5_3 rank3_comp5_4 rank3_comp5_5 rank3_comp5_6 rank3_comp5_7 rank3_comp5_8 rank3_comp5_9 rank3_comp5_10 rank3_comp5_11
+            rank3_comp6_0 rank3_comp6_1 rank3_comp6_2 rank3_comp6_3 rank3_comp6_4 rank3_comp6_5 rank3_comp6_6 rank3_comp6_7 rank3_comp6_8 rank3_comp6_9 rank3_comp6_10 rank3_comp6_11
+            rank3_comp7_0 rank3_comp7_1 rank3_comp7_2 rank3_comp7_3 rank3_comp7_4 rank3_comp7_5 rank3_comp7_6 rank3_comp7_7 rank3_comp7_8 rank3_comp7_9 rank3_comp7_10 rank3_comp7_11
+            rank3_comp8_0 rank3_comp8_1 rank3_comp8_2 rank3_comp8_3 rank3_comp8_4 rank3_comp8_5 rank3_comp8_6 rank3_comp8_7 rank3_comp8_8 rank3_comp8_9 rank3_comp8_10 rank3_comp8_11
+            vtt gnd ac_dimm_flyby_wrapper
+ USE_AC_FLYBY_EXTRACTION  = USE_AC_FLYBY_EXTRACTION
+ AC_FLYBY_EXTRACTION_FILE = str(AC_FLYBY_EXTRACTION_FILE)
+ MEMAC_COMPS_PER_RANK  = MEMAC_COMPS_PER_RANK

xlane_rx12_r3c0 vcc gnd rank3_comp0_0 rank3_comp0_1 rank3_comp0_2 rank3_comp0_3 rank3_comp0_4 rank3_comp0_5 rank3_comp0_6 rank3_comp0_7 rank3_comp0_8 rank3_comp0_9 rank3_comp0_10 rank3_comp0_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.IF (MEMAC_COMPS_PER_RANK > 1) 
xlane_rx12_r3c1 vcc gnd rank3_comp1_0 rank3_comp1_1 rank3_comp1_2 rank3_comp1_3 rank3_comp1_4 rank3_comp1_5 rank3_comp1_6 rank3_comp1_7 rank3_comp1_8 rank3_comp1_9 rank3_comp1_10 rank3_comp1_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 2) 
xlane_rx12_r3c2 vcc gnd rank3_comp2_0 rank3_comp2_1 rank3_comp2_2 rank3_comp2_3 rank3_comp2_4 rank3_comp2_5 rank3_comp2_6 rank3_comp2_7 rank3_comp2_8 rank3_comp2_9 rank3_comp2_10 rank3_comp2_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 3) 
xlane_rx12_r3c3 vcc gnd rank3_comp3_0 rank3_comp3_1 rank3_comp3_2 rank3_comp3_3 rank3_comp3_4 rank3_comp3_5 rank3_comp3_6 rank3_comp3_7 rank3_comp3_8 rank3_comp3_9 rank3_comp3_10 rank3_comp3_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 4) 
xlane_rx12_r3c4 vcc gnd rank3_comp4_0 rank3_comp4_1 rank3_comp4_2 rank3_comp4_3 rank3_comp4_4 rank3_comp4_5 rank3_comp4_6 rank3_comp4_7 rank3_comp4_8 rank3_comp4_9 rank3_comp4_10 rank3_comp4_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 5) 
xlane_rx12_r3c5 vcc gnd rank3_comp5_0 rank3_comp5_1 rank3_comp5_2 rank3_comp5_3 rank3_comp5_4 rank3_comp5_5 rank3_comp5_6 rank3_comp5_7 rank3_comp5_8 rank3_comp5_9 rank3_comp5_10 rank3_comp5_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 6) 
xlane_rx12_r3c6 vcc gnd rank3_comp6_0 rank3_comp6_1 rank3_comp6_2 rank3_comp6_3 rank3_comp6_4 rank3_comp6_5 rank3_comp6_6 rank3_comp6_7 rank3_comp6_8 rank3_comp6_9 rank3_comp6_10 rank3_comp6_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 7) 
xlane_rx12_r3c7 vcc gnd rank3_comp7_0 rank3_comp7_1 rank3_comp7_2 rank3_comp7_3 rank3_comp7_4 rank3_comp7_5 rank3_comp7_6 rank3_comp7_7 rank3_comp7_8 rank3_comp7_9 rank3_comp7_10 rank3_comp7_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.IF (MEMAC_COMPS_PER_RANK > 8) 
xlane_rx12_r3c8 vcc gnd rank3_comp8_0 rank3_comp8_1 rank3_comp8_2 rank3_comp8_3 rank3_comp8_4 rank3_comp8_5 rank3_comp8_6 rank3_comp8_7 rank3_comp8_8 rank3_comp8_9 rank3_comp8_10 rank3_comp8_11 lane_rx12 
+ IBIS_FILE=str(MEM_IBIS_MODEL_FILE) IBIS_MODEL_0_NAME=str(MEMAC_COMP_IBIS) IBIS_MODEL_1_NAME=str(MEMCLK_COMP_IBIS)
+ BUFFER_FUNCTION_P0=AC_M_P0_BUFTYPE    BUFFER_IBISTYPE_P0=AC_M_P0_IBISTYPE
+ BUFFER_FUNCTION_P1=AC_M_P1_BUFTYPE    BUFFER_IBISTYPE_P1=AC_M_P1_IBISTYPE
+ BUFFER_FUNCTION_P2=AC_M_P2_BUFTYPE    BUFFER_IBISTYPE_P2=AC_M_P2_IBISTYPE
+ BUFFER_FUNCTION_P3=AC_M_P3_BUFTYPE    BUFFER_IBISTYPE_P3=AC_M_P3_IBISTYPE
+ BUFFER_FUNCTION_P4=AC_M_P4_BUFTYPE    BUFFER_IBISTYPE_P4=AC_M_P4_IBISTYPE
+ BUFFER_FUNCTION_P5=AC_M_P5_BUFTYPE    BUFFER_IBISTYPE_P5=AC_M_P5_IBISTYPE
+ BUFFER_FUNCTION_P6=AC_M_P6_BUFTYPE    BUFFER_IBISTYPE_P6=AC_M_P6_IBISTYPE
+ BUFFER_FUNCTION_P7=AC_M_P7_BUFTYPE    BUFFER_IBISTYPE_P7=AC_M_P7_IBISTYPE
+ BUFFER_FUNCTION_P8=AC_M_P8_BUFTYPE    BUFFER_IBISTYPE_P8=AC_M_P8_IBISTYPE
+ BUFFER_FUNCTION_P9=AC_M_P9_BUFTYPE    BUFFER_IBISTYPE_P9=AC_M_P9_IBISTYPE
+ BUFFER_FUNCTION_P10=AC_M_P10_BUFTYPE  BUFFER_IBISTYPE_P10=AC_M_P10_IBISTYPE
+ BUFFER_FUNCTION_P11=AC_M_P11_BUFTYPE  BUFFER_IBISTYPE_P11=AC_M_P11_IBISTYPE
+ r_pin_p0 =AC_M_P0_RVAL  c_pin_p0 =AC_M_P0_CVAL  l_pin_p0 =AC_M_P0_LVAL
+ r_pin_p1 =AC_M_P1_RVAL  c_pin_p1 =AC_M_P1_CVAL  l_pin_p1 =AC_M_P1_LVAL 
+ r_pin_p2 =AC_M_P2_RVAL  c_pin_p2 =AC_M_P2_CVAL  l_pin_p2 =AC_M_P2_LVAL 
+ r_pin_p3 =AC_M_P3_RVAL  c_pin_p3 =AC_M_P3_CVAL  l_pin_p3 =AC_M_P3_LVAL 
+ r_pin_p4 =AC_M_P4_RVAL  c_pin_p4 =AC_M_P4_CVAL  l_pin_p4 =AC_M_P4_LVAL 
+ r_pin_p5 =AC_M_P5_RVAL  c_pin_p5 =AC_M_P5_CVAL  l_pin_p5 =AC_M_P5_LVAL 
+ r_pin_p6 =AC_M_P6_RVAL  c_pin_p6 =AC_M_P6_CVAL  l_pin_p6 =AC_M_P6_LVAL 
+ r_pin_p7 =AC_M_P7_RVAL  c_pin_p7 =AC_M_P7_CVAL  l_pin_p7 =AC_M_P7_LVAL 
+ r_pin_p8 =AC_M_P8_RVAL  c_pin_p8 =AC_M_P8_CVAL  l_pin_p8 =AC_M_P8_LVAL 
+ r_pin_p9 =AC_M_P9_RVAL  c_pin_p9 =AC_M_P9_CVAL  l_pin_p9 =AC_M_P9_LVAL 
+ r_pin_p10=AC_M_P10_RVAL c_pin_p10=AC_M_P10_CVAL l_pin_p10=AC_M_P10_LVAL
+ r_pin_p11=AC_M_P11_RVAL c_pin_p11=AC_M_P11_CVAL l_pin_p11=AC_M_P11_LVAL

.ENDIF
.ENDIF



* ANALYSIS COMMANDS **************************************
.tran 10ps 1000n

* OUTPUT COMMANDS ****************************************
.probe v(*)

* END ****************************************************
.end
**********************************************************
