** (C) 2001-2020 Intel Corporation. All rights reserved.
** Your use of Intel Corporation's design tools, logic functions and other 
** software and tools, and its AMPP partner logic functions, and any output 
** files from any of the foregoing (including device programming or simulation 
** files), and any associated documentation or information are expressly subject 
** to the terms and conditions of the Intel Program License Subscription 
** Agreement, Intel FPGA IP License Agreement, or other applicable 
** license agreement, including, without limitation, that your use is for the 
** sole purpose of programming logic devices manufactured by Intel and sold by 
** Intel or its authorized distributors.  Please refer to the applicable 
** agreement for further details.


******************************************************************************
** This file provides a template for customers to insert a simulation model of
** the 12-line data bus PCB model from the FPGA to the multi-rank data bus 
** model (dq_mr_conn_wrapper). The model can either be an approximation based
** on ideal transmission lines, or a full 24-port extraction from a PCB layout.
**
** This model can use any of the data lanes within an interface, or only the
** estimated worst-case lane can be used as a proxy for the remaining lanes.
** It is assumed that board design guidelines have been followed carefully
** enough to mitigate the effects of crosstalk from neighboring channels on
** this one, but if not, manual modifications need to be made as this wrapper
** only assumes that 12 lines from the same IO12 grouping are to be used.
** Protocols with DQS groups that are larger than an IO12 lane should use
** the lane estimated to have the worst-case SI performance (typically, the
** lane with the greater number of signals).
******************************************************************************
.subckt dq_pcb_wrapper         fpga_0   fpga_1   fpga_2   fpga_3   fpga_4   fpga_5
+                              fpga_6   fpga_7   fpga_8   fpga_9   fpga_10  fpga_11
+                              dqch_0   dqch_1   dqch_2   dqch_3   dqch_4   dqch_5
+                              dqch_6   dqch_7   dqch_8   dqch_9   dqch_10  dqch_11
+                              pcb_gnd
+ USE_PCB_EXTRACTION    = 0
+ PCB_EXTRACTION_FILE   = str("model")

*******************************
* Parameters
*******************************
.param DEFAULT_TL_TD = 2000p
.param DEFAULT_TL_Z0 = 50

*******************************
* Models
*******************************
.model sdq_pcb_model S TSTONEFILE=str(PCB_EXTRACTION_FILE)


.IF (USE_PCB_EXTRACTION == 0)

* Generic 2ns Transmission Line Interface
Tpcb_0  fpga_0   pcb_gnd dqch_0   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_1  fpga_1   pcb_gnd dqch_1   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_2  fpga_2   pcb_gnd dqch_2   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_3  fpga_3   pcb_gnd dqch_3   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_4  fpga_4   pcb_gnd dqch_4   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_5  fpga_5   pcb_gnd dqch_5   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_6  fpga_6   pcb_gnd dqch_6   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_7  fpga_7   pcb_gnd dqch_7   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_8  fpga_8   pcb_gnd dqch_8   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_9  fpga_9   pcb_gnd dqch_9   pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_10 fpga_10  pcb_gnd dqch_10  pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0
Tpcb_11 fpga_11  pcb_gnd dqch_11  pcb_gnd  td=DEFAULT_TL_TD Z0=DEFAULT_TL_Z0

.ELSE
* Use customer extraction
Sac_pcb_model
+ fpga_0   fpga_1  fpga_2  fpga_3  fpga_4  fpga_5  fpga_6  fpga_7  fpga_8  fpga_9  fpga_10 fpga_11
+ dqch_0   dqch_1  dqch_2  dqch_3  dqch_4  dqch_5  dqch_6  dqch_7  dqch_8  dqch_9  dqch_10 dqch_11
+ pcb_gnd
+ mname=sdq_pcb_model

.ENDIF




.ends dq_pcb_wrapper
